package demo.barcharts.category ;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Font ;
import java.awt.GridLayout ;
import java.text.NumberFormat ;

import javax.swing.JPanel ;

import com.ijchart.xychart.annotation.CategoryTextAnnotation ;
import com.ijchart.xychart.axis.value.NumberValueAxis3D ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.TextPosition ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.format.IFormatter ;
import com.ijchart.xychart.marker.ValueMarker ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.range.NumberRange ;
import com.ijchart.xychart.render.category.Bar3DRender ;
import com.ijchart.xychart.ui.Layer ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class BarChart3DDemo4 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.VERTICAL ;

  private final NumberFormat format = NumberFormat.getPercentInstance() ;

  public BarChart3DDemo4() {
    format.setMaximumFractionDigits(0) ;

    setLayout(new GridLayout(1, 1)) ;
    addPanel() ;
  }

  private void addPanel() {
    final IMatrixDataset dataset = createDataset() ;
    final IJChart chart = ChartFactory.createCategoryBar3DChart(
      "Student Grades", "Students", "Grade", dataset, orientation, false, true) ;

    CategoryPlot plot = (CategoryPlot) chart.getPlot() ;
    plot.setDomainGridlineVisible(false) ;

    Bar3DRender render = (Bar3DRender) plot.getRender() ;
    render.setBaseItemLabelVisible(true) ;
    render.setBaseItemLabelPosition(TextPosition.OUTSIDE_TOP) ;
    ValueMarker valuemarker = new ValueMarker(0.69999999999999996D, new Color(200, 200, 255),
                                              new Color(200, 200, 255), new BasicStroke(1.0F), 1.0F) ;
    NumberValueAxis3D valueAxis = (NumberValueAxis3D) plot.getValueAxis() ;
    valueAxis.addValueMarker(Layer.BACKGROUND, valuemarker) ;

    render.setMaximumBarWidth(10D) ;
    CategoryTextAnnotation anno = new CategoryTextAnnotation("Minimum grade to pass", "Robert",
                                                             0.70999999999999996D) ;
    anno.setTextFont(new Font("SansSerif", 0, 12)) ;
    render.addAnnotation(anno) ;

    IMatrixDataset ds = render.getDataset() ;
    int rows = ds.getRowCount() ;
    render.setAutoSelectFillPaint(false) ;
    for (int i = 0 ; i < rows ; i++) {
      NumberChartData data = (NumberChartData) ds.getData(i, 0) ;
      if (data.getValue() >= 0.69999999999999996D) {
        render.setItemFillPaint(i, 0, Color.green) ;
      }else{
        render.setItemFillPaint(i, 0, Color.red) ;
      }
    }

    valueAxis.setUserDefineDatasetRange(new NumberRange(0, 1)) ;
    valueAxis.setTickValueFormatter(new IFormatter() {
      public String format(Object obj, int index) {
        String str = obj.toString() ;
        String tick = format.format(Double.parseDouble(str)) ;
        return tick ;
      }

    }) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    String[] keys = {"Robert", "Mary", "John", "Ellen", "Jack", "David", "Mark", "Andy"} ;
    for (int i = 0 ; i < keys.length ; i++) {
      dataset.addRowKey(keys[i]) ;
    }
    dataset.addColKey("Series 1") ;

    double data[] = {0.77000000000000002D, 0.93000000000000005D, 0.58999999999999997D, 0.75D, 0.63D,
                    0.94999999999999996D, 0.70999999999999996D, 0.65000000000000002D
    } ;
    dataset.addRowData(data.length) ;

    for (int i = 0 ; i < data.length ; i++) {
      dataset.addData(i, new NumberChartData(data[i])) ;
    }

    return dataset ;
  }

}
