package demo.barcharts.category ;

import java.awt.Color ;
import java.awt.GradientPaint ;
import java.awt.GridLayout ;
import java.util.List ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.NumberValueAxis ;
import com.ijchart.xychart.chartinfo.ChartShapeCollection ;
import com.ijchart.xychart.chartinfo.LegendShapeCollection ;
import com.ijchart.xychart.chartinfo.draw.ChangeChartShapeAdapter ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.GradientPaintTransformType ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.range.NumberRange ;
import com.ijchart.xychart.render.category.Bar2DRender ;
import com.ijchart.xychart.shape.IChartShape ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class BarChartDemo2 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.HORIZONTAL ;

  public BarChartDemo2() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel() ;
  }

  private void addPanel() {
    final IMatrixDataset dataset = createDataset() ;
    final IJChart chart = ChartFactory.createCategoryBar2DChart(
      "Bar Chart Demo 2", "Category", "Score (%)", dataset, orientation, true, true) ;

    CategoryPlot plot = (CategoryPlot) chart.getPlot() ;
    NumberValueAxis numberaxis = (NumberValueAxis) plot.getValueAxis() ;
    numberaxis.setUserDefineDatasetRange(new NumberRange(0, 100)) ;

    Bar2DRender render = (Bar2DRender) plot.getRender() ;
    render.setBaseItemDrawBorder(false) ;
    GradientPaint gradientpaint = new GradientPaint(0.0F, 0.0F, new Color(0, 0, 128), 0.0F, 0.0F,
                                                    Color.blue) ;
    GradientPaint gradientpaint1 = new GradientPaint(0.0F, 0.0F, new Color(0, 128, 0), 0.0F, 0.0F,
                                                     Color.green) ;
    GradientPaint gradientpaint2 = new GradientPaint(0.0F, 0.0F, new Color(128, 0, 0), 0.0F, 0.0F,
                                                     Color.red) ;
    render.setAutoSelectFillPaint(false) ;
    render.setBaseItemFillPaintTransformer(GradientPaintTransformType.HORIZONTAL) ;
    render.setColumnItemFillPaint(0, gradientpaint) ;
    render.setColumnItemFillPaint(1, gradientpaint1) ;
    render.setColumnItemFillPaint(2, gradientpaint2) ;

    chart.setChangeChartShape(new ChangeChartShapeAdapter() {
      public void beforeDraw(ChartShapeCollection chartShapes) {
        LegendShapeCollection legendShapes = chart.getChartShapes().getLegendShapes() ;
        List shapes = legendShapes.getLegendShapes() ;

        IChartShape shape = (IChartShape) shapes.get(0) ;
        shape.setTooltipText("Tooltip: " + dataset.getColKey(0)) ;
        shape = (IChartShape) shapes.get(1) ;
        shape.setTooltipText("Tooltip: " + dataset.getColKey(1)) ;
        shape = (IChartShape) shapes.get(2) ;
        shape.setTooltipText("Tooltip: " + dataset.getColKey(2)) ;
      }

    }) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Factor 1") ;
    dataset.addRowKey("Factor 2") ;
    dataset.addRowKey("Factor 3") ;
    dataset.addRowKey("Factor 4") ;
    dataset.addRowKey("Factor 5") ;
    dataset.addRowKey("Factor 6") ;
    dataset.addRowKey("Factor 7") ;
    dataset.addRowKey("Factor 8") ;

    dataset.addColKey("Series 1") ;
    dataset.addColKey("Series 2") ;
    dataset.addColKey("Series 3") ;

    double data[][] = { {1.0D, 43D, 35D, 58D, 54D, 77D, 71D, 89D}, {54D, 75D, 63D, 83D, 43D, 46D,
                      27D, 13D}, {41D, 33D, 22D, 34D, 62D, 32D, 42D, 34D}
    } ;
    dataset.addRowData(8) ;

    for (int i = 0 ; i < 8 ; i++) {
      for (int j = 0 ; j < data.length ; j++) {
        dataset.addData(i, new NumberChartData(data[j][i])) ;
      }
    }

    return dataset ;
  }

}
