package demo.barcharts.xy ;

import java.awt.Font ;
import java.awt.GridLayout ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.StackedXYBarValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.TextPosition ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.label.DefaultLabelGenerator ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.render.xy.XYStackedBarRender ;
import com.ijchart.xychart.tickunit.LongTickUnit ;
import com.ijchart.xychart.title.TextTitle ;
import com.ijchart.xychart.tooltip.DefaultTooltipGenerator ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class StackedXYBarChartDemo2 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.VERTICAL ;

  public StackedXYBarChartDemo2() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel() ;
  }

  private void addPanel() {
    final IMatrixDataset dataset = createDataset() ;
    final IJChart chart = ChartFactory.createStackedXYBarChart(
      "Holes-In-One / Double Eagles", "Date", "Count", dataset, orientation, true,
      true, false, false) ;

    XYPlot xyplot = (XYPlot) chart.getPlot() ;
    StackedXYBarValueAxis domainAxis = (StackedXYBarValueAxis) xyplot.getDomainAxis() ;
    domainAxis.setTickUnit(new LongTickUnit()) ;
    domainAxis.setLowerMargin(0.01D) ;
    domainAxis.setUpperMargin(0.01D) ;

    StackedXYBarValueAxis valueAxis = (StackedXYBarValueAxis) xyplot.getValueAxis() ;
    valueAxis.setTickUnit(new LongTickUnit()) ;

    XYStackedBarRender render = (XYStackedBarRender) xyplot.getRender() ;
    render.setMargin(0.14999999999999999D) ;
    render.setBaseItemLabelVisible(true) ;
    render.setRowItemLabelPosition(0, TextPosition.INSIDE) ;
    render.setRowItemLabelPosition(1, TextPosition.OUTSIDE_TOP) ;
    render.setItemLabelPosition(0, 0, TextPosition.OUTSIDE_TOP) ;
    render.setLabelGenerator(new DefaultLabelGenerator() {
      public String generateLabel(IMatrixDataset dataset, int row, int column) {
        XYNumberChartData data = (XYNumberChartData) dataset.getData(row, column) ;
        return String.valueOf((int) data.getY()) ;
      }
    }) ;
    render.setToolTipGenerator(new DefaultTooltipGenerator() {
      public String generateToolTip(IMatrixDataset dataset, int row, int column) {
        Object rowKey = dataset.getRowKey(row) ;
        XYNumberChartData data = (XYNumberChartData) dataset.getData(row, column) ;
        return rowKey + " : " + (int) data.getX() + " = " + (int) data.getY() ;
      }
    }) ;

    chart.setShowLegend(false) ;
    chart.addSubtitle(new TextTitle("PGA Tour, 1983 to 2003")) ;
    TextTitle texttitle = new TextTitle(
      "http://www.golfdigest.com/majors/masters/index.ssf?/majors/masters/gw20040402albatross.html",
      new Font("Dialog", 0, 12)) ;
    chart.addSubtitle(texttitle) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    chartpanel.setTextAntiAlias(true) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Albatrosses") ;
    dataset.addRowKey("Aces") ;

    double[][] data1 = { {1983, 0.0D}, {1984, 2D}, {1985, 1.0D}, {1986, 1.0D}, {1987, 2D}, {1988,
                       2D}, {1989, 1.0D}, {1990, 5D}, {1991, 5D}, {1992, 2D}, {1993, 4D}, {1994, 3D},
                       {1995, 2D}, {1996, 1.0D}, {1997, 2D}, {1998, 1.0D}, {1999, 4D}, {2000, 6D},
                       {2001, 5D}, {2002, 4D}, {2003, 2D}
    } ;
    double[][] data2 = { {1983, 21D}, {1984, 24D}, {1985, 32D}, {1986, 20D}, {1987, 28D}, {1988,
                       17D}, {1989, 31D}, {1990, 32D}, {1991, 29D}, {1992, 31D}, {1993, 25D}, {1994,
                       44D}, {1995, 35D}, {1996, 40D}, {1997, 32D}, {1998, 32D}, {1999, 30D}, {2000,
                       29D}, {2001, 28D}, {2002, 39D}, {2003, 32D}
    } ;
    dataset.addRowData(2) ;

    for (int i = 0 ; i < data1.length ; i++) {
      dataset.addData(0, new XYNumberChartData(data1[i][0], data1[i][1])) ;
      dataset.addData(1, new XYNumberChartData(data2[i][0], data2[i][1])) ;
    }

    return dataset ;
  }

}
