package demo.miscellaneous ;

import java.awt.BorderLayout ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.value.NumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.AxisLocation ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.format.AbstractChartDataFormatterAdapter ;
import com.ijchart.xychart.format.IChartDataFormatter ;
import com.ijchart.xychart.format.IFormatter ;
import com.ijchart.xychart.format.MillisecondsToDateFormatter ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.render.category.LineAndShapeRender ;
import com.ijchart.xychart.tickunit.DateTickUnit ;
import com.ijchart.xychart.tooltip.DefaultTooltipGenerator ;
import com.ijchart.xychart.util.DateUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class EventFrequencyDemo1 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.HORIZONTAL ;

  private static final long HALF_DAY_IN_MILLS = 12 * 60 * 60 * 1000L ;

  public EventFrequencyDemo1() {
    setLayout(new BorderLayout()) ;
    addPanel1() ;
  }

  private void addPanel1() {
    IMatrixDataset dataset = createDataset() ;
    final IJChart chart = ChartFactory.createCategoryLineAndShapeChart("Event Frequency Demo",
                                                                       "Category", "Value", dataset,
                                                                       orientation, true, true) ;
    CategoryPlot plot = (CategoryPlot) chart.getPlot() ;
    plot.setValueAxisLocation(AxisLocation.TOP_OR_LEFT) ;
    plot.setDomainGridlineVisible(false) ;

    CategoryAxis categoryAxis = plot.getCategoryAxis() ;
    categoryAxis.setMaximumCategoryLabelWidthRatio(10F) ;

    NumberValueAxis valueAxis = (NumberValueAxis) plot.getValueAxis() ;
    valueAxis.setTickUnit(new DateTickUnit(null)) ;
    valueAxis.setLowerMargin(0.03F) ;

    LineAndShapeRender render = (LineAndShapeRender) plot.getRender() ;
    render.setBaseItemLineVisible(false) ;

    IChartDataFormatter formatter = new AbstractChartDataFormatterAdapter() {
      private IFormatter format1 = new MillisecondsToDateFormatter("yyyy-MMM-dd") ;

      public String format(IMatrixDataset dataset, int row, int column) {
        NumberChartData data = (NumberChartData) dataset.getData(row, column) ;
        return "(" + format1.format(data.getData(), row) + ")" ;
      }
    } ;
    render.setToolTipGenerator(new DefaultTooltipGenerator(formatter)) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Series 1") ;
    dataset.addRowKey("Series 2") ;
    dataset.addRowKey("Series 3") ;

    dataset.addColKey("Requirement 1") ;
    dataset.addColKey("Requirement 2") ;
    dataset.addColKey("Requirement 3") ;
    dataset.addColKey("Requirement 4") ;

    dataset.addRowData(3) ;
    dataset.addData(0, new NumberChartData(dayToMills(12, 6, 2002))) ;
    dataset.addData(0, new NumberChartData(dayToMills(12, 6, 2002))) ;
    dataset.addData(0, new NumberChartData(dayToMills(14, 6, 2002))) ;
    dataset.addData(0, new NumberChartData(dayToMills(22, 8, 2002))) ;

    dataset.addData(1, new NumberChartData(dayToMills(15, 6, 2002))) ;
    dataset.addData(1, null) ;
    dataset.addData(1, new NumberChartData(dayToMills(10, 7, 2002))) ;

    dataset.addData(2, null) ;
    dataset.addData(2, new NumberChartData(dayToMills(20, 7, 2002))) ;

    return dataset ;
  }

  private static long dayToMills(int day, int month, int year) {
    return DateUtil.dateToTimeInMillis(year, month, day) + HALF_DAY_IN_MILLS ;
  }

}
