package demo.miscellaneous.markers ;

import java.awt.BasicStroke ;
import java.awt.BorderLayout ;
import java.awt.Color ;
import java.awt.Font ;
import java.text.SimpleDateFormat ;
import java.util.Date ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.DateTickUnitType ;
import com.ijchart.xychart.constant.HorizontalAlignment ;
import com.ijchart.xychart.constant.TextPosition ;
import com.ijchart.xychart.constant.VerticalAlignment ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.format.AbstractChartDataFormatterAdapter ;
import com.ijchart.xychart.format.IChartDataFormatter ;
import com.ijchart.xychart.format.IFormatter ;
import com.ijchart.xychart.format.MillisecondsToDateFormatter ;
import com.ijchart.xychart.marker.IntervalValueMarker ;
import com.ijchart.xychart.marker.ValueMarker ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.range.NumberRange ;
import com.ijchart.xychart.render.xy.XYLineAndShapeRender ;
import com.ijchart.xychart.tickunit.DateTickUnit ;
import com.ijchart.xychart.tooltip.DefaultTooltipGenerator ;
import com.ijchart.xychart.ui.Layer ;
import com.ijchart.xychart.util.DateUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class MarkerDemo2 extends JPanel {
  private ChartOrientation orientation = ChartOrientation.VERTICAL ;

  public MarkerDemo2() {
    setLayout(new BorderLayout()) ;
    addPanel1() ;
  }

  private void addPanel1() {
    DefaultMatrixDataset dataset = createDataset() ;
    IJChart chart = ChartFactory.createXYLineAndShapeChart("Marker Demo 2", "X", "Temperature",
                                                           dataset, orientation, false, true, true, false) ;

    XYPlot plot = (XYPlot) chart.getPlot() ;
    plot.setValueGridlinePaint(Color.lightGray) ;
    plot.setDomainGridlinePaint(Color.lightGray) ;
    plot.setDomainGridlineStroke(new BasicStroke(1.0F)) ;
    plot.setValueGridlineStroke(new BasicStroke(1.0F)) ;

    XYNumberValueAxis xAxis = (XYNumberValueAxis) plot.getDomainAxis() ;
    SimpleDateFormat dateFormat = new SimpleDateFormat("dd-MMM") ;
    xAxis.setAxisLabel(dateFormat.format(new Date(2005, 5, 30))) ;
    DateTickUnit unit = new DateTickUnit(DateTickUnitType.DATE_UNIT_HOUR, new SimpleDateFormat("HH")) ;
    xAxis.setTickUnit(unit) ;

    XYNumberValueAxis yAxis = (XYNumberValueAxis) plot.getValueAxis() ;
    yAxis.setUserDefineDatasetRange(new NumberRange(0.0D, 100D)) ;

    XYLineAndShapeRender xyitemrenderer = (XYLineAndShapeRender) plot.getRender() ;
    xyitemrenderer.setAutoSelectBorderPaint(false) ;
    xyitemrenderer.setRowItemBorderPaint(0, Color.green) ;
    xyitemrenderer.setRowItemBorderStroke(0, new BasicStroke(2.0F)) ;
    xyitemrenderer.setBaseItemShapeVisible(false) ;
    xyitemrenderer.setBackgroundPaint(new Color(240, 240, 240)) ;

    ValueMarker valuemarker = new ValueMarker(80D) ;
    valuemarker.setBorderPaint(Color.red) ;
    valuemarker.setBorderStroke(new BasicStroke(2.0F)) ;
    valuemarker.setText("Temperature Threshold") ;
    valuemarker.setTextFont(new Font("SansSerif", 0, 11)) ;
    valuemarker.setTextPaint(Color.red) ;
    valuemarker.setTextHorizontalAlign(HorizontalAlignment.LEFT) ;
    yAxis.addValueMarker(Layer.FOREGROUND, valuemarker) ;

    long startTime = DateUtil.dateToTimeInMillis(2005, 6, 30, 18) ;
    long endTime = DateUtil.dateToTimeInMillis(2005, 6, 30, 20) ;
    IntervalValueMarker intervalMarker = new IntervalValueMarker(startTime, endTime) ;
    intervalMarker.setFillPaint(new Color(150, 150, 255)) ;
    intervalMarker.setText("Automatic Cooling") ;
    intervalMarker.setTextFont(new Font("SansSerif", 0, 11)) ;
    intervalMarker.setTextPaint(Color.blue) ;
    intervalMarker.setTextPosition(TextPosition.OUTSIDE_LEFT) ;
    intervalMarker.setTextVerticalAlign(VerticalAlignment.TOP) ;
    xAxis.addIntervalValueMarker(Layer.BACKGROUND, intervalMarker) ;

    ValueMarker valueMarker1 = new ValueMarker(startTime) ;
    valueMarker1.setBorderPaint(Color.blue) ;
    valueMarker1.setBorderStroke(new BasicStroke(2.0F)) ;
    xAxis.addValueMarker(Layer.BACKGROUND, valueMarker1) ;

    ValueMarker valueMarker2 = new ValueMarker(endTime) ;
    valueMarker2.setBorderPaint(Color.red) ;
    valueMarker2.setBorderStroke(new BasicStroke(2.0F)) ;
    xAxis.addValueMarker(Layer.BACKGROUND, valueMarker2) ;

    XYLineAndShapeRender render = (XYLineAndShapeRender) plot.getRender() ;
    IChartDataFormatter formatter = new AbstractChartDataFormatterAdapter() {
      private IFormatter format = new MillisecondsToDateFormatter("yyyy-MM-dd HH:mm") ;

      public String format(IMatrixDataset dataset, int row, int column) {
        XYNumberChartData data = (XYNumberChartData) dataset.getData(row, column) ;
        return "(" + format.format(data.getXData(), row) + "," + data.getY() + ")" ;
      }
    } ;
    DefaultTooltipGenerator tooltip = new DefaultTooltipGenerator(formatter) ;
    render.setToolTipGenerator(tooltip) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static DefaultMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("Temperature") ;
    dataset.addRowData(1) ;

    dataset.addData(0, createData(0, 30, 6, 2005, 45.299999999999997D)) ;
    dataset.addData(0, createData(1, 30, 6, 2005, 48.899999999999999D)) ;
    dataset.addData(0, createData(2, 30, 6, 2005, 52.100000000000001D)) ;
    dataset.addData(0, createData(3, 30, 6, 2005, 44.799999999999997D)) ;
    dataset.addData(0, createData(4, 30, 6, 2005, 49.899999999999999D)) ;
    dataset.addData(0, createData(5, 30, 6, 2005, 55.5D)) ;
    dataset.addData(0, createData(6, 30, 6, 2005, 58.200000000000003D)) ;
    dataset.addData(0, createData(7, 30, 6, 2005, 58.100000000000001D)) ;
    dataset.addData(0, createData(8, 30, 6, 2005, 63.700000000000003D)) ;
    dataset.addData(0, createData(9, 30, 6, 2005, 66.299999999999997D)) ;
    dataset.addData(0, createData(10, 30, 6, 2005, 69.799999999999997D)) ;
    dataset.addData(0, createData(11, 30, 6, 2005, 70.099999999999994D)) ;
    dataset.addData(0, createData(12, 30, 6, 2005, 72.400000000000006D)) ;
    dataset.addData(0, createData(13, 30, 6, 2005, 69.700000000000003D)) ;
    dataset.addData(0, createData(14, 30, 6, 2005, 68.599999999999994D)) ;
    dataset.addData(0, createData(15, 30, 6, 2005, 70.900000000000006D)) ;
    dataset.addData(0, createData(16, 30, 6, 2005, 73.400000000000006D)) ;
    dataset.addData(0, createData(17, 30, 6, 2005, 77.5D)) ;
    dataset.addData(0, createData(18, 30, 6, 2005, 82.900000000000006D)) ;
    dataset.addData(0, createData(19, 30, 6, 2005, 62.100000000000001D)) ;
    dataset.addData(0, createData(20, 30, 6, 2005, 37.299999999999997D)) ;
    dataset.addData(0, createData(21, 30, 6, 2005, 40.700000000000003D)) ;
    dataset.addData(0, createData(22, 30, 6, 2005, 44.200000000000003D)) ;
    dataset.addData(0, createData(23, 30, 6, 2005, 49.799999999999997D)) ;

    return dataset ;
  }

  private static XYNumberChartData createData(int hour, int day, int month, int year, double value) {
    long time = DateUtil.dateToTimeInMillis(year, month, day, hour) ;
    return new XYNumberChartData(time, value) ;
  }
}
