package demo.others.shapes ;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Dimension ;
import java.awt.Graphics ;
import java.awt.Graphics2D ;
import java.awt.GridLayout ;
import java.awt.RenderingHints ;
import java.awt.Shape ;
import java.awt.event.MouseEvent ;
import java.awt.event.MouseMotionAdapter ;
import java.awt.geom.Ellipse2D ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.util.HashMap ;
import java.util.Map ;

import javax.swing.BorderFactory ;
import javax.swing.JLabel ;
import javax.swing.JPanel ;

import com.ijchart.xychart.shape.MinMaxChartShape ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class MinMaxChartShapePanel extends JPanel {
  private Map hashMap = new HashMap() ;

  private JLabel label ;

  public MinMaxChartShapePanel() {
    init() ;
  }

  private void init() {
    hashMap.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON) ;

    setLayout(new GridLayout(4, 2)) ;
    addPanel1() ;
    addPanel2() ;
    addPanel3() ;
    addPanel4() ;

    label = new JLabel("begin") ;
    add(label) ;
  }

  private void addPanel1() {
    final double size = 16.0 ;
    final double delta = 8.0 ;

    final Shape circle = new Ellipse2D.Double( -delta, -delta, size, size) ;
    final Shape rect = new Rectangle2D.Double( -delta, -delta, size, size) ;

    Point2D minPoint = new Point2D.Double(75, 65) ;
    Point2D maxPoint = new Point2D.Double(75, 100) ;

    final MinMaxChartShape shape = new MinMaxChartShape(maxPoint, minPoint) ;
//    shape.setRotate(Math.PI / 4) ;

    JPanel panel = new JPanel() {
      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;
        g.drawRect(0, 0, 120, 120) ;

        shape.setDrawGroupLine(true) ;
        shape.setGroupLinePaint(Color.red) ;
        shape.setGroupLineStroke(new BasicStroke(2)) ;

        shape.setMinShapePaint(Color.BLUE) ;
        shape.setMinShapeStroke(new BasicStroke(2)) ;

        shape.setMaxShapePaint(Color.CYAN) ;
        shape.setMaxShapeStroke(new BasicStroke(2)) ;

        shape.setMinShape(circle) ;
        shape.setMaxShape(rect) ;

        shape.setSelected(true) ;
        shape.draw((Graphics2D) g) ;
      }

    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + shape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(150, 150)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

  private void addPanel2() {
    final double size = 16.0 ;
    final double delta = 8.0 ;

    final Shape circle = new Ellipse2D.Double( -delta, -delta, size, size) ;
    final Shape rect = new Rectangle2D.Double( -delta, -delta, size, size) ;

    Point2D minPoint = new Point2D.Double(35, 15) ;
    Point2D maxPoint = new Point2D.Double(25, 50) ;

    final MinMaxChartShape shape = new MinMaxChartShape(minPoint, maxPoint) ;

    JPanel panel = new JPanel() {
      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;
        g.drawRect(0, 0, 100, 100) ;

        shape.setDrawGroupLine(true) ;
        shape.setGroupLinePaint(Color.red) ;
        shape.setGroupLineStroke(new BasicStroke(2)) ;

        shape.setMinShapePaint(Color.BLUE) ;
        shape.setMinShapeStroke(new BasicStroke(2)) ;

        shape.setMaxShapePaint(Color.CYAN) ;
        shape.setMaxShapeStroke(new BasicStroke(2)) ;

        shape.setMinShape(circle) ;
        shape.setMaxShape(rect) ;

        shape.setSelected(true) ;
        shape.draw((Graphics2D) g) ;
      }

    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + shape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(150, 150)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

  private void addPanel3() {
    final double size = 16.0 ;
    final double delta = 8.0 ;

    final Shape circle = new Ellipse2D.Double( -delta, -delta, size, size) ;
    final Shape rect = new Rectangle2D.Double( -delta, -delta, size, size) ;

    Point2D minPoint = new Point2D.Double(35, 15) ;
    Point2D maxPoint = new Point2D.Double(25, 50) ;

    final MinMaxChartShape shape = new MinMaxChartShape(minPoint, maxPoint) ;
    shape.setRotate(Math.PI / 8) ;

    JPanel panel = new JPanel() {
      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;
        g.drawRect(0, 0, 100, 100) ;

        shape.setDrawGroupLine(true) ;
        shape.setGroupLinePaint(Color.red) ;
        shape.setGroupLineStroke(new BasicStroke(2)) ;

        shape.setMinShapePaint(Color.BLUE) ;
        shape.setMinShapeStroke(new BasicStroke(2)) ;

        shape.setMaxShapePaint(Color.CYAN) ;
        shape.setMaxShapeStroke(new BasicStroke(2)) ;

        shape.setMinShape(circle) ;
        shape.setMaxShape(rect) ;

        shape.setSelected(true) ;
        shape.draw((Graphics2D) g) ;
      }

    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + shape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(150, 150)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

  private void addPanel4() {
    final double size = 16.0 ;
    final double delta = 8.0 ;

    final Shape circle = new Ellipse2D.Double( -delta, -delta, size, size) ;
    final Shape rect = new Rectangle2D.Double( -delta, -delta, size, size) ;

    Point2D minPoint = new Point2D.Double(35, 15) ;
    Point2D maxPoint = new Point2D.Double(25, 50) ;

    final MinMaxChartShape shape = new MinMaxChartShape(minPoint, maxPoint) ;
    shape.setRotate( -Math.PI / 2) ;

    JPanel panel = new JPanel() {
      public void paint(Graphics g) {
        Graphics2D g2 = (Graphics2D) g ;
        g2.addRenderingHints(hashMap) ;

        setBackground(Color.white) ;
        super.paintComponent(g) ;
        g.drawRect(0, 0, 100, 100) ;

        shape.setDrawGroupLine(true) ;
        shape.setGroupLinePaint(Color.red) ;
        shape.setGroupLineStroke(new BasicStroke(2)) ;

        shape.setMinShapePaint(Color.BLUE) ;
        shape.setMinShapeStroke(new BasicStroke(2)) ;

        shape.setMaxShapePaint(Color.CYAN) ;
        shape.setMaxShapeStroke(new BasicStroke(2)) ;

        shape.setMinShape(circle) ;
        shape.setMaxShape(rect) ;

        shape.setSelected(true) ;
        shape.draw((Graphics2D) g) ;
      }

    } ;

    panel.addMouseMotionListener(new MouseMotionAdapter() {
      public void mouseMoved(MouseEvent e) {
        label.setText("(" + e.getX() + "," + e.getY() + ")" + shape.contains(e.getX(), e.getY())) ;
      }
    }) ;

    panel.setPreferredSize(new Dimension(150, 150)) ;
    panel.setBorder(BorderFactory.createLineBorder(Color.red)) ;
    add(panel) ;
  }

}
