package demo.timeseriescharts;

import java.awt.GridLayout ;
import java.text.SimpleDateFormat ;
import java.util.Calendar ;

import javax.swing.JPanel ;

import com.ijchart.xychart.axis.value.XYNumberValueAxis ;
import com.ijchart.xychart.client.ChartFactory ;
import com.ijchart.xychart.client.ChartPanel ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.DefaultMatrixDataset ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.format.IFormatter ;
import com.ijchart.xychart.plot.XYPlot ;
import com.ijchart.xychart.util.DateUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class PeriodAxisDemo1 extends JPanel {

  public PeriodAxisDemo1() {
    setLayout(new GridLayout(1, 1)) ;
    addPanel1() ;
  }

  private void addPanel1() {
    IMatrixDataset dataset = createDataset() ;
    IJChart chart = ChartFactory.createXYLineAndShapeChart("Legal & General Unit Trust Prices", "Date",
                                                   "Price Per Unit", dataset,
                                                   ChartOrientation.VERTICAL, true, true,
                                                   true, false) ;
    XYPlot xyplot = (XYPlot) chart.getPlot() ;
    xyplot.setDomainCrosshairVisible(true) ;
    xyplot.setValueCrosshairVisible(true) ;

    XYNumberValueAxis yAxis = (XYNumberValueAxis) xyplot.getValueAxis() ;
    yAxis.setAxisRangeIncludeZero(false) ;

    XYNumberValueAxis xAxis = (XYNumberValueAxis) xyplot.getDomainAxis() ;
    xAxis.setTickValueFormatter(new IFormatter() {
      private Calendar cal = Calendar.getInstance() ;

      private SimpleDateFormat format = new SimpleDateFormat("yy-MMM") ;

      public String format(Object obj, int index) {
        if (obj != null) {
          String str = obj.toString() ;
          cal.setTimeInMillis(Long.parseLong(str)) ;

          return format.format(cal.getTime()) ;
        }

        return null ;
      }
    }) ;

    ChartPanel chartpanel = new ChartPanel(chart) ;
    add(chartpanel) ;
  }

  private static IMatrixDataset createDataset() {
    DefaultMatrixDataset dataset = new DefaultMatrixDataset() ;
    dataset.addRowKey("L&G European Index Trust") ;
    dataset.addRowKey("L&G UK Index Trust") ;
    dataset.addRowData(2) ;

    dataset.addData(0, createData(2001, 2, 181.80000000000001D)) ;
    dataset.addData(0, createData(2001, 3, 167.30000000000001D)) ;
    dataset.addData(0, createData(2001, 4, 153.80000000000001D)) ;
    dataset.addData(0, createData(2001, 5, 167.59999999999999D)) ;
    dataset.addData(0, createData(2001, 6, 158.80000000000001D)) ;
    dataset.addData(0, createData(2001, 7, 148.30000000000001D)) ;
    dataset.addData(0, createData(2001, 8, 153.90000000000001D)) ;
    dataset.addData(0, createData(2001, 9, 142.69999999999999D)) ;
    dataset.addData(0, createData(2001, 10, 123.2D)) ;
    dataset.addData(0, createData(2001, 11, 131.80000000000001D)) ;
    dataset.addData(0, createData(2001, 12, 139.59999999999999D)) ;
    dataset.addData(0, createData(2002, 1, 142.90000000000001D)) ;
    dataset.addData(0, createData(2002, 2, 138.69999999999999D)) ;
    dataset.addData(0, createData(2002, 3, 137.30000000000001D)) ;
    dataset.addData(0, createData(2002, 4, 143.90000000000001D)) ;
    dataset.addData(0, createData(2002, 5, 139.80000000000001D)) ;
    dataset.addData(0, createData(2002, 6, 137D)) ;
    dataset.addData(0, createData(2002, 7, 132.80000000000001D)) ;

    dataset.addData(1, createData(2001, 2, 129.59999999999999D)) ;
    dataset.addData(1, createData(2001, 3, 123.2D)) ;
    dataset.addData(1, createData(2001, 4, 117.2D)) ;
    dataset.addData(1, createData(2001, 5, 124.09999999999999D)) ;
    dataset.addData(1, createData(2001, 6, 122.59999999999999D)) ;
    dataset.addData(1, createData(2001, 7, 119.2D)) ;
    dataset.addData(1, createData(2001, 8, 116.5D)) ;
    dataset.addData(1, createData(2001, 9, 112.7D)) ;
    dataset.addData(1, createData(2001, 10, 101.5D)) ;
    dataset.addData(1, createData(2001, 11, 106.09999999999999D)) ;
    dataset.addData(1, createData(2001, 12, 110.3D)) ;
    dataset.addData(1, createData(2002, 1, 111.7D)) ;
    dataset.addData(1, createData(2002, 2, 111D)) ;
    dataset.addData(1, createData(2002, 3, 109.59999999999999D)) ;
    dataset.addData(1, createData(2002, 4, 113.2D)) ;
    dataset.addData(1, createData(2002, 5, 111.59999999999999D)) ;
    dataset.addData(1, createData(2002, 6, 108.8D)) ;
    dataset.addData(1, createData(2002, 7, 101.59999999999999D)) ;

    return dataset ;
  }

  private static XYNumberChartData createData(int year, int month, double value) {
    return new XYNumberChartData(DateUtil.dateToTimeInMillis(year, month), value) ;
  }

}
