package com.ijchart.xychart.annotation ;

import java.awt.Image ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.AnnotationShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.shape.RectangleImageChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 * <zh>{@link com.ijchart.xychart.plot.XYPlot}ͼƬע.</zh>
 * <en>A image annotation that can be placed on a {@link com.ijchart.xychart.plot.XYPlot}.</en>
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYImageAnnotation extends AbstractAnnotation implements IXYAnnotation, Cloneable,
  Serializable {

  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 4699783784755025592L ;

  /**
   * <zh>ͼƬx,xݿռ,Ǿ.</zh>
   * <en>The x coordinate (in data space).</en>
   */
  private double x ;

  /**
   * <zh>ͼƬy,yݿռ,Ǿ.</zh>
   * <en>The y coordinate (in data space).</en>
   */
  private double y ;

  /**
   * <zh>ͼƬĿ.</zh>
   * <en>The image width.</en>
   */
  private int imageWidth ;

  /**
   * <zh>ͼƬĸ߶.</zh>
   * <en>The image height.</en>
   */
  private int imageHeight ;

  /**
   * <zh>ͼƬ.</zh>
   * <en>The image.</en>
   */
  private transient Image image ;

  /**
   * <zh>xyȷͼƬλ.</zh>
   * <en>Create a new annotation to be displayed at the given location.</en>
   *
   * @param x double <zh>x.</zh>
   * <en>The x coordinate.</en>
   * @param y double <zh>y.</zh>
   * <en>The y coordinate.</en>
   * @param image Image <zh>ͼƬ.</zh>
   * <en>The image.</en>
   */
  public XYImageAnnotation(double x, double y, Image image) {
    this(x, y, image, -1, -1) ;
  }

  /**
   * <zh>xyȷͼƬλ.</zh>
   * <en>Create a new annotation to be displayed at the given location.</en>
   *
   * @param x double <zh>x.</zh>
   * <en>The x coordinate.</en>
   * @param y double <zh>y.</zh>
   * <en>The y coordinate.</en>
   * @param image Image <zh>ͼƬ.</zh>
   * <en>The image.</en>
   * @param imageWidth int <zh>ͼƬĿ.</zh>
   * <en>The image width.</en>
   * @param imageHeight int <zh>ͼƬĸ߶.</zh>
   * <en>The image height.</en>
   */
  public XYImageAnnotation(double x, double y, Image image, int imageWidth, int imageHeight) {
    if (image == null) {
      throw new IllegalArgumentException("Null 'image' argument.") ;
    }

    this.x = x ;
    this.y = y ;
    this.image = image ;
    this.imageWidth = imageWidth ;
    this.imageHeight = imageHeight ;
  }

  /**
   * <zh>ͼƬx.</zh>
   * <en>Returns the x coordinate for image.</en>
   *
   * @return double <zh>x.</zh>
   * <en>The x coordinate.</en>
   */
  public double getX() {
    return this.x ;
  }

  /**
   * <zh>ͼƬx,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the x coordinate for image and sends
   * a {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param x double <zh>x.</zh>
   * <en>The x coordinate.</en>
   */
  public void setX(double x) {
    if (this.x != x) {
      this.x = x ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ͼƬy.</zh>
   * <en>Returns the y coordinate for image.</en>
   *
   * @return double <zh>y.</zh>
   * <en>The y coordinate.</en>
   */
  public double getY() {
    return this.y ;
  }

  /**
   * <zh>ͼƬy,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the y coordinate for image and sends
   * a {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param y double <zh>y.</zh>
   * <en>The y coordinate.</en>
   */
  public void setY(double y) {
    if (this.y != y) {
      this.y = y ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ͼζ.</zh>
   * <en>Returns the shape.</en>
   *
   * @return Shape <zh>ͼ (Ϊnull).</zh>
   * <en>The shape (never null).</en>
   */
  public Image getImage() {
    return this.image ;
  }

  /**
   * <zh>ͼζ,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the shape and sends
   * a {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param image Image <zh>ͼƬ (Ϊnull).</zh>
   * <en>The image (null not permitted).</en>
   */
  public void setImage(Image image) {
    if (image == null) {
      throw new IllegalArgumentException("Null 'image' argument.") ;
    }

    this.image = image ;
    fireAnnotationChanged() ;
  }

  /**
   * <zh>ͼƬĿ.</zh>
   * <en>Returns the image width.</en>
   *
   * @return int <zh>.</zh>
   * <en>The width.</en>
   */
  public int getImageWidth() {
    return this.imageWidth ;
  }

  /**
   * <zh>ͼƬĿ,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the image width and sends
   * a {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param width int <zh>.</zh>
   * <en>The width.</en>
   */
  public void setImageWidth(int width) {
    if (this.imageWidth != width) {
      this.imageWidth = width ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ͼƬĸ߶.</zh>
   * <en>Returns the image height.</en>
   *
   * @return int <zh>߶.</zh>
   * <en>The height.</en>
   */
  public int getImageHeight() {
    return this.imageHeight ;
  }

  /**
   * <zh>ͼƬĸ߶,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the image height and sends
   * a {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param height int <zh>߶.</zh>
   * <en>The height.</en>
   */
  public void setImageHeight(int height) {
    if (this.imageHeight != height) {
      this.imageHeight = height ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ע(annotation)ͼ.</zh>
   * <en>Draws the annotation.</en>
   *
   * @param xAxis AbstractValueAxis <zh>x (Ϊnull).</zh>
   * <en>The x-axis (null not permitted).</en>
   * @param xAxisEdge RectangleEdge <zh>xλ (Ϊnull).</zh>
   * <en>The x-axis edge (null not permitted).</en>
   * @param yAxis AbstractValueAxis <zh>y (Ϊnull).</zh>
   * <en>The y-axis (null not permitted).</en>
   * @param yAxisEdge RectangleEdge <zh>yλ (Ϊnull).</zh>
   * <en>The y-axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param annotationShapes AnnotationShapeCollection <zh>ע(annotation)ͼμ (Ϊnull).</zh>
   * <en>Collects information about the annotation (null not permitted).</en>
   */
  public void drawAnnotation(AbstractValueAxis xAxis, RectangleEdge xAxisEdge,
                             AbstractValueAxis yAxis, RectangleEdge yAxisEdge,
                             Rectangle2D dataArea, ChartOrientation orientation,
                             AnnotationShapeCollection annotationShapes) {
    if (xAxis == null) {
      throw new IllegalArgumentException("Null 'xAxis' argument.") ;
    }
    if (xAxisEdge == null) {
      throw new IllegalArgumentException("Null 'xAxisEdge' argument.") ;
    }
    if (yAxis == null) {
      throw new IllegalArgumentException("Null 'yAxis' argument.") ;
    }
    if (yAxisEdge == null) {
      throw new IllegalArgumentException("Null 'yAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (annotationShapes == null) {
      throw new IllegalArgumentException("Null 'annotationShapes' argument.") ;
    }

    double xPos = xAxis.valueToJava2D(dataArea, xAxisEdge, this.x, false) ;
    if (xPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
      return ;
    }

    double yPos = yAxis.valueToJava2D(dataArea, yAxisEdge, this.y, false) ;
    if (yPos == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
      return ;
    }

    int imageWidth = (this.imageWidth <= 0) ? this.image.getWidth(null) : this.imageWidth ;
    int imageHeight = (this.imageHeight <= 0) ? this.image.getHeight(null) : this.imageHeight ;

    Rectangle2D drawArea = null ;
    if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      drawArea = new Rectangle2D.Double(yPos, xPos, Math.min(imageWidth, dataArea.getMaxX() - yPos),
                                        Math.min(imageHeight, dataArea.getMaxY() - xPos)) ;
    } else if (orientation.equals(ChartOrientation.VERTICAL)) {
      drawArea = new Rectangle2D.Double(xPos, yPos, Math.min(imageWidth, dataArea.getMaxX() - xPos),
                                        Math.min(imageHeight, dataArea.getMaxY() - yPos)) ;
    }

    RectangleImageChartShape imageShape = new RectangleImageChartShape(drawArea, this.image) ;
    imageShape.setTooltipText(getTooltipText()) ;
    annotationShapes.addAnnotationShape(imageShape) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof XYImageAnnotation)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    XYImageAnnotation that = (XYImageAnnotation) obj ;
    if (this.x != that.x) {
      return false ;
    }
    if (this.y != that.y) {
      return false ;
    }
    if (!EqualsUtil.equals(this.image, that.image)) {
      return false ;
    }
    if (this.imageWidth != that.imageWidth) {
      return false ;
    }
    if (this.imageHeight != that.imageHeight) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double hashcode = super.hashCode() ;
    hashcode = 37 * hashcode + this.x ;
    hashcode = 37 * hashcode + this.y ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.image) ;
    hashcode = 37 * hashcode + this.imageWidth ;
    hashcode = 37 * hashcode + this.imageHeight ;

    return (int) hashcode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "XYImageAnnotation" ;
  }

}
