package com.ijchart.xychart.annotation ;

import java.awt.BasicStroke ;
import java.awt.Color ;
import java.awt.Paint ;
import java.awt.Shape ;
import java.awt.Stroke ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.AnnotationShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.shape.GeneralPathChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;

/**
 * <p>Title: IJChart</p>
 * <zh>{@link com.ijchart.xychart.plot.XYPlot}ͼע.</zh>
 * <en>A shape annotation that can be placed on a {@link com.ijchart.xychart.plot.XYPlot}.</en>
 * <p>Description: a chart library for the Java(tm) platform.</p>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYShapeAnnotation extends AbstractAnnotation implements IXYAnnotation, Cloneable,
  Serializable {

  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 1405561837582323285L ;

  /**
   * <zh>ȱʡĻͼα߿paint.</zh>
   * <en>The default paint used to draw the shape's border.</en>
   */
  public static final Paint DEFAULT_BORDER_PAINT = Color.black ;

  /**
   * <zh>ȱʡĻͼα߿stroke.</zh>
   * <en>The default stroke used to draw the shape's border.</en>
   */
  public static final Stroke DEFAULT_BORDER_STROKE = new BasicStroke(1) ;

  /**
   * <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  private transient Shape shape ;

  /**
   * <zh>ͼα߿paint.</zh>
   * <en>The paint used to draw the shape's border.</en>
   */
  private transient Paint borderPaint ;

  /**
   * <zh>ͼα߿stroke.</zh>
   * <en>The stroke used to draw the shape's border.</en>
   */
  private transient Stroke borderStroke ;

  /**
   * <zh>ͼõpaint.</zh>
   * <en>The paint used to fill the shape.</en>
   */
  private transient Paint fillPaint ;

  /**
   * <zh>ͼεȷעλ.</zh>
   * <en>Create a new annotation to be displayed at the given location.</en>
   *
   * @param shape Shape <zh>ͼ (Ϊnull).</zh>
   * <en>The shape (null not permitted).</en>
   */
  public XYShapeAnnotation(Shape shape) {
    this(shape, DEFAULT_BORDER_PAINT, DEFAULT_BORDER_STROKE, null) ;
  }

  /**
   * <zh>ͼεȷעλ.</zh>
   * <en>Create a new annotation to be displayed at the given location.</en>
   *
   * @param shape Shape <zh>ͼ (Ϊnull).</zh>
   * <en>The shape (null not permitted).</en>
   * @param borderPaint Paint <zh>ͼα߿paint (Ϊnull).</zh>
   * <en>The paint used to draw the shape's border (null permitted).</en>
   * @param borderStroke Stroke <zh>ͼα߿stroke (Ϊnull).</zh>
   * <en>The stroke used to draw the shape's border (null permitted).</en>
   */
  public XYShapeAnnotation(Shape shape, Paint borderPaint, Stroke borderStroke) {
    this(shape, borderPaint, borderStroke, null) ;
  }

  /**
   * <zh>ͼεȷעλ.</zh>
   * <en>Create a new annotation to be displayed at the given location.</en>
   *
   * @param shape Shape <zh>ͼ (Ϊnull).</zh>
   * <en>The shape (null not permitted).</en>
   * @param borderPaint Paint <zh>ͼα߿paint (Ϊnull).</zh>
   * <en>The paint used to draw the shape's border (null permitted).</en>
   * @param borderStroke Stroke <zh>ͼα߿stroke (Ϊnull).</zh>
   * <en>The stroke used to draw the shape's border (null permitted).</en>
   * @param fillPaint Paint <zh>ͼõpaint (Ϊnull).</zh>
   * <en>The paint used to fill the shape (null permitted).</en>
   */
  public XYShapeAnnotation(Shape shape, Paint borderPaint, Stroke borderStroke, Paint fillPaint) {
    if (shape == null) {
      throw new IllegalArgumentException("Null 'shape' argument.") ;
    }

    this.shape = shape ;
    this.borderPaint = borderPaint ;
    this.borderStroke = borderStroke ;
    this.fillPaint = fillPaint ;
  }

  /**
   * <zh>ͼζ.</zh>
   * <en>Returns the shape.</en>
   *
   * @return Shape <zh>ͼ (Ϊnull).</zh>
   * <en>The shape (never null).</en>
   */
  public Shape getShape() {
    return this.shape ;
  }

  /**
   * <zh>ͼζ,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the shape and sends
   * a {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param shape Shape <zh>ͼ (Ϊnull).</zh>
   * <en>The shape (null not permitted).</en>
   */
  public void setShape(Shape shape) {
    if (shape == null) {
      throw new IllegalArgumentException("Null 'shape' argument.") ;
    }

    this.shape = shape ;
    fireAnnotationChanged() ;
  }

  /**
   * <zh>ػͼα߿paint.</zh>
   * <en>Returns the paint used to draw the shape's border.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The border paint (maybe null).</en>
   */
  public Paint getBorderPaint() {
    return this.borderPaint ;
  }

  /**
   * <zh>ûͼα߿paint,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the paint used to draw the shape's border and sends
   * a {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The border paint (null permitted).</en>
   */
  public void setBorderPaint(Paint paint) {
    if (!(this.borderPaint == null && paint == null)) {
      this.borderPaint = paint ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ػͼα߿stroke.</zh>
   * <en>Returns the stroke used to draw the shape's border.</en>
   *
   * @return Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The border stroke (maybe null).</en>
   */
  public Stroke getBorderStroke() {
    return this.borderStroke ;
  }

  /**
   * <zh>ûͼα߿stroke,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>Sets the stroke used to draw the shape's border and sends
   * a {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param stroke Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The border stroke (null permitted).</en>
   */
  public void setBorderStroke(Stroke stroke) {
    if (!(this.borderStroke == null && stroke == null)) {
      this.borderStroke = stroke ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ͼõpaint.</zh>
   * <en>Returns the paint used to fill the shape.</en>
   *
   * @return Paint <zh>paint (Ϊnull).</zh>
   * <en>The fill paint (maybe null).</en>
   */
  public Paint getFillPaint() {
    return this.fillPaint ;
  }

  /**
   * <zh>ͼõpaint,м
   * {@link com.ijchart.xychart.event.AnnotationChangeEvent}¼.</zh>
   * <en>The paint used to fill the shape and sends
   * a {@link com.ijchart.xychart.event.AnnotationChangeEvent} to all registered listeners.</en>
   *
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The fill paint (null permitted).</en>
   */
  public void setFillPaint(Paint paint) {
    if (!(this.fillPaint == null && paint == null)) {
      this.fillPaint = paint ;
      fireAnnotationChanged() ;
    }
  }

  /**
   * <zh>ע(annotation)ͼ.</zh>
   * <en>Draws the annotation.</en>
   *
   * @param xAxis AbstractValueAxis <zh>x (Ϊnull).</zh>
   * <en>The x-axis (null not permitted).</en>
   * @param xAxisEdge RectangleEdge <zh>xλ (Ϊnull).</zh>
   * <en>The x-axis edge (null not permitted).</en>
   * @param yAxis AbstractValueAxis <zh>y (Ϊnull).</zh>
   * <en>The y-axis (null not permitted).</en>
   * @param yAxisEdge RectangleEdge <zh>yλ (Ϊnull).</zh>
   * <en>The y-axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param annotationShapes AnnotationShapeCollection <zh>ע(annotation)ͼμ (Ϊnull).</zh>
   * <en>Collects information about the annotation (null not permitted).</en>
   */
  public void drawAnnotation(AbstractValueAxis xAxis, RectangleEdge xAxisEdge,
                             AbstractValueAxis yAxis, RectangleEdge yAxisEdge,
                             Rectangle2D dataArea, ChartOrientation orientation,
                             AnnotationShapeCollection annotationShapes) {
    if (xAxis == null) {
      throw new IllegalArgumentException("Null 'xAxis' argument.") ;
    }
    if (xAxisEdge == null) {
      throw new IllegalArgumentException("Null 'xAxisEdge' argument.") ;
    }
    if (yAxis == null) {
      throw new IllegalArgumentException("Null 'yAxis' argument.") ;
    }
    if (yAxisEdge == null) {
      throw new IllegalArgumentException("Null 'yAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (annotationShapes == null) {
      throw new IllegalArgumentException("Null 'annotationShapes' argument.") ;
    }

    Rectangle2D bounds = this.shape.getBounds2D() ;
    double x0 = bounds.getMinX() ;
    double x1 = bounds.getMaxX() ;
    double xx0 = xAxis.valueToJava2D(dataArea, xAxisEdge, x0, false) ;
    if (xx0 == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
      return ;
    }
    double xx1 = xAxis.valueToJava2D(dataArea, xAxisEdge, x1, false) ;
    if (xx1 == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
      return ;
    }

    double m00 = (xx1 - xx0) / (x1 - x0) ;
    double m02 = xx0 - x0 * m00 ;

    double y0 = bounds.getMaxY() ;
    double y1 = bounds.getMinY() ;
    double yy0 = yAxis.valueToJava2D(dataArea, yAxisEdge, y0, false) ;
    if (yy0 == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
      return ;
    }
    double yy1 = yAxis.valueToJava2D(dataArea, yAxisEdge, y1, false) ;
    if (yy1 == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
      return ;
    }

    double m11 = (yy1 - yy0) / (y1 - y0) ;
    double m12 = yy0 - m11 * y0 ;

    //  create transform & transform shape
    Shape s = null ;
    if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      AffineTransform t1 = new AffineTransform(0.0f, 1.0f, 1.0f, 0.0f, 0.0f, 0.0f) ;
      AffineTransform t2 = new AffineTransform(m11, 0.0f, 0.0f, m00, m12, m02) ;
      s = t1.createTransformedShape(this.shape) ;
      s = t2.createTransformedShape(s) ;
    } else if (orientation.equals(ChartOrientation.VERTICAL)) {
      AffineTransform t = new AffineTransform(m00, 0, 0, m11, m02, m12) ;
      s = t.createTransformedShape(this.shape) ;
    }

    GeneralPathChartShape pathShape = new GeneralPathChartShape(new GeneralPath(s)) ;
    pathShape.setTooltipText(getTooltipText()) ;
    if (this.borderPaint != null && this.borderStroke != null) {
      pathShape.setDrawBorder(true) ;
      pathShape.setBorderPaint(this.borderPaint) ;
      pathShape.setBorderStroke(this.borderStroke) ;
    } else {
      pathShape.setDrawBorder(false) ;
    }

    if (this.fillPaint != null) {
      pathShape.setFillShape(true) ;
      pathShape.setFillPaint(this.fillPaint) ;
    }

    annotationShapes.addAnnotationShape(pathShape) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof XYShapeAnnotation)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    XYShapeAnnotation that = (XYShapeAnnotation) obj ;
    if (!EqualsUtil.equals(this.shape, that.shape)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.borderPaint, that.borderPaint)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.borderStroke, that.borderStroke)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.fillPaint, that.fillPaint)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double hashcode = super.hashCode() ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.shape) ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.borderPaint) ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.borderStroke) ;
    hashcode = 37 * hashcode + HashcodeUtil.hashCode(this.fillPaint) ;

    return (int) hashcode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "XYShapeAnnotation" ;
  }

  /**
   * <zh>л</zh>
   * <en>Provides serialization support</en>
   *
   * @param stream <zh></zh>
   * <en>the output stream</en>
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;

    SerialUtil.writeShape(stream, this.shape) ;
    SerialUtil.writePaint(stream, this.borderPaint) ;
    SerialUtil.writeStroke(stream, this.borderStroke) ;
    SerialUtil.writePaint(stream, this.fillPaint) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;

    this.shape = SerialUtil.readShape(stream) ;
    this.borderPaint = SerialUtil.readPaint(stream) ;
    this.borderStroke = SerialUtil.readStroke(stream) ;
    this.fillPaint = SerialUtil.readPaint(stream) ;

  }
}
