package com.ijchart.xychart.axis ;

import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.ui.RectangleEdge ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ڻͼĸռĿռ</zh>
 * <en> A record that contains the space required at each edge of a plot.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class AxisSpace implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -1320913013576339220L ;

  /**
   * <zh>˿ռ.</zh>
   * <en>The top space. </en>
   */
  private double top ;

  /**
   * <zh>׶˿ռ.</zh>
   * <en>The bottom space. </en>
   */
  private double bottom ;

  /**
   * <zh>ռ.</zh>
   * <en>The left space. </en>
   */
  private double left ;

  /**
   * <zh>Ҳռ.</zh>
   * <en>The right space. </en>
   */
  private double right ;

  /**
   * <zh>µĶ,ҿռ䶼Ϊ0.</zh>
   * <en>Creates a new axis space record.</en>
   */
  public AxisSpace() {
    this.top = 0.0 ;
    this.bottom = 0.0 ;
    this.left = 0.0 ;
    this.right = 0.0 ;
  }

  /**
   * <zh>ض˿ռС.</zh>
   * <en>Returns the space reserved for axes at the top of the plot area.</en>
   *
   * @return <zh>˿ռС.</zh>
   * <en>The space (in Java2D units).</en>
   */
  public double getTop() {
    return this.top ;
  }

  /**
   * <zh>ö˿ռС.</zh>
   * <en>Sets the space reserved for axes at the top of the plot area.</en>
   *
   * @param space <zh>˿ռС.</zh>
   * <en>The space (in Java2D units).</en>
   */
  public void setTop(double space) {
    this.top = space ;
  }

  /**
   * <zh>ص׶˿ռС.</zh>
   * <en>Returns the space reserved for axes at the bottom of the plot area.</en>
   *
   * @return <zh>׶˿ռС.</zh>
   * <en>The space (in Java2D units).</en>
   */
  public double getBottom() {
    return this.bottom ;
  }

  /**
   * <zh>õ׶˿ռС.</zh>
   * <en>Sets the space reserved for axes at the bottom of the plot area.</en>
   *
   * @param space <zh>׶˿ռС.</zh>
   * <en>The space (in Java2D units).</en>
   */
  public void setBottom(double space) {
    this.bottom = space ;
  }

  /**
   * <zh>ռС.</zh>
   * <en>Returns the space reserved for axes at the left of the plot area.</en>
   *
   * @return <zh>ռС.</zh>
   * <en>The space (in Java2D units).</en>
   */
  public double getLeft() {
    return this.left ;
  }

  /**
   * <zh>ռС.</zh>
   * <en>Sets the space reserved for axes at the left of the plot area.</en>
   *
   * @param space <zh>ռС.</zh>
   * <en>The space (in Java2D units).</en>
   */
  public void setLeft(double space) {
    this.left = space ;
  }

  /**
   * <zh>ҲռС.</zh>
   * <en>Returns the space reserved for axes at the right of the plot area.</en>
   *
   * @return <zh>ҲռС.</zh>
   * <en>The space (in Java2D units).</en>
   */
  public double getRight() {
    return this.right ;
  }

  /**
   * <zh>ҲռС.</zh>
   * <en>Sets the space reserved for axes at the right of the plot area.</en>
   *
   * @param space <zh>ҲռС.</zh>
   * <en>The space (in Java2D units).</en>
   */
  public void setRight(double space) {
    this.right = space ;
  }

  /**
   * <zh>ĳĿռС.</zh>
   * <en>Adds space to the top, bottom, left or right edge of the plot area.</en>
   *
   * @param space <zh>ռС.</zh>
   * <en>The space (in Java2D units).</en>
   * @param edge <zh>ĳ߽ (Ϊnull).</zh>
   * <en>The edge (null not permitted).</en>
   */
  public void add(double space, RectangleEdge edge) {
    if (edge == null) {
      throw new IllegalArgumentException("Null 'edge' argument.") ;
    }

    if (edge.equals(RectangleEdge.TOP)) {
      this.top += space ;
    } else if (edge.equals(RectangleEdge.BOTTOM)) {
      this.bottom += space ;
    } else if (edge.equals(RectangleEdge.LEFT)) {
      this.left += space ;
    } else if (edge.equals(RectangleEdge.RIGHT)) {
      this.right += space ;
    } else {
      throw new IllegalStateException("Unrecognised 'edge' argument.") ;
    }
  }

  /**
   * <zh>ñȥСarea.</zh>
   * <en>Shrinks the given rectangle by the amount of these space.</en>
   *
   * @param area Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The rectangle area (null not permitted).</en>
   */
  public void trim(Rectangle2D area) {
    if (area == null) {
      throw new IllegalArgumentException("Null 'area' argument.") ;
    }

    area.setRect(area.getX() + this.left, area.getY() + this.top,
                 area.getWidth() - this.left - this.right,
                 area.getHeight() - this.top - this.bottom) ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Returns a clone of the object.</en>
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>Ƿ</zh>
   * <en>Tests this object for equality with another object.</en>
   *
   * @param obj <zh>ҪȽϵĶ.</zh>
   * <en>The object to compare against.</en>
   *
   * @return true or false.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }

    if (!(obj instanceof AxisSpace)) {
      return false ;
    }
    AxisSpace that = (AxisSpace) obj ;
    if (this.top != that.top) {
      return false ;
    }
    if (this.bottom != that.bottom) {
      return false ;
    }
    if (this.left != that.left) {
      return false ;
    }
    if (this.right != that.right) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code for this object.</en>
   *
   * @return <zh>hashcodeֵ.</zh>
   * <en>A hash code.</en>
   */
  public int hashCode() {
    long l = Double.doubleToLongBits(this.top) ;
    int result = 37 * 23 + (int) (l ^ (l >>> 32)) ;
    l = Double.doubleToLongBits(this.bottom) ;
    result = 37 * result + (int) (l ^ (l >>> 32)) ;
    l = Double.doubleToLongBits(this.left) ;
    result = 37 * result + (int) (l ^ (l >>> 32)) ;
    l = Double.doubleToLongBits(this.right) ;
    result = 37 * result + (int) (l ^ (l >>> 32)) ;

    return result ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representing the object (for debugging purposes).</en>
   *
   * @return A string.
   */
  public String toString() {
    return "[left=" + this.left + ",right=" + this.right
      + ",top=" + this.top + ",bottom=" + this.bottom + "]" ;
  }
}
