package com.ijchart.xychart.axis.value ;

import java.awt.BasicStroke ;
import java.awt.Graphics2D ;
import java.awt.Paint ;
import java.awt.Shape ;
import java.awt.Stroke ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;
import java.text.DateFormat ;
import java.util.ArrayList ;
import java.util.Arrays ;
import java.util.Date ;
import java.util.List ;
import java.util.TimeZone ;

import com.ijchart.xychart.chartinfo.AxisShapeCollection ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.DateTickUnitType ;
import com.ijchart.xychart.constant.ValueAxisOffsetType ;
import com.ijchart.xychart.data.NumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.exception.InvalidChartDataException ;
import com.ijchart.xychart.format.IFormatter ;
import com.ijchart.xychart.range.NumberRange ;
import com.ijchart.xychart.render.IRenderWithAxis ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.shape.LineChartShape ;
import com.ijchart.xychart.shape.RectangleTextChartShape ;
import com.ijchart.xychart.tickunit.DateTickUnit ;
import com.ijchart.xychart.tickunit.DoubleTickUnit ;
import com.ijchart.xychart.tickunit.INumberTickUnit ;
import com.ijchart.xychart.tickunit.LogTickUnit ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.ui.Rectangle ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.RectangleInsets ;
import com.ijchart.xychart.util.AngleUtil ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.MathUtil ;
import com.ijchart.xychart.util.StringUtil ;
import com.ijchart.xychart.util.TextUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ʾֵ,ϵֵΪdouble.</zh>
 * <en>The class for axes that display value data, where values are measured
 * using the double primitive.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class NumberValueAxis extends AbstractValueAxis implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -4108110604308595030L ;

  /**
   * <zh>ȱʡڲϵĿ̶ֵ.</zh>
   * <en>The default {@link INumberTickUnit} for generate axis tick units.</en>
   */
  public static final INumberTickUnit DEFAULT_NUMBER_TICK_UNIT = new DoubleTickUnit() ;

  /**
   * <zh>ڲϵĿ̶ֵ.</zh>
   * <en>The {@link INumberTickUnit} for generate axis tick units.</en>
   */
  protected INumberTickUnit tickUnit ;

  /**
   * <zh>ʱ.</zh>
   * <en>The time zone.</en>
   */
  private TimeZone timeZone ;

  /**
   * <zh>ʽĿ̶ֵ.</zh>
   * <en>The formated tick labels.</en>
   */
  protected String[] tickLabels ;

  /**
   * <zh>͵Ŀ̶ֵ.</zh>
   * <en>The number ticks.</en>
   */
  protected String[] numberTicks ;

  /**
   * <zh>̶ֵӦֵ.</zh>
   * <en>The number ticks.</en>
   */
  protected double[] valueTicks ;

  /**
   * <zh>xĿ̶λ.</zh>
   * <en>The x position of x axis.</en>
   */
  private double[] xTickPositions ;

  /**
   * <zh>yĿ̶λ.</zh>
   * <en>The y position of y axis.</en>
   */
  private double[] yTickPositions ;

  /**
   * <zh>ûԶݷΧ,ֵΪnullʱ򲻻ԶݷΧ.</zh>
   * <en>The dataset range of user defined,it is not automatically calculated when this value
   *  is not null.</en>
   */
  private NumberRange userDefineDatasetRange ;

  /**
   * <zh>ʱʹõݷΧ.</zh>
   * <en>The dataset range for zoom chart.</en>
   */
  private NumberRange zoomDatasetRange ;

  /**
   * <zh>ݼ.</zh>
   * <en>Constructs an axis using a dataset.</en>
   *
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   */
  public NumberValueAxis(IMatrixDataset dataset) {
    this(null, dataset) ;
  }

  /**
   * <zh>ǩݼ.</zh>
   * <en>Constructs an axis using label and a dataset.</en>
   *
   * @param label <zh>ǩ.</zh>
   * <en>The axis label (null permitted).</en>
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   */
  public NumberValueAxis(String label, IMatrixDataset dataset) {
    super(label, dataset) ;

    this.tickUnit = DEFAULT_NUMBER_TICK_UNIT ;
    this.tickLabels = null ;
    this.userDefineDatasetRange = null ;
    this.zoomDatasetRange = null ;
    this.timeZone = null ;
  }

  /**
   * <zh>ڲϵĿ̶ֵ.</zh>
   * <en>Returns the {@link INumberTickUnit} for generate axis tick units.</en>
   *
   * @return INumberTickUnit <zh>̶ֵ.</zh>
   * <en>The {@link INumberTickUnit} for generate axis tick units.</en>
   */
  public INumberTickUnit getTickUnit() {
    return this.tickUnit ;
  }

  /**
   * <zh>ڲϵĿ̶ֵ,м{@link com.ijchart.xychart.event.AxisChangeEvent}
   * ¼.</zh>
   * <en>The {@link INumberTickUnit} for generate axis tick units and notifies registered listeners
   * that the axis has changed.</en>
   *
   * @param tickUnit INumberTickUnit <zh>ϵĿ̶ֵ (Ϊnull).</zh>
   * <en>The {@link INumberTickUnit} for generate axis tick units (null not permitted).</en>
   */
  public void setTickUnit(INumberTickUnit tickUnit) {
    if (tickUnit == null) {
      throw new IllegalArgumentException("Null 'tickUnit' argument.") ;
    }

    if (!this.tickUnit.equals(tickUnit)) {
      this.tickUnit = tickUnit ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>ʱ.</zh>
   * <en>Returns the time zone.</en>
   *
   * @return TimeZone <zh>ʱ.</zh>
   * <en>The time zone.</en>
   */
  public TimeZone getTimeZone() {
    return this.timeZone ;
  }

  /**
   * <zh>ʱ,м{@link com.ijchart.xychart.event.AxisChangeEvent}¼.</zh>
   * <en>Sets the time zone and sends a
   * {@link com.ijchart.xychart.event.AxisChangeEvent} to all registered listeners.</en>
   *
   * @param timeZone TimeZone <zh>ʱ.</zh>
   * <en>The time zone.</en>
   */
  public void setTimeZone(TimeZone timeZone) {
    if (!(this.timeZone == null && timeZone == null)) {
      this.timeZone = timeZone ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>͵Ŀ̶ֵ.</zh>
   * <en>Return the number ticks.</en>
   *
   * @return String[] <zh>͵Ŀ̶ֵ.</zh>
   * <en>The number ticks.</en>
   */
  public String[] getNumberTicks() {
    return this.numberTicks ;
  }

  /**
   * <zh>ûԶݷΧ.</zh>
   * <en>Returns the dataset range of user defined.</en>
   *
   * @return NumberRange <zh>ݷΧ (Ϊnull).</zh>
   * <en>The dataset range (maybe null).</en>
   */
  public NumberRange getUserDefineDatasetRange() {
    return this.userDefineDatasetRange ;
  }

  /**
   * <zh>ûԶݷΧ,м{@link com.ijchart.xychart.event.AxisChangeEvent}
   * ¼.</zh>
   * <en>Sets the dataset range of user defined and notifies registered listeners
   * that the axis has changed.</en>
   *
   * @param lower double <zh>Сֵ.</zh>
   * <en>The lower bound of the range.</en>
   * @param upper double <zh>ֵ.</zh>
   * <en>The upper bound of the range.</en>
   */
  public void setUserDefineDatasetRange(double lower, double upper) {
    this.setUserDefineDatasetRange(new NumberRange(lower, upper)) ;
  }

  /**
   * <zh>ûԶݷΧ,м{@link com.ijchart.xychart.event.AxisChangeEvent}
   * ¼.</zh>
   * <en>Sets the dataset range of user defined and notifies registered listeners
   * that the axis has changed.</en>
   *
   * @param range NumberRange <zh>ݷΧ (Ϊnull).</zh>
   * <en>The dataset range (null permitted).</en>
   */
  public void setUserDefineDatasetRange(NumberRange range) {
    if (!(this.userDefineDatasetRange == null && range == null)) {
      this.userDefineDatasetRange = range ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>ʱʹõݷΧ.</zh>
   * <en>Returns the dataset range for zoom chart.</en>
   *
   * @return NumberRange <zh>ʱʹõݷΧ (Ϊnull).</zh>
   * <en>The dataset range for zoom chart (maybe null).</en>
   */
  public NumberRange getZoomDatasetRange() {
    return this.zoomDatasetRange ;
  }

  /**
   * <zh>ʱʹõݷΧ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the dataset range for zoom chart and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param datasetRange NumberRange <zh>ʱʹõݷΧ.</zh>
   * <en>The dataset range for zoom chart.</en>
   */
  public void setZoomDatasetRange(NumberRange datasetRange) {
    if (!(this.zoomDatasetRange == null && datasetRange == null)) {
      this.zoomDatasetRange = datasetRange ;
      fireAxisChanged() ;
    }
  }

  /**
   * <zh>ıǩĹĿ.</zh>
   * <en>Calculate the width of axis.</en>
   *
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   * @param maxLengthLabel String <zh>ıǩ.</zh>
   * <en>The maximum length tick label.</en>
   *
   * @return double <zh>Ŀ.</zh>
   * <en>The width of axis.</en>
   */
  public double estimateAxisWidth(RectangleEdge edge, String maxLengthLabel) {
    float strokeWidth = 1F ;
    Stroke stroke = getAxisLineStroke() ;
    if (stroke != null && stroke instanceof BasicStroke) {
      BasicStroke tmpStroke = (BasicStroke) stroke ;
      strokeWidth = tmpStroke.getLineWidth() ;
    }

    double tickMarkLength = 0 ;
    if (isTickMarkVisible()) {
      tickMarkLength = getTickMarkOutsideLength() ;
    }

    double tickMargin = 0 ;
    if (isTickMarkVisible()) {
      tickMargin = WIDTH_TICK_MARK_AND_TICK_LABEL ;
    }

    double tickLabelLen = 0 ;
    if (isTickLabelsVisible() && !StringUtil.isEmpty(maxLengthLabel)) {
      Rectangle2D bound = TextUtil.getStringBound(getTickLabelFont(), maxLengthLabel) ;
      if (RectangleEdge.isTopOrBottom(edge)) {
        tickLabelLen = bound.getHeight() ;
      } else {
        tickLabelLen = bound.getWidth() ;
      }
    }

    double axisLabelLen = 0 ;
    if (isAxisLabelVisible()) {
      Rectangle2D bound = TextUtil.getStringBound(getAxisLabelFont(), getAxisLabel()) ;
      if (RectangleEdge.isTopOrBottom(edge)) {
        axisLabelLen = bound.getHeight() ;
      } else {
        axisLabelLen = bound.getWidth() ;
      }
    }

    return strokeWidth + tickMarkLength + tickMargin + tickLabelLen + axisLabelLen ;
  }

  /**
   * <zh>ϵĿ̶.</zh>
   * <en>Draws tick marks in a specified area and edge.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param axisArea Rectangle2D <zh>.</zh>
   * <en>The axis draw area.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   * @param shapeCollection AxisShapeCollection <zh>ͼμ.</zh>
   * <en>Collects information about the axis (null permitted).</en>
   * @param cursor <zh>λ.</zh>
   * <en>The cursor location.</en>
   *
   * @return double <zh>̶ȹλ.</zh>
   * <en>The cursor location after draw tick mark.</en>
   */
  protected double drawTickMark(Graphics2D g2, Rectangle2D axisArea, RectangleEdge edge,
                                AxisShapeCollection shapeCollection, double cursor) {
    this.xTickPositions = null ;
    this.yTickPositions = null ;
    if (shapeCollection == null) {
      calculateLabels(g2, axisArea, edge) ;
      return 0 ;
    } else {
      calculateLabels(g2, axisArea, edge) ;
    }

    if (!isTickMarkVisible()) {
      return cursor ;
    }

    String[] formatedLabels = getFormatedTickLabels(g2, axisArea, edge) ;
    LineChartShape shape = null ;
    int tickMarkCount = getTickMarkCount() ;
    boolean offsetValueLine = isOffsetValueLine() ;

    int offsetTickMarkCount = offsetValueLine ? ((tickMarkCount > 1) ? -1 : 0) : 0 ; //Ҫƫƿ̶ֵ,̶ҲҪ1
    if (RectangleEdge.isTopOrBottom(edge)) { //top or bottom axis
      calculateXTickPositions(axisArea, edge) ;
      double categoryWidth = 0 ;
      if (this.xTickPositions.length > 1) {
        categoryWidth = Math.abs(this.xTickPositions[0] - this.xTickPositions[1]) ;
      }

      double offsetLength = (offsetValueLine ? 0.5 * categoryWidth : 0) ;
      for (int i = 0 ; i < this.xTickPositions.length + offsetTickMarkCount ; i++) {
        if (!isDrawEmptyLabelTickMark() && (formatedLabels != null && i < formatedLabels.length
                                            && StringUtil.isEmpty(formatedLabels[i]))) {
          continue ;
        }

        double x = this.xTickPositions[i] + offsetLength ;
        if (!(x >= axisArea.getMinX() && x <= axisArea.getMaxX())) { //x᷶Χ֮ھͺ
          continue ;
        }

        if (edge.equals(RectangleEdge.TOP)) {
          shape = new LineChartShape(x, cursor - getTickMarkOutsideLength(),
                                     x, cursor + getTickMarkInsideLength()) ;
        } else if (edge.equals(RectangleEdge.BOTTOM)) {
          shape = new LineChartShape(x, axisArea.getY() - getTickMarkInsideLength(),
                                     x, axisArea.getY() + getTickMarkOutsideLength()) ;
        }

        shape.setDrawBorder(true) ;
        shape.setBorderStroke(getTickMarkStroke()) ;
        shape.setBorderPaint(getTickMarkPaint()) ;
        shapeCollection.addTickMarkShape(shape) ; //һͼ
      }

      if (edge.equals(RectangleEdge.TOP)) {
        cursor -= getTickMarkOutsideLength() ;
      } else if (edge.equals(RectangleEdge.BOTTOM)) {
        cursor += getTickMarkOutsideLength() ;
      }
    } else if (RectangleEdge.isLeftOrRight(edge)) { //left or right axis
      calculateYTickPositions(axisArea, edge) ;
      double categoryWidth = 0 ;
      if (this.yTickPositions.length > 1) {
        categoryWidth = Math.abs(this.yTickPositions[0] - this.yTickPositions[1]) ;
      }

      double offsetLength = (offsetValueLine ? 0.5 * categoryWidth : 0) ;
      for (int i = 0 ; i < tickMarkCount + offsetTickMarkCount ; i++) {
        if (!isDrawEmptyLabelTickMark() && (formatedLabels != null && i < formatedLabels.length
                                            && StringUtil.isEmpty(formatedLabels[i]))) {
          continue ;
        }

        double y = this.yTickPositions[i] + offsetLength ;
        if (!(y >= axisArea.getMinY() && y <= axisArea.getMaxY())) { //y᷶Χ֮ھͺ
          continue ;
        }

        if (edge.equals(RectangleEdge.LEFT)) {
          shape = new LineChartShape(cursor - getTickMarkOutsideLength(), y,
                                     cursor + getTickMarkInsideLength(), y) ;
        } else if (edge.equals(RectangleEdge.RIGHT)) {
          shape = new LineChartShape(cursor - getTickMarkInsideLength(), y,
                                     cursor + getTickMarkOutsideLength(), y) ;
        }

        shape.setDrawBorder(true) ;
        shape.setBorderStroke(getTickMarkStroke()) ;
        shape.setBorderPaint(getTickMarkPaint()) ;
        shapeCollection.addTickMarkShape(shape) ; //һͼ
      }

      if (edge.equals(RectangleEdge.LEFT)) {
        cursor -= getTickMarkOutsideLength() ;
      } else if (edge.equals(RectangleEdge.RIGHT)) {
        cursor += getTickMarkOutsideLength() ;
      }
    }

    return cursor ;
  }

  /**
   * <zh>̶ȵĸ.</zh>
   * <en>Returns the tick mark count.</en>
   *
   * @return int <zh>̶ȵĸ.</zh>
   * <en>The tick mark count.</en>
   */
  protected int getTickMarkCount() {
    return (this.tickLabels == null) ? 0 : this.tickLabels.length ;
  }

  /**
   * <zh>ϵĿ̶ֵıǩ.</zh>
   * <en>Draws tick mark labels in a specified area and edge.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param plotArea Rectangle2D <zh>ͼplot.</zh>
   * <en>The area within which the plot should be drawn.</en>
   * @param axisArea Rectangle2D <zh>.</zh>
   * <en>The axis draw area.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   * @param shapeCollection AxisShapeCollection <zh>ͼμ.</zh>
   * <en>Collects information about the axis (null permitted).</en>
   * @param cursor <zh>λ.</zh>
   * <en>The cursor location.</en>
   *
   * @return double <zh>̶ֵλ.</zh>
   * <en>The cursor location after draw tick label.</en>
   */
  protected double drawTickLabel(Graphics2D g2, Rectangle2D plotArea, Rectangle2D axisArea,
                                 RectangleEdge edge, AxisShapeCollection shapeCollection,
                                 double cursor) {
    if (!isVisible() || !isTickLabelsVisible()) {
      return cursor ;
    }

    String[] labels = getTickLabels(g2, axisArea, edge) ;
    if (StringUtil.isEmpty(labels, true)) {
      return cursor ;
    }

    return drawTickLabelReally(g2, plotArea, axisArea, edge, shapeCollection, cursor, labels) ;
  }

  /**
   * <zh>Ļǩķ.</zh>
   * <en>The really method for draw tick labels.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param plotArea Rectangle2D <zh>ͼplot.</zh>
   * <en>The area within which the plot should be drawn.</en>
   * @param axisArea Rectangle2D <zh>.</zh>
   * <en>The axis draw area.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   * @param shapeCollection AxisShapeCollection <zh>ͼμ.</zh>
   * <en>Collects information about the axis (null permitted).</en>
   * @param cursor <zh>λ.</zh>
   * <en>The cursor location.</en>

   * @param labels String[] <zh>ǩ.</zh>
   * <en>The labels array.</en>
   * @return double <zh>̶ֵλ.</zh>
   * <en>The cursor location after draw tick label.</en>
   */
  protected double drawTickLabelReally(Graphics2D g2, Rectangle2D plotArea, Rectangle2D axisArea,
                                       RectangleEdge edge, AxisShapeCollection shapeCollection,
                                       double cursor, String[] labels) {
    int tickMarkCount = labels.length ;
    double tickLabelMaxWidth = getLabelMaxDrawWidth(g2, plotArea, edge, labels) ;
    Rectangle2D tickLabelDrawRect = null ;

    boolean offsetValueLine = isOffsetValueLine() ;
    int offsetTickLabelCount = offsetValueLine ? ((tickMarkCount > 1) ? -1 : 0) : 0 ; //Ҫƫƿ̶ֵ,̶ҲҪ1
    tickMarkCount += offsetTickLabelCount ;
    if (RectangleEdge.isTopOrBottom(edge)) { //top or bottom axis
      calculateXTickPositions(axisArea, edge) ;
      double categoryWidth = 0 ;
      if (this.xTickPositions.length > 1) {
        categoryWidth = Math.abs(this.xTickPositions[0] - this.xTickPositions[1]) ;
      }

      Rectangle2D labelRect = calculateLabelBound(g2, categoryWidth, tickLabelMaxWidth,
                                                  getTickLabelAngle(), edge, labels,
                                                  getTickLabelFont(), getTickLabelInsets()) ;
      AffineTransform transform = AffineTransform.getRotateInstance(getTickLabelAngle()) ;
      Rectangle2D rotateBound = transform.createTransformedShape(labelRect).getBounds2D() ;
      double offsetX = rotateBound.getWidth() * 0.5 ;

      double offsetLength = (offsetValueLine ? 0.5 * categoryWidth : 0) ;
      double cursorOffset = 0 ;
      for (int i = 0 ; i < tickMarkCount ; i++) {
        double x = this.xTickPositions[i] + offsetLength ;
        if (!(x >= axisArea.getMinX() && x <= axisArea.getMaxX())) { //x᷶Χ֮ھͺ
          continue ;
        }

        if (edge.equals(RectangleEdge.TOP)) {
          tickLabelDrawRect = new Rectangle2D.Double(x - offsetX, cursor + labelRect.getY(),
                                                     labelRect.getWidth(), labelRect.getHeight()) ;

          if (cursorOffset == 0) { //λ
            cursorOffset = -getTickLabelRotateBound(tickLabelDrawRect, getTickLabelAngle()).
                           getHeight() ;
          }
        } else if (edge.equals(RectangleEdge.BOTTOM)) {
          tickLabelDrawRect = new Rectangle2D.Double(x - offsetX, cursor + labelRect.getY(),
                                                     labelRect.getWidth(), labelRect.getHeight()) ;
          if (cursorOffset == 0) { //λ
            cursorOffset = getTickLabelRotateBound(tickLabelDrawRect, getTickLabelAngle()).
                           getHeight() ;
          }
        }

        shapeCollection.addTickLabelShape(createChartShape(tickLabelDrawRect, labels[i])) ;
      }
      cursor += cursorOffset ;
    } else if (RectangleEdge.isLeftOrRight(edge)) { //left or right axis
      calculateYTickPositions(axisArea, edge) ;
      double categoryWidth = 0 ;
      if (this.yTickPositions.length > 1) {
        categoryWidth = Math.abs(this.yTickPositions[0] - this.yTickPositions[1]) ;
      }

      Rectangle2D labelRect = calculateLabelBound(g2, categoryWidth, tickLabelMaxWidth,
                                                  getTickLabelAngle(), edge, labels,
                                                  getTickLabelFont(), getTickLabelInsets()) ;
      AffineTransform transform = AffineTransform.getRotateInstance(getTickLabelAngle()) ;
      Rectangle2D rotateBound = transform.createTransformedShape(labelRect).getBounds2D() ;
      double offsetY = rotateBound.getHeight() * 0.5 ;

      double offsetLength = (offsetValueLine ? 0.5 * categoryWidth : 0) ;
      double cursorOffset = 0 ;
      for (int i = 0 ; i < tickMarkCount ; i++) {
        double y = this.yTickPositions[i] + offsetLength ;
        if (!(y >= axisArea.getMinY() && y <= axisArea.getMaxY())) { //y᷶Χ֮ھͺ
          continue ;
        }

        if (edge.equals(RectangleEdge.LEFT)) {
          tickLabelDrawRect = new Rectangle2D.Double(cursor + labelRect.getX(), y - offsetY,
                                                     labelRect.getWidth(), labelRect.getHeight()) ;
          if (cursorOffset == 0) { //λ
            cursorOffset = -getTickLabelRotateBound(tickLabelDrawRect, getTickLabelAngle()).
                           getWidth() ;
          }
        } else if (edge.equals(RectangleEdge.RIGHT)) {
          tickLabelDrawRect = new Rectangle2D.Double(cursor + labelRect.getX(), y - offsetY,
                                                     labelRect.getWidth(), labelRect.getHeight()) ;
          if (cursorOffset == 0) { //λ
            cursorOffset = getTickLabelRotateBound(tickLabelDrawRect, getTickLabelAngle()).getWidth() ;
          }
        }

        IChartShape shape = createChartShape(tickLabelDrawRect, labels[i]) ;
        shapeCollection.addTickLabelShape(shape) ;
      }
      cursor += cursorOffset ;
    }

    return cursor ;
  }

  /**
   * <zh>̶ֵ.</zh>
   * <en>Returns the tick mark labels.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param axisArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The axis draw area (null not permitted).</en>
   * @param edge RectangleEdge <zh>λ (Ϊnull).</zh>
   * <en>The axis position (null not permitted).</en>
   *
   * @return String[] <zh>̶ֵ.</zh>
   * <en>The tick mark labels.</en>
   */
  protected String[] getTickLabels(Graphics2D g2, Rectangle2D axisArea, RectangleEdge edge) {
    return (this.tickLabels == null) ? new String[0] : this.tickLabels ;
  }

  /**
   * <zh>ֵ,IAxis.draw()֮.</zh>
   * <en>Draw the value line,only call after IAxis.draw().</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param dataArea Rectangle2D <zh>ͼdata() (Ϊnull).</zh>
   * <en>The area within which the data should be drawn (null not permitted).</en>
   * @param edge RectangleEdge <zh>λ (Ϊnull).</zh>
   * <en>The axis position (null not permitted).</en>
   * @param shapeCollection RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   */
  public void drawValueLine(Graphics2D g2, Rectangle2D dataArea, RectangleEdge edge,
                            RenderShapeCollection shapeCollection) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (edge == null) {
      throw new IllegalArgumentException("Null 'edge' argument.") ;
    }
    if (shapeCollection == null) {
      throw new IllegalArgumentException("Null 'shapeCollection' argument.") ;
    }

    if (!isValueLineVisible() || this.tickLabels == null || this.tickLabels.length <= 1) {
      return ;
    }

    double value = getValueLineValue() ;
    if (Double.isNaN(value)) {
      return ;
    }

    boolean offsetValueLine = isOffsetValueLine() ; //ǷҪƫ

    //̶ֵСֵ
    double minTickValue = 0, maxTickValue = 0 ;
    if (isLogTickUnit()) {
      String[] ticks = ((LogTickUnit)this.tickUnit).getExponentLabels() ;
      minTickValue = Double.parseDouble(ticks[0]) ;
      maxTickValue = Double.parseDouble(ticks[ticks.length - 1]) ;
      if (offsetValueLine) {
        maxTickValue = Double.parseDouble(ticks[ticks.length - 2]) ;
      }
    } else {
      minTickValue = this.valueTicks[0] ;
      maxTickValue = this.valueTicks[this.valueTicks.length - 1] ;
      if (offsetValueLine) {
        maxTickValue = this.valueTicks[this.valueTicks.length - 2] ;
      }
    }

    IChartShape shape = null ;
    if (RectangleEdge.isTopOrBottom(edge)) { //top or bottom axis
      double startX = dataArea.getX() + dataArea.getWidth() * getLowerMargin() ;
      double endX = dataArea.getMaxX() - dataArea.getWidth() * getUpperMargin() ;

      calculateXTickPositions(dataArea, edge) ;
      double categoryWidth = 0 ;
      if (this.xTickPositions.length > 1) {
        categoryWidth = Math.abs(this.xTickPositions[0] - this.xTickPositions[1]) ;
      }
      double offsetLength = (offsetValueLine ? 0.5 * categoryWidth : 0) ;
      if (value >= minTickValue && value <= maxTickValue) { //Сֵ֮
        double lineValueLen = 0 ; //ֵ߾Сֵľ
        if (minTickValue == maxTickValue) {
          lineValueLen = startX ;
        } else {
          lineValueLen = (value - minTickValue) / (maxTickValue - minTickValue) * (endX - startX) ;
        }

        double valueLineX = 0 ; //ֵߵĿʼλ
        if (isInverted()) {
          valueLineX = endX - lineValueLen ;
        } else {
          valueLineX = startX + lineValueLen ;
        }

        shape = createValueLineShape(valueLineX + offsetLength, dataArea.getMinY(),
                                     valueLineX + offsetLength, dataArea.getMaxY()) ;
      }
    } else if (RectangleEdge.isLeftOrRight(edge)) { //left or right axis
      double startY = dataArea.getY() + dataArea.getHeight() * getUpperMargin() ;
      double endY = dataArea.getMaxY() - dataArea.getHeight() * getLowerMargin() ;

      calculateYTickPositions(dataArea, edge) ;
      double categoryWidth = 0 ;
      if (this.yTickPositions.length > 1) {
        categoryWidth = Math.abs(this.yTickPositions[0] - this.yTickPositions[1]) ;
      }
      double offsetLength = (offsetValueLine ? 0.5 * categoryWidth : 0) ;
      if (value >= minTickValue && value <= maxTickValue) { //Сֵ֮
        double lineValueLen = 0 ; //ֵ߾Сֵľ
        if (minTickValue == maxTickValue) {
          lineValueLen = startY ;
        } else {
          lineValueLen = (value - minTickValue) / (maxTickValue - minTickValue) * (endY - startY) ;
        }

        double valueLineY = 0 ; //ֵߵĿʼλ
        if (isInverted()) {
          valueLineY = startY + lineValueLen ;
        } else {
          valueLineY = endY - lineValueLen ;
        }

        shape = createValueLineShape(dataArea.getX(), valueLineY + offsetLength,
                                     dataArea.getMaxX(), valueLineY + offsetLength) ;
      }
    }

    if (shape != null) {
      shapeCollection.addValueLineShape(shape) ; //һͼ
    }
  }

  /**
   * <zh>ֵͼ.</zh>
   * <en>Create a value line shape.</en>
   *
   * @param x1 double <zh>һx.</zh>
   * <en>The first point's x coordinate.</en>
   * @param y1 double <zh>һy.</zh>
   * <en>The first point's y coordinate.</en>
   * @param x2 double <zh>ڶx.</zh>
   * <en>The second point's x coordinate.</en>
   * @param y2 double <zh>ڶy.</zh>
   * <en>The second point's y coordinate.</en>
   *
   * @return IChartShape
   */
  protected IChartShape createValueLineShape(double x1, double y1, double x2, double y2) {
    IChartShape lineShape = new LineChartShape(x1, y1, x2, y2) ;
    lineShape.setBorderStroke(getValueLineStroke()) ;
    lineShape.setBorderPaint(getValueLinePaint()) ;

    return lineShape ;
  }

  /**
   * <zh>ϵıǩҪ,ҲɲmaximumCategoryLabelWidthRatioȷͼ
   * ĿȻ߶.</zh>
   * <en>Calculation width of painting coordinate axis labels required by the maximum width
   * and not more than maximumCategoryLabelWidthRatio determined by the parameters of
   * the entire region of the chart width or height</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param plotArea Rectangle2D <zh>ͼplot.</zh>
   * <en>The area within which the plot should be drawn.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   * @param formatedLabels String[] <zh>ʽ̶ֵ.</zh>
   * <en>The formatted tick label values.</en>
   *
   * @return double <zh>.</zh>
   * <en>The width.</en>
   */
  protected double getLabelMaxDrawWidth(Graphics2D g2, Rectangle2D plotArea,
                                        RectangleEdge edge, String[] formatedLabels) {
    if (!isTickLabelsVisible()) {
      return 0 ;
    }

    if (StringUtil.isEmpty(formatedLabels, true)) {
      return 0 ;
    }

    double areaWidth = 0 ;
    if (RectangleEdge.isLeftOrRight(edge)) {
      //ƱǩҪ
      double labelWidth = 0 ;
      if (isAxisLabelVisible()) {
        Rectangle2D labelRect = new Rectangle2D.Double(0, 0, plotArea.getHeight(), 0) ;
        RectangleTextChartShape shape = new RectangleTextChartShape(labelRect, getAxisLabel()) ;
        shape.setMargin(getAxisLabelInsets()) ;
        shape.setTextFont(getAxisLabelFont()) ;
        //תlabel
        Rectangle bound = shape.getMinimumBound(g2) ;
        AffineTransform transform = AffineTransform.getRotateInstance(getAxisLabelAngle()) ;
        labelWidth = transform.createTransformedShape(bound.toRectangle2D()).getBounds().
                     getWidth() ;
      }

      //ȥlabeltick mark
      areaWidth = plotArea.getWidth() * getMaximumCategoryLabelWidthRatio() ;
      areaWidth -= labelWidth ;
      if (isTickMarkVisible()) {
        areaWidth -= getTickMarkOutsideLength() ;
      }
    } else if (RectangleEdge.isTopOrBottom(edge)) {
      //ƱǩҪ߶
      double labelHeight = 0 ;
      if (isAxisLabelVisible()) {
        Rectangle2D labelRect = new Rectangle2D.Double(0, 0, plotArea.getWidth(), 0) ;
        RectangleTextChartShape shape = new RectangleTextChartShape(labelRect, getAxisLabel()) ;
        shape.setMargin(getAxisLabelInsets()) ;
        shape.setTextFont(getAxisLabelFont()) ;
        //תlabel
        Rectangle bound = shape.getMinimumBound(g2) ;
        AffineTransform transform = AffineTransform.getRotateInstance(getAxisLabelAngle()) ;
        labelHeight = transform.createTransformedShape(bound.toRectangle2D()).getBounds().
                      getHeight() ;
      }

      //ȥlabeltick mark߶
      areaWidth = plotArea.getHeight() * getMaximumCategoryLabelWidthRatio() ;
      areaWidth -= labelHeight ;
      if (isTickMarkVisible()) {
        areaWidth -= getTickMarkOutsideLength() ;
      }
    }

    //save old graphics state
    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;
    //ĿȻ߶
    AffineTransform transform = AffineTransform.getRotateInstance(getTickLabelAngle()) ;
    RectangleInsets insets = getTickLabelInsets() ;
    double maxWidth = 0 ;
    for (int i = 0 ; i < formatedLabels.length ; i++) {
      g2.setFont(getTickLabelFont(formatedLabels[i])) ;
      Rectangle2D bound = TextUtil.getStringBound(g2, formatedLabels[i]) ;
      if (insets != null) {
        insets.expandRect(bound) ;
      }

      Shape shape = transform.createTransformedShape(bound) ;
      bound = shape.getBounds() ;

      if (RectangleEdge.isLeftOrRight(edge)) {
        maxWidth = (bound.getWidth() > maxWidth) ? bound.getWidth() : maxWidth ;
      } else if (RectangleEdge.isTopOrBottom(edge)) {
        maxWidth = (bound.getHeight() > maxWidth) ? bound.getHeight() : maxWidth ;
      }
    }

    state.restoreOldState() ;

    if (maxWidth > areaWidth) {
      return areaWidth ;
    } else {
      return maxWidth ;
    }
  }

  /**
   * <zh>ظʽʾĿ̶ֵ.</zh>
   * <en>Returns the formated tick labels.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param maxAxisArea Rectangle2D <zh>,϶СplotArea (Ϊnull).</zh>
   * <en>The maximum area within which the axis should be drawn (null not permitted).</en>
   * @param edge RectangleEdge <zh>λ (Ϊnull).</zh>
   * <en>The axis position (null not permitted).</en>
   *
   * @return String[] <zh>̶ֵ.</zh>
   * <en>The array of tick labels.</en>
   */
  protected String[] getFormatedTickLabels(Graphics2D g2, Rectangle2D maxAxisArea,
                                           RectangleEdge edge) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (maxAxisArea == null) {
      throw new IllegalArgumentException("Null 'maxAxisArea' argument.") ;
    }
    if (edge == null) {
      throw new IllegalArgumentException("Null 'edge' argument.") ;
    }

    calculateLabels(g2, maxAxisArea, edge) ;
    return this.tickLabels ;
  }

  /**
   * <zh>ʵʵıǩֵ.</zh>
   * <en>Calculate the </en>
   * @param g2 Graphics2D
   * @param axisArea Rectangle2D
   * @param edge RectangleEdge
   */
  protected void calculateLabels(Graphics2D g2, Rectangle2D axisArea, RectangleEdge edge) {
    //ǩֵ
    Graphics2DState graphicsState = new Graphics2DState(g2) ;
    graphicsState.saveOldState() ;
    g2.setFont(getTickLabelFont()) ;

    String str = "123afgj" ;
    double textHeight = TextUtil.getStringBound(g2, str).getHeight() ;
    int estimateCount = 0 ;

    AffineTransform transform = AffineTransform.getRotateInstance(getTickLabelAngle()) ;
    if (RectangleEdge.isLeftOrRight(edge)) {
      double axisDrawLabelWidth = axisArea.getHeight() * (1 - getLowerMargin() - getUpperMargin()) ;
      estimateCount = (int) (axisDrawLabelWidth / textHeight) ;
      estimateCount = (estimateCount <= 0) ? 1 : estimateCount ;

      this.tickLabels = getNumberLabels(estimateCount) ;
      double maxWidth = 1.0 ;
      for (int i = 0 ; i < this.tickLabels.length ; i++) {
        if (!StringUtil.isEmpty(this.tickLabels[i])) {
          Rectangle2D textBound = TextUtil.getStringBound(g2, this.tickLabels[i]) ;
          double tickLabelHeight = transform.createTransformedShape(textBound).getBounds2D().
                                   getHeight() ;
          maxWidth = Math.max(maxWidth, tickLabelHeight) ;
        }
      }
      estimateCount = (int) (axisDrawLabelWidth / maxWidth) ;
    } else if (RectangleEdge.isTopOrBottom(edge)) {
      double axisDrawLabelWidth = axisArea.getWidth() * (1 - getLowerMargin() - getUpperMargin()) ;
      estimateCount = (int) (axisDrawLabelWidth / textHeight) ;
      estimateCount = (estimateCount <= 0) ? 1 : estimateCount ;

      this.tickLabels = getNumberLabels(estimateCount) ;
      double maxWidth = 1.0 ;
      for (int i = 0 ; i < this.tickLabels.length ; i++) {
        if (!StringUtil.isEmpty(this.tickLabels[i])) {
          Rectangle2D textBound = TextUtil.getStringBound(g2, this.tickLabels[i]) ;
          double tickLabelWidth = transform.createTransformedShape(textBound).getBounds2D().
                                  getWidth() + 0.5 ;
          maxWidth = Math.max(maxWidth, tickLabelWidth) ;
        }
      }
      estimateCount = (int) (axisDrawLabelWidth / maxWidth) ;
    }

    if (estimateCount <= 0) {
      estimateCount = 1 ;
    }

    this.tickLabels = getNumberLabels(estimateCount) ;
    if (estimateCount > 2 && this.tickLabels.length >= estimateCount * 0.75) { //ǩ̫
      this.tickLabels = getNumberLabels((estimateCount <= 2) ? estimateCount
                                        : (int) (estimateCount * 0.75)) ;
    }

    //ȥظĿ̶ֵ
    if (this.tickLabels != null) {
      int removeCnt = 0 ;
      int len = this.tickLabels.length ;
      int[] flagArr = new int[len] ;
      for (int i = 0 ; i < len ; i++) {
        if (this.zoomDatasetRange != null
            && !this.zoomDatasetRange.contains(convertNumberTick(this.numberTicks[i]))) { //zoomDatasetRangeΧ֮
          removeCnt++ ;
          flagArr[i] = 1 ;
          continue ;
        }

        for (int j = i + 1 ; j < len ; j++) {
          if (this.tickLabels[i] == null || this.tickLabels[i].equals(this.tickLabels[j])) {
            removeCnt++ ;
            flagArr[j] = 1 ;
            break ;
          }
        }
      }

      List numbers = new ArrayList() ; //Ҫ¼ֵ
      if (removeCnt > 0) {
        List newTickLabels = new ArrayList() ;
        for (int i = 0 ; i < this.tickLabels.length ; i++) {
          if (flagArr[i] != 1) {
            newTickLabels.add(this.tickLabels[i]) ;
            numbers.add(this.numberTicks[i]) ;
          }
        }
        this.tickLabels = (String[]) newTickLabels.toArray(new String[0]) ;
      } else {
        numbers = Arrays.asList(this.numberTicks) ;
      }

      this.valueTicks = new double[numbers.size()] ;
      for (int i = 0 ; i < numbers.size() ; i++) { //̶ȶӦֵ
        String number = (String) numbers.get(i) ;
        this.valueTicks[i] = convertNumberTick(number) ;
      }
    }

    //״̬,ѲݷΧڵĿ̶ȥ
    if (this.zoomDatasetRange != null) {
      double lower = this.zoomDatasetRange.getLowerBound() ;
      double upper = this.zoomDatasetRange.getUpperBound() ;
      List newNumberTicks = new ArrayList() ;
      for (int i = 0 ; i < this.numberTicks.length ; i++) {
        double v = convertNumberTick(this.numberTicks[i]) ;
        if (this.zoomDatasetRange.contains(v) && v != lower && v != upper) { //zoomDatasetRangeΧ֮
          newNumberTicks.add(this.numberTicks[i]) ;
        }
      }

      if (isLogTickUnit()) {
        LogTickUnit logUnit = (LogTickUnit)this.tickUnit ;
        String lowerUnit = String.valueOf(MathUtil.sign(lower) * Math.log(lower)
                                          / logUnit.getBaseLogValue()) ;
        newNumberTicks.add(0, logUnit.getLogarithmBase() + "^" + lowerUnit) ;
        String upperUnit = String.valueOf(MathUtil.sign(upper) * Math.log(upper)
                                          / logUnit.getBaseLogValue()) ;
        newNumberTicks.add(logUnit.getLogarithmBase() + "^" + upperUnit) ;
      } else {
        newNumberTicks.add(0, Double.toString(lower)) ;
        newNumberTicks.add(Double.toString(upper)) ;
      }
      this.numberTicks = (String[]) newNumberTicks.toArray(new String[0]) ;
    }

    graphicsState.restoreOldState() ;
  }

  /**
   * <zh>ֱǩֵ.</zh>
   * <en>Calculate the number labels.</en>
   *
   * @param labelCount int <zh>ǩĸ.</zh>
   * <en>The number of labels</en>
   * @return String[]
   */
  protected String[] getNumberLabels(int labelCount) {
    IMatrixDataset dataset = getDataset() ;

    double minValue = 0, maxValue = 0 ; //С,ֵ
    if (dataset.isEmpty()) { //ݼΪ
      NumberRange range = this.zoomDatasetRange ;
      if (range == null) {
        range = getEmptyDatasetRange() ;
      }
      minValue = range.getLowerBound() ;
      maxValue = range.getUpperBound() ;
    } else {
      NumberRange range = this.zoomDatasetRange ;
      if (range == null) {
        range = getDatasetRange() ;
      }
      if (range != null) {
        minValue = range.getLowerBound() ;
        maxValue = range.getUpperBound() ;
      }
    }

    this.numberTicks = this.tickUnit.getTickUnits(minValue, maxValue, labelCount,
                                                  isRangeIncludeZero()) ;
    String[] tickUnits = new String[this.numberTicks.length] ;
    System.arraycopy(this.numberTicks, 0, tickUnits, 0, this.numberTicks.length) ;

    IFormatter formatter = getTickValueFormatter() ;
    if (formatter != null) {
      for (int i = 0 ; i < tickUnits.length ; i++) {
        tickUnits[i] = formatter.format(this.numberTicks[i], i) ;
      }
    } else {
      if (this.tickUnit instanceof DateTickUnit) {
        DateTickUnit dateUnit = (DateTickUnit)this.tickUnit ;
        dateUnit.setTimeZone(this.timeZone) ;
        DateTickUnitType unitType = dateUnit.getUnitType() ;
        if (unitType == null) {
          unitType = DateTickUnit.calculateDateTickUnitType(minValue, maxValue, tickUnits.length) ;
        }
        DateFormat dateFormatter = dateUnit.getDateFormatter() ;
        if (dateFormatter == null) {
          dateFormatter = DateTickUnit.getDataFormatter(unitType) ;
        }

        for (int i = 0 ; i < tickUnits.length ; i++) {
          long value = Long.parseLong(this.numberTicks[i], 10) ;
          Date valueDate = new Date(value) ;
          tickUnits[i] = dateFormatter.format(valueDate) ;
        }
      }
    }

    return (tickUnits == null) ? new String[0] : tickUnits ;
  }

  /**
   * <zh>õݼݷΧ.</zh>
   * <en>Returns the value range of a dataset.</en>
   *
   * @return NumberRange <zh>ݷΧ (Ϊnull).</zh>
   * <en>The value range of a dataset (never null).</en>
   */
  public NumberRange getDatasetRange() {
    if (getUserDefineDatasetRange() != null) {
      return getUserDefineDatasetRange() ;
    }

    double minValue = Double.NaN ;
    double maxValue = Double.NaN ;
    IMatrixDataset dataset = getDataset() ;

    if (dataset != null) {
      int rows = dataset.getRowCount() ;

      for (int i = 0 ; i < rows ; i++) {
        int currRowCols = dataset.getColumnCount(i) ;
        for (int j = 0 ; j < currRowCols ; j++) {
          Object data = dataset.getData(i, j) ;
          if (data != null) {
            if (!(data instanceof NumberChartData)) {
              throw new InvalidChartDataException(
                "Invalid chart data type,musts be 'NumberChartData'.") ;
            } else {
              NumberChartData numberData = (NumberChartData) data ;
              double value = numberData.getValue() ;
              if (!Double.isNaN(value)) {
                if (Double.isNaN(minValue)) {
                  minValue = value ;
                } else {
                  minValue = Math.min(minValue, value) ;
                }

                if (Double.isNaN(maxValue)) {
                  maxValue = value ;
                } else {
                  maxValue = Math.max(maxValue, value) ;
                }
              }
            }
          }
        }
      }
    }

    if (Double.isNaN(minValue)) {
      minValue = 0 ;
    }
    if (Double.isNaN(maxValue)) {
      maxValue = 0 ;
    }

    return new NumberRange(minValue, maxValue) ;
  }

  /**
   * <zh>õݼݷΧ.</zh>
   * <en>Returns the value range of a dataset.</en>
   *
   * @return NumberRange <zh>ݷΧ (Ϊnull).</zh>
   * <en>The value range of a dataset (never null).</en>
   */
  public NumberRange getTickRange() {
    if (this.zoomDatasetRange != null) {
      return this.zoomDatasetRange ;
    }

    if (this.valueTicks == null || this.valueTicks.length == 0) {
      return null ;
    }

    if (this.valueTicks.length == 1) {
      return new NumberRange(this.valueTicks[0], this.valueTicks[0]) ;
    }

    return new NumberRange(this.valueTicks[0], this.valueTicks[this.valueTicks.length - 1]) ;
  }

  /**
   * <zh>һRectangleTextChartShape.</zh>
   * <en>Create a RectangleTextChartShape object.</en>
   *
   * @param tickLabelDrawRect Rectangle2D <zh>̶ֵİΧ.</zh>
   * <en>The tick label bound.</en>
   * @param tickLabel String <zh>̶ֵ.</zh>
   * <en>The tick label.</en>
   *
   * @return IChartShape <zh>ͼζ.</zh>
   * <en>A {@link IChartShape}.</en>
   */
  protected IChartShape createChartShape(Rectangle2D tickLabelDrawRect, String tickLabel) {
    RectangleTextChartShape shape = new RectangleTextChartShape(tickLabelDrawRect, tickLabel) ;
    shape.setRotate(AngleUtil.conversionAngle(getTickLabelAngle())) ;
    shape.setTextFont(getTickLabelFont(tickLabel)) ;
    shape.setTextPaint(getTickLabelPaint(tickLabel)) ;
    shape.setMargin(getTickLabelInsets()) ;
    shape.setTextMaxShowLine(1) ;
    shape.setForceShowAllText(getTickLabelLines() > 1) ;
    shape.setDrawBorder(false) ;
    shape.setTooltipText(tickLabel) ;

    shape.setTextHorizontalAlign(getTickLabelHorizontalAlign()) ;
    shape.setTextVerticalAlign(getTickLabelVerticalAlign()) ;

    return shape ;
  }

  /**
   * <zh>,IAxis.draw()֮.</zh>
   * <en>Draw the grid line,only call after IAxis.draw().</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param dataArea Rectangle2D <zh>ͼdata() (Ϊnull).</zh>
   * <en>The area within which the data should be drawn (null not permitted).</en>
   * @param axisEdge RectangleEdge <zh>λ (Ϊnull).</zh>
   * <en>The axis position (null not permitted).</en>
   * @param shapeCollection RenderShapeCollection <zh>ͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param stroke Stroke <zh>stroke (Ϊnull).</zh>
   * <en>The grid line stroke.</en>
   * @param paint Paint <zh>paint (Ϊnull).</zh>
   * <en>The grid line paint (null not permitted).</en>
   */
  public void drawGridLine(Graphics2D g2, Rectangle2D dataArea, RectangleEdge axisEdge,
                           RenderShapeCollection shapeCollection, Stroke stroke, Paint paint) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (axisEdge == null) {
      throw new IllegalArgumentException("Null 'axisEdge' argument.") ;
    }
    if (shapeCollection == null) {
      throw new IllegalArgumentException("Null 'shapeCollection' argument.") ;
    }
    if (stroke == null) {
      throw new IllegalArgumentException("Null 'stroke' argument.") ;
    }
    if (paint == null) {
      throw new IllegalArgumentException("Null 'paint' argument.") ;
    }

    int tickMarkCount = getTickMarkCount() ;
    if (tickMarkCount == 0) {
      return ;
    }

    if (isGridLineVisible()) {
      boolean offsetValueLine = isOffsetValueLine() ;
      int offsetTickMarkCount = offsetValueLine ? ((tickMarkCount > 1) ? -1 : 0) : 0 ; //Ҫƫƿ̶ֵ,̶ҲҪ1
      if (RectangleEdge.isTopOrBottom(axisEdge)) { //top or bottom axis
        double startX = dataArea.getX() + dataArea.getWidth() * getLowerMargin() ;
        if (tickMarkCount == 1 && this.zoomDatasetRange == null) {
          double endX = dataArea.getMaxX() - dataArea.getWidth() * getUpperMargin() ;
          double centerX = (startX + endX) * 0.5 ;

          LineChartShape shape = new LineChartShape(centerX, dataArea.getMinY(), centerX,
                                                    dataArea.getMaxY()) ;
          shape.setDrawBorder(true) ;
          shape.setBorderStroke(stroke) ;
          shape.setBorderPaint(paint) ;
          shapeCollection.addValueGridLineShape(shape) ; //һͼ
        } else {
          calculateXTickPositions(dataArea, axisEdge) ;
          double categoryWidth = 0 ;
          if (this.xTickPositions.length > 1) {
            categoryWidth = Math.abs(this.xTickPositions[0] - this.xTickPositions[1]) ;
          }
          double offsetLength = (offsetValueLine ? 0.5 * categoryWidth : 0) ;

          for (int i = 0 ; i < tickMarkCount + offsetTickMarkCount ; i++) {
            double x = this.xTickPositions[i] + offsetLength ;
            if (!(x >= dataArea.getMinX() && x <= dataArea.getMaxX()) || x == dataArea.getMinX()) { //x᷶Χ֮ڻغʱͺ
              continue ;
            }

            LineChartShape shape = new LineChartShape(x, dataArea.getMinY(), x, dataArea.getMaxY()) ;
            shape.setDrawBorder(true) ;
            shape.setBorderStroke(stroke) ;
            shape.setBorderPaint(paint) ;
            shapeCollection.addValueGridLineShape(shape) ; //һͼ
          }
        }
      } else if (RectangleEdge.isLeftOrRight(axisEdge)) { //left or right axis
        double startY = dataArea.getY() + dataArea.getHeight() * getUpperMargin() ;
        if (tickMarkCount == 1 && this.zoomDatasetRange == null) {
          double endY = dataArea.getMaxY() - dataArea.getHeight() * getLowerMargin() ;
          double centerY = (startY + endY) * 0.5 ;

          LineChartShape shape = new LineChartShape(dataArea.getMinX(), centerY,
                                                    dataArea.getMaxX(), centerY) ;
          shape.setDrawBorder(true) ;
          shape.setBorderStroke(stroke) ;
          shape.setBorderPaint(paint) ;
          shapeCollection.addValueGridLineShape(shape) ; //һͼ
        } else {
          calculateYTickPositions(dataArea, axisEdge) ;
          double categoryWidth = 0 ;
          if (this.yTickPositions.length > 1) {
            categoryWidth = Math.abs(this.yTickPositions[0] - this.yTickPositions[1]) ;
          }
          double offsetLength = (offsetValueLine ? 0.5 * categoryWidth : 0) ;

          for (int i = 0 ; i < tickMarkCount + offsetTickMarkCount ; i++) {
            double y = this.yTickPositions[i] + offsetLength ;
            if (!(y >= dataArea.getMinY() && y <= dataArea.getMaxY()) || y == dataArea.getMaxY()) { //y᷶Χ֮ڻغʱͺ
              continue ;
            }

            LineChartShape shape = new LineChartShape(dataArea.getMinX(), y, dataArea.getMaxX(), y) ;

            shape.setDrawBorder(true) ;
            shape.setBorderStroke(stroke) ;
            shape.setBorderPaint(paint) ;
            shapeCollection.addValueGridLineShape(shape) ; //һͼ
          }
        }
      }
    }

    if (isGridBandVisible() && tickMarkCount > 1) {
      boolean offsetValueLine = isOffsetValueLine() ;
      int offsetTickMarkCount = offsetValueLine ? ((tickMarkCount > 1) ? -1 : 0) : 0 ; //Ҫƫƿ̶ֵ,̶ҲҪ1
      if (RectangleEdge.isTopOrBottom(axisEdge)) { //top or bottom axis
        double startX = dataArea.getX() + dataArea.getWidth() * getLowerMargin() ;
        double categoryWidth = dataArea.getWidth() * (1 - getLowerMargin() - getUpperMargin())
                               / ((tickMarkCount <= 1) ? 1 : (tickMarkCount - 1)) ;
        double offsetLength = (offsetValueLine ? 0.5 * categoryWidth : 0) ;

        for (int i = 0 ; i < tickMarkCount + offsetTickMarkCount ; i++) {
          double x1 = Math.max(dataArea.getMinX(), startX + (i - 0.5) * categoryWidth) ;
          double x2 = Math.min(dataArea.getMaxX(), startX + (i + 0.5) * categoryWidth) ;
          Rectangle2D drawArea = new Rectangle2D.Double(x1 + offsetLength, dataArea.getMinY(),
                                                        x2 - x1, dataArea.getHeight()) ;
          RectangleTextChartShape shape = new RectangleTextChartShape(drawArea, null) ;
          shape.setDrawBorder(false) ;
          shape.setFillShape(true) ;
          shape.setFillPaint((i % 2 == 0) ? getGridBandPaint() : getGridBandAlternatePaint()) ;
          shapeCollection.addValueGridBandShape(shape) ; //һͼ
        }
      } else if (RectangleEdge.isLeftOrRight(axisEdge)) { //left or right axis
        double startY = dataArea.getY() + dataArea.getHeight() * getUpperMargin() ;
        double categoryWidth = dataArea.getHeight() * (1 - getLowerMargin() - getUpperMargin())
                               / ((tickMarkCount <= 1) ? 1 : (tickMarkCount - 1)) ;
        double offsetLength = (offsetValueLine ? 0.5 * categoryWidth : 0) ;

        for (int i = 0 ; i < tickMarkCount + offsetTickMarkCount ; i++) {
          double y1 = Math.max(dataArea.getMinY(), startY + (i - 0.5) * categoryWidth) ;
          double y2 = Math.min(dataArea.getMaxY(), startY + (i + 0.5) * categoryWidth) ;
          Rectangle2D drawArea = new Rectangle2D.Double(dataArea.getMinX(), y1 + offsetLength,
                                                        dataArea.getWidth(), y2 - y1) ;
          RectangleTextChartShape shape = new RectangleTextChartShape(drawArea, null) ;
          shape.setDrawBorder(false) ;
          shape.setFillShape(true) ;
          shape.setFillPaint((i % 2 == 0) ? getGridBandPaint() : getGridBandAlternatePaint()) ;
          shapeCollection.addValueGridBandShape(shape) ; //һͼ
        }
      }
    }

  }

  /**
   * <zh>ĳֵӦλ,IAxis.draw()֮.</zh>
   * <en>Converts a data value to a coordinate in Java2D space, assuming that the
   * axis runs along one edge of the specified dataArea.</en>
   *
   * @param dataArea Rectangle2D <zh>ͼdata() (Ϊnull).</zh>
   * <en>The area within which the data should be drawn (null not permitted).</en>
   * @param edge RectangleEdge <zh>λ (Ϊnull).</zh>
   * <en>The axis position (null not permitted).</en>
   * @param value double <zh>ֵ.</zh>
   * <en>The data value.</en>
   * @param forceInDataArea boolean <zh>ǷǿdataArea֮.</zh>
   * <en>Whether or not the return value force in the region dataArea.</en>
   *
   * @return double <zh>λ.</zh>
   * <en>The Java2D coordinate.</en>
   */
  public double valueToJava2D(Rectangle2D dataArea, RectangleEdge edge, double value,
                              boolean forceInDataArea) {
    return valueToJava2D(dataArea, edge, value, forceInDataArea, true) ;
  }

  /**
   * <zh>ĳֵӦλ,IAxis.draw()֮.</zh>
   * <en>Converts a data value to a coordinate in Java2D space, assuming that the
   * axis runs along one edge of the specified dataArea.</en>
   *
   * @param dataArea Rectangle2D <zh>ͼdata() (Ϊnull).</zh>
   * <en>The area within which the data should be drawn (null not permitted).</en>
   * @param edge RectangleEdge <zh>λ (Ϊnull).</zh>
   * <en>The axis position (null not permitted).</en>
   * @param value double <zh>ֵ.</zh>
   * <en>The data value.</en>
   * @param forceInDataArea boolean <zh>ǷǿdataArea֮.</zh>
   * <en>Whether or not the return value force in the region dataArea.</en>
   * @param expandAxisRange boolean <zh>ǷչķΧ.</zh>
   * <en>Whether or not expand the range of axis.</en>
   *
   * @return double <zh>λ.</zh>
   * <en>The Java2D coordinate.</en>
   */
  public double valueToJava2D(Rectangle2D dataArea, RectangleEdge edge, double value,
                              boolean forceInDataArea, boolean expandAxisRange) {
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (edge == null) {
      throw new IllegalArgumentException("Null 'edge' argument.") ;
    }

    double axisValue = value ;
    if (this.numberTicks == null || this.numberTicks.length == 0 || Double.isNaN(axisValue)) {
      if (this.numberTicks == null) {
        this.numberTicks = new String[0] ;
      }
      return AbstractValueAxis.INVALID_COORDINATE_VALUE ;
    }

    //̶ֵСֵ
    double minTickValue = Double.NaN ;
    double maxTickValue = Double.NaN ;
    if (isLogTickUnit()) {
      LogTickUnit logUnit = (LogTickUnit)this.tickUnit ;
      axisValue = MathUtil.sign(axisValue) * Math.log(Math.abs(axisValue))
                  / logUnit.getBaseLogValue() ;
      if (Double.isNaN(axisValue)) {
        axisValue = 0 ;
      }
      String[] logTickLabels = logUnit.getExponentLabels() ;
      if (this.zoomDatasetRange != null) {
        double v = this.zoomDatasetRange.getLowerBound() ;
        minTickValue = MathUtil.sign(v) * Math.log(Math.abs(v)) / logUnit.getBaseLogValue() ;

        v = this.zoomDatasetRange.getUpperBound() ;
        maxTickValue = MathUtil.sign(v) * Math.log(Math.abs(v)) / logUnit.getBaseLogValue() ;
      } else {
        if (logTickLabels.length > 0) {
          minTickValue = Double.parseDouble(logTickLabels[0]) ;
          maxTickValue = Double.parseDouble(logTickLabels[this.numberTicks.length - 1]) ;
        } else {
          minTickValue = 0 ;
          maxTickValue = 0 ;
        }
      }
    } else {
      if (this.zoomDatasetRange != null) {
        minTickValue = this.zoomDatasetRange.getLowerBound() ;
        maxTickValue = this.zoomDatasetRange.getUpperBound() ;
      } else {
        if (this.numberTicks.length > 0) {
          minTickValue = convertNumberTick(this.numberTicks[0]) ;
          maxTickValue = convertNumberTick(this.numberTicks[this.numberTicks.length - 1]) ;
        } else {
          minTickValue = 0 ;
          maxTickValue = 0 ;
        }
      }
    }

    double lMargin = (this.zoomDatasetRange == null) ? getLowerMargin() : 0 ;
    double uMargin = (this.zoomDatasetRange == null) ? getUpperMargin() : 0 ;
    if (RectangleEdge.isTopOrBottom(edge)) { //top or bottom axis
      double startX = dataArea.getX() + dataArea.getWidth() * lMargin ;
      double endX = dataArea.getMaxX() - dataArea.getWidth() * uMargin ;

      if (axisValue >= minTickValue && axisValue <= maxTickValue) { //Сֵ֮
        double lineValueLen = 0 ; //ֵ߾Сֵľ
        if (minTickValue == maxTickValue) {
          return (startX + endX) * 0.5 ;
        } else {
          lineValueLen = getDataPosition(startX, endX, axisValue) ;
        }

        double valueLineX = 0 ; //ֵߵĿʼλ
        if (isInverted()) {
          valueLineX = endX - lineValueLen ;
        } else {
          valueLineX = startX + lineValueLen ;
        }

        return valueLineX ;
      } else { //֮
        if (forceInDataArea) { //ǿ֮
          if (isInverted()) {
            if (Math.abs(axisValue - minTickValue) < Math.abs(axisValue - maxTickValue)) { //Сֵ
              return endX ;
            } else { //ֵ
              return startX ;
            }
          } else {
            if (Math.abs(axisValue - maxTickValue) < Math.abs(axisValue - minTickValue)) { //ֵ
              return endX ;
            } else { //Сֵһ
              return startX ;
            }
          }
        } else { //֮
          if (expandAxisRange) {
            if (isInverted()) { //ݷת
              if (Math.abs(axisValue - minTickValue) < Math.abs(axisValue - maxTickValue)) { //Сֵ
                if (minTickValue == maxTickValue) { //ֵ
                  return endX ;
                } else {
                  return endX
                    + ((axisValue - minTickValue) * (endX - startX) / (minTickValue - maxTickValue)) ;
                }
              } else { //ֵ
                if (minTickValue == maxTickValue) { //ֵ
                  return startX ;
                } else {
                  return startX
                    - ((maxTickValue - axisValue) * (endX - startX) / (minTickValue - maxTickValue)) ;
                }
              }
            } else {
              if (Math.abs(axisValue - minTickValue) < Math.abs(axisValue - maxTickValue)) { //Сֵ
                if (minTickValue == maxTickValue) { //ֵ
                  return startX ;
                } else {
                  return startX
                    - ((minTickValue - axisValue) * (endX - startX) / (maxTickValue - minTickValue)) ;
                }
              } else { //ֵ
                if (minTickValue == maxTickValue) { //ֵ
                  return endX ;
                } else {
                  return endX
                    + ((axisValue - maxTickValue) * (endX - startX) / (maxTickValue - minTickValue)) ;
                }
              }
            }
          } else {
            return AbstractValueAxis.INVALID_COORDINATE_VALUE ;
          }
        }
      }
    } else if (RectangleEdge.isLeftOrRight(edge)) { //left or right axis
      double startY = dataArea.getY() + dataArea.getHeight() * uMargin ;
      double endY = dataArea.getMaxY() - dataArea.getHeight() * lMargin ;

      if (axisValue >= minTickValue && axisValue <= maxTickValue) { //Сֵ֮
        double lineValueLen = 0 ; //ֵ߾Сֵľ
        if (minTickValue == maxTickValue) {
          return (startY + endY) * 0.5 ;
        } else {
          lineValueLen = getDataPosition(startY, endY, axisValue) ;
        }

        double valueLineY = 0 ; //ֵߵĿʼλ
        if (isInverted()) {
          valueLineY = startY + lineValueLen ;
        } else {
          valueLineY = endY - lineValueLen ;
        }

        return valueLineY ;
      } else { //֮
        if (forceInDataArea) { //ǿ֮
          if (isInverted()) {
            if (Math.abs(axisValue - minTickValue) < Math.abs(axisValue - maxTickValue)) { //Сֵ
              return startY ;
            } else { //ֵ
              return endY ;
            }
          } else {
            if (Math.abs(axisValue - maxTickValue) < Math.abs(axisValue - minTickValue)) { //ֵ
              return startY ;
            } else { //Сֵһ
              return endY ;
            }
          }
        } else { //֮
          if (expandAxisRange) {
            if (isInverted()) { //ݷת
              if (Math.abs(axisValue - minTickValue) < Math.abs(axisValue - maxTickValue)) { //Сֵ
                if (minTickValue == maxTickValue) { //ֵ
                  return startY ;
                } else {
                  return startY
                    - ((minTickValue - axisValue) * (endY - startY) / (maxTickValue - minTickValue)) ;
                }
              } else { //ֵ
                if (minTickValue == maxTickValue) { //ֵ
                  return endY ;
                } else {
                  return endY
                    + ((axisValue - maxTickValue) * (endY - startY) / (maxTickValue - minTickValue)) ;
                }
              }
            } else {
              if (Math.abs(axisValue - minTickValue) < Math.abs(axisValue - maxTickValue)) { //Сֵ
                if (minTickValue == maxTickValue) { //ֵ
                  return endY ;
                } else {
                  return endY
                    + ((minTickValue - axisValue) * (endY - startY) / (maxTickValue - minTickValue)) ;
                }
              } else { //ֵ
                if (minTickValue == maxTickValue) { //ֵ
                  return startY ;
                } else {
                  return startY
                    - ((axisValue - maxTickValue) * (endY - startY) / (maxTickValue - minTickValue)) ;
                }
              }
            }
          } else {
            return AbstractValueAxis.INVALID_COORDINATE_VALUE ;
          }
        }
      }
    }

    //ֵСֵֵ֮
    return AbstractValueAxis.INVALID_COORDINATE_VALUE ;
  }

  /**
   * <zh>ݷΧǷ0.</zh>
   * <en>Returns whether or not include zero of the dataset.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  protected boolean isRangeIncludeZero() {
    return (this.zoomDatasetRange == null) ? isAxisRangeIncludeZero() : false ;
  }

  /**
   * <zh>valueֵڵľλ,λλstartPosendPosֵ֮.</zh>
   * <en>Calculate the position by the value,the position between startPos and endPos.</en>
   *
   * @param startPos double <zh>ʼλ.</zh>
   * <en>The start position.</en>
   * @param endPos double <zh>λ.</zh>
   * <en>The end position.</en>
   * @param value double <zh>Ҫλõֵ.</zh>
   * <en>The value.</en>
   *
   * @return double <zh>λõֵ.</zh>
   * <en>The value position</en>
   */
  private double getDataPosition(double startPos, double endPos, double value) {
    if (this.numberTicks == null || this.numberTicks.length < 2) {
      return startPos ;
    }

    //ǵʾڿ̶ʱÿ»ʱܲͬ,һֵӦλҪֶμ
    int tickCount = this.numberTicks.length ; //̶
    double length = Math.abs(endPos - startPos) ; //п̶ȵĳ
    double oneTickLength = (tickCount <= 1) ? 0 : (length / (tickCount - 1)) ; //ÿ̶ȳ

    double[] tickValues = new double[tickCount] ;
    if (isLogTickUnit()) {
      String[] ticks = ((LogTickUnit)this.tickUnit).getExponentLabels() ;
      for (int i = 0 ; i < tickCount ; i++) { //תΪֵ͵
        tickValues[i] = Double.parseDouble(ticks[i]) ;
      }
    } else {
      for (int i = 0 ; i < tickCount ; i++) { //תΪֵ͵
        tickValues[i] = convertNumberTick(this.numberTicks[i]) ;
      }
    }

    //λ
    for (int i = 0 ; i < tickCount - 1 ; i++) {
      if (value >= tickValues[i] && value <= tickValues[i + 1]) {
        if (tickValues[i + 1] == tickValues[i]) {
          return i * oneTickLength ;
        } else {
          //̶ֵ֮ĳ
          double valueLen = value - tickValues[0] ;
          double maxValueLen = tickValues[0] - tickValues[tickCount - 1] ;

          return Math.abs(valueLen / maxValueLen * length) ;

        }
      }
    }

    return startPos ;
  }

  /**
   * <zh>ǷҪƫ.</zh>
   * <en>Returns whether or not offset axis.</en>
   *
   * @return boolean  <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  protected boolean isOffsetValueLine() {
    IRenderWithAxis render = getRender() ;
    boolean offsetValueLine = false ;
    if (render != null) {
      ValueAxisOffsetType offsetType = render.getValueAxisOffsetType() ;
      if (offsetType != null && offsetType.equals(ValueAxisOffsetType.OFFSET_TYPE_HALF_CATEGORY)) {
        offsetValueLine = true ;
      }
    }

    return offsetValueLine ;
  }

  /**
   * <zh>tickUnitǷLogTickUnit.</zh>
   * <en>Whether or not tickUnit is LogTickUnit.</en>
   * @return boolean
   */
  protected boolean isLogTickUnit() {
    return this.tickUnit != null && this.tickUnit instanceof LogTickUnit ;
  }

  /**
   * <zh>ַ͵Ŀ̶ֵתΪ.</zh>
   * <en>Convert String tick to number.</en>
   *
   * @param tick String
   * @return double
   */
  protected double convertNumberTick(String tick) {
    String number = new String(tick) ;
    int idx = number.indexOf("^") ;
    if (idx >= 0) {
      int minusIdx = number.indexOf("-") ;
      int sign = 1 ;
      if (minusIdx >= 0) {
        sign = -1 ;
        number = number.substring(1) ;
      }

      String value = number.substring(0, idx) ;
      String power = number.substring(idx + 1) ;
      return sign * Math.pow(Double.parseDouble(value), Double.parseDouble(power)) ;
    } else {
      return Double.parseDouble(number) ;
    }
  }

  /**
   * <zh>xTickPositionsֵ.</zh>
   * <en>Calculate the x tick positions.</en>
   *
   * @param axisArea Rectangle2D <zh>.</zh>
   * <en>The axis draw area.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   */
  private void calculateXTickPositions(Rectangle2D axisArea, RectangleEdge edge) {
    if (this.xTickPositions == null) {
      this.xTickPositions = new double[this.valueTicks.length] ;
      for (int i = 0 ; i < this.valueTicks.length ; i++) {
        this.xTickPositions[i] = valueToJava2D(axisArea, edge, this.valueTicks[i], false) ;
      }
    }
  }

  /**
   * <zh>yTickPositionsֵ.</zh>
   * <en>Calculate the y tick positions.</en>
   *
   * @param axisArea Rectangle2D <zh>.</zh>
   * <en>The axis draw area.</en>
   * @param edge RectangleEdge <zh>λ.</zh>
   * <en>The axis position.</en>
   */
  private void calculateYTickPositions(Rectangle2D axisArea, RectangleEdge edge) {
    if (this.yTickPositions == null) {
      this.yTickPositions = new double[this.valueTicks.length] ;
      for (int i = 0 ; i < this.valueTicks.length ; i++) {
        this.yTickPositions[i] = valueToJava2D(axisArea, edge, this.valueTicks[i], false) ;
      }
    }
  }

  /**
   * <zh>¡.</zh>
   * <en>Creates a clone of the axis.</en>
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException if some component of the axis does
   *         not support cloning.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>ԱǷһ.</zh>
   * <en>Tests this axis for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪԵĶ (Ϊnull).</zh>
   * <en>The object (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof NumberValueAxis)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    NumberValueAxis that = (NumberValueAxis) obj ;
    if (!EqualsUtil.equals(this.tickUnit, that.tickUnit)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.tickLabels, that.tickLabels)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.numberTicks, that.numberTicks)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.userDefineDatasetRange, that.userDefineDatasetRange)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.zoomDatasetRange, that.zoomDatasetRange)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code for this object.</en>
   *
   * @return <zh>hashcode.</zh>
   * <en>A hash code.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 29 * result + HashcodeUtil.hashCode(this.tickUnit) ;
    result = 29 * result + HashcodeUtil.hashCode(this.tickLabels) ;
    result = 29 * result + HashcodeUtil.hashCode(this.numberTicks) ;
    result = 29 * result + HashcodeUtil.hashCode(this.userDefineDatasetRange) ;
    result = 29 * result + HashcodeUtil.hashCode(this.zoomDatasetRange) ;

    return result ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return A string representation of the object.
   */
  public String toString() {
    return "NumberValueAxis" ;
  }

}
