package com.ijchart.xychart.axis.value ;

import java.io.Serializable ;

import com.ijchart.xychart.constant.XYValueAxisType ;
import com.ijchart.xychart.data.YIntervalChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.exception.InvalidChartDataException ;
import com.ijchart.xychart.range.NumberRange ;
import com.ijchart.xychart.tickunit.DateTickUnit ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>XY plotʹõֵ,ÿһ{@link YIntervalChartData}.</zh>
 * <en>The number value axis used for XY plot,every data is {@link YIntervalChartData}.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class YIntervalValueAxis extends AbstractNumberValueAxisWithType implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -3658518803469333368L ;

  /**
   * <zh>ݼ.</zh>
   * <en>Constructs an axis using a dataset.</en>
   *
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   * @param valueAxisType XYValueAxisType <zh>value (Ϊnull).</zh>
   * <en>The type of value axis (null not permitted).</en>
   * @param isDateAxis boolean <zh>Ƿ͵.</zh>
   * <en>Whether or not is a date axis.</en>
   */
  public YIntervalValueAxis(IMatrixDataset dataset, XYValueAxisType valueAxisType,
                            boolean isDateAxis) {
    this(null, dataset, valueAxisType, isDateAxis) ;
  }

  /**
   * <zh>ǩݼ.</zh>
   * <en>Constructs an axis using label and a dataset.</en>
   *
   * @param label <zh>ǩ.</zh>
   * <en>The axis label (null permitted).</en>
   * @param dataset IMatrixDataset <zh>άݼ (Ϊnull).</zh>
   * <en>The two dimension matrix dataset (null not permitted).</en>
   * @param valueAxisType XYValueAxisType <zh>value (Ϊnull).</zh>
   * <en>The type of value axis (null not permitted).</en>
   * @param isDateAxis boolean <zh>Ƿ͵.</zh>
   * <en>Whether or not is a date axis.</en>
   */
  public YIntervalValueAxis(String label, IMatrixDataset dataset, XYValueAxisType valueAxisType,
                            boolean isDateAxis) {
    super(label, dataset, valueAxisType) ;

    if (isDateAxis) {
      this.tickUnit = new DateTickUnit(null) ;
    }
  }

  /**
   * <zh>õݼݷΧ.</zh>
   * <en>Returns the value range of a dataset.</en>
   *
   * @return NumberRange <zh>ݷΧ (Ϊnull).</zh>
   * <en>The value range of a dataset (never null).</en>
   */
  public NumberRange getDatasetRange() {
    if (getUserDefineDatasetRange() != null) {
      return getUserDefineDatasetRange() ;
    }

    double minValue = Double.NaN ;
    double maxValue = Double.NaN ;
    IMatrixDataset dataset = getDataset() ;

    boolean isX = false, isY = false ; //Ƿxyֵ
    if (getValueAxisType().equals(XYValueAxisType.XY_VALUE_AXIS_X)) {
      isX = true ;
    } else if (getValueAxisType().equals(XYValueAxisType.XY_VALUE_AXIS_Y)) {
      isY = true ;
    }

    if (dataset != null) {
      int rows = dataset.getRowCount() ;
      for (int i = 0 ; i < rows ; i++) {
        int currRowCols = dataset.getColumnCount(i) ;

        for (int j = 0 ; j < currRowCols ; j++) {
          Object data = dataset.getData(i, j) ;
          if (data != null) {
            if (!(data instanceof YIntervalChartData)) {
              throw new InvalidChartDataException(
                "Invalid chart data type,must be 'YIntervalChartData'.") ;
            } else {
              YIntervalChartData yInterval = (YIntervalChartData) data ;

              if (isX) {
                double x = yInterval.getX() ;
                if (!Double.isNaN(x)) {
                  if (Double.isNaN(minValue)) {
                    minValue = x ;
                  } else {
                    minValue = Math.min(minValue, x) ;
                  }

                  if (Double.isNaN(maxValue)) {
                    maxValue = x ;
                  } else {
                    maxValue = Math.max(maxValue, x) ;
                  }
                }
              } else if (isY) {
                double yLow = yInterval.getYLow() ;
                double yHigh = yInterval.getYHigh() ;
                double y = yInterval.getY() ;

                if (!Double.isNaN(y)) {
                  if (Double.isNaN(minValue)) {
                    minValue = y ;
                  } else {
                    minValue = Math.min(minValue, y) ;
                  }

                  if (Double.isNaN(maxValue)) {
                    maxValue = y ;
                  } else {
                    maxValue = Math.max(maxValue, y) ;
                  }
                }

                if (!Double.isNaN(yLow)) {
                  if (Double.isNaN(minValue)) {
                    minValue = yLow ;
                  } else {
                    minValue = Math.min(minValue, yLow) ;
                  }

                  if (Double.isNaN(maxValue)) {
                    maxValue = yLow ;
                  } else {
                    maxValue = Math.max(maxValue, yLow) ;
                  }
                }

                if (!Double.isNaN(yHigh)) {
                  if (Double.isNaN(minValue)) {
                    minValue = yHigh ;
                  } else {
                    minValue = Math.min(minValue, yHigh) ;
                  }

                  if (Double.isNaN(maxValue)) {
                    maxValue = yHigh ;
                  } else {
                    maxValue = Math.max(maxValue, yHigh) ;
                  }
                }

              }
            }
          }
        }
      }
    }

    if (Double.isNaN(minValue)) {
      minValue = 0 ;
    }
    if (Double.isNaN(maxValue)) {
      maxValue = 0 ;
    }

    return new NumberRange(minValue, maxValue) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument;
   * false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!(obj instanceof YIntervalValueAxis)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    return super.hashCode() ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "YIntervalValueAxis" ;
  }

}
