package com.ijchart.xychart.chartinfo ;

import java.awt.Graphics2D ;
import java.awt.Shape ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.IOException ;
import java.io.ObjectInputStream ;
import java.io.ObjectOutputStream ;
import java.io.Serializable ;
import java.util.Arrays ;
import java.util.Collections ;
import java.util.List ;

import javax.swing.event.EventListenerList ;

import com.ijchart.xychart.constant.ChartShapeDrawType ;
import com.ijchart.xychart.event.ChartShapeChangeEvent ;
import com.ijchart.xychart.listener.IChartShapeChangeListener ;
import com.ijchart.xychart.shape.AbstractTextChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.ui.Graphics2DState ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;
import com.ijchart.xychart.util.SerialUtil ;
import com.ijchart.xychart.util.StringUtil ;
import com.ijchart.xychart.web.IAreaFragment ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ṩͼμϵĳʵ.</zh>
 * <en>The abstract class for all chart shape collection class.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public abstract class AbstractShapeCollection implements IChartShapeChangeListener,
  IShapeCollection, Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -456688030173690834L ;

  /**
   * <zh>ͼμڵ.</zh>
   * <en>The draw area for all shapes.</en>
   */
  private transient Rectangle2D drawArea ;

  /**
   * <zh>ǰͼλ״̬.</zh>
   * <en>The graphics state.</en>
   */
  private Graphics2DState state ;

  /**
   * <zh>,ɱл.</zh>
   * <en>The listener collection,not serializable.</en>
   */
  private transient EventListenerList listenerList ;

  /**
   * <zh>ͼεк,һΪ0.</zh>
   * <en>The sequence of shapes,first is 0.</en>
   */
  protected int shapeIdSequence ;

  /**
   * <zh>clipͼ.</zh>
   * <en>The old clip shape.</en>
   */
  private Shape oldClip ;

  /**
   * <zh>.</zh>
   * <en>The constructor.</en>
   */
  public AbstractShapeCollection() {
    this.drawArea = null ;
    this.shapeIdSequence = 0 ;
    this.listenerList = new EventListenerList() ;
  }

  /**
   * <zh>һϵеͼ,ǹͬĳ,ͨ¼,ڴ¼ʱṩٶ,Ҫ
   * еͼжǷڱѡ,ѡͨСѰҵķΧ.</zh>
   * <en>For a series of graphics and their common belonging to a particular region, by recording
   * the region, in dealing with the incident when the mouse to provide faster, no need to
   * determine whether all the graphics are selected by the mouse, optional through this area
   * To narrow the scope of the search.</en>
   *
   * @param drawArea Rectangle2D <zh>ͼμڵ (Ϊnull).</zh>
   * <en>The draw area for all shapes (null not permitted).</en>
   */
  public void setDrawArea(Rectangle2D drawArea) {
    if (drawArea == null) {
      throw new IllegalArgumentException("Null 'drawArea' argument.") ;
    }

    this.drawArea = (Rectangle2D) (drawArea.clone()) ;
  }

  /**
   * <zh>ͼμͬ.</zh>
   * <en>Returns the draw area for all shapes.</en>
   *
   * @return Rectangle2D <zh>ͼμͬ (Ϊnull).</zh>
   * <en>The draw area for all shapes (never null).</en>
   */
  public Rectangle2D getDrawArea() {
    return this.drawArea ;
  }

  /**
   * <zh>ͼкΪ0.</zh>
   * <en>Reset the sequence of shape to 0.</en>
   */
  public void resetShapeIdSequence() {
    this.shapeIdSequence = 0 ;
  }

  /**
   * <zh>ͼμϹͬǷpoint.</zh>
   * <en>Whether or not the draw area contains a point.</en>
   *
   * @param point Point2D <zh>ҪǷĵ (Ϊnull).</zh>
   * <en>The point (null not permitted).</en>
   *
   * @return boolean <zh>򷵻true,򷵻false.</zh>
   * <en>Returns true if the draw area contains point,otherwise return false.</en>
   */
  public boolean isDrawAreaContains(Point2D point) {
    if (point == null) {
      throw new IllegalArgumentException("Null 'point' argument.") ;
    }

    if (this.drawArea == null) {
      return false ;
    }

    return this.drawArea.contains(point) ;
  }

  /**
   * <zh>עͼ{@link IChartShape}仯ļ߶.</zh>
   * <en>Registers an object for notification of changes to the {@link IChartShape}.</en>
   *
   * @param listener <zh>עļ.</zh>
   * <en>the object to register.</en>
   */
  public void addChangeListener(IChartShapeChangeListener listener) {
    this.listenerList.add(IChartShapeChangeListener.class, listener) ;
  }

  /**
   * <zh>ɾעͼ{@link IChartShape}仯߶.</zh>
   * <en>Deregisters an object for notification of changes to the {@link IChartShape}.</en>
   *
   * @param listener <zh>Ҫɾļ߶.</zh>
   * <en>the object to deregister.</en>
   */
  public void removeChangeListener(IChartShapeChangeListener listener) {
    this.listenerList.remove(IChartShapeChangeListener.class, listener) ;
  }

  /**
   * <zh>еͼ{@link IChartShape}仯߶.</zh>
   * <en>Deregisters all object for notification of changes to the {@link IChartShape}.</en>
   */
  public void removeAllChangeListener() {
    Object[] listeners = this.listenerList.getListenerList() ;

    for (int i = listeners.length - 2 ; i >= 0 ; i -= 2) {
      this.listenerList.remove(IChartShapeChangeListener.class,
                               (IChartShapeChangeListener) listeners[i + 1]) ;
    }
  }

  /**
   * <zh>ָlistenerǷ,ҪڵԪ.</zh>
   * <en>Returns true if the specified object is registered with
   * the {@link IChartShape} as a listener.  Most applications won't need to call this
   * method, it exists mainly for use by unit testing code.</en>
   *
   * @param listener <zh>ҪԵļ.</zh>
   * <en>the listener.</en>
   * @return A boolean.
   */
  public boolean hasListener(IChartShapeChangeListener listener) {
    return Arrays.asList(this.listenerList.getListenerList()).contains(listener) ;
  }

  /**
   * <zh>֪ͨעļͼη˱仯.</zh>
   * <en>Notifies all registered listeners that the chart shape has changed.</en>
   *
   * @param event <zh>¼.</zh>
   * <en>contains information about the event that triggered the notification.</en>
   */
  protected void notifyListeners(ChartShapeChangeEvent event) {
    Object[] listeners = this.listenerList.getListenerList() ;
    for (int i = listeners.length - 2 ; i >= 0 ; i -= 2) {
      if (listeners[i] == IChartShapeChangeListener.class) {
        ((IChartShapeChangeListener) listeners[i + 1]).chartShapeChanged(event) ;
      }
    }
  }

  /**
   * <zh>ͨмͼη˱仯.</zh>
   * <en>Notifies all registered listeners that the chart shape has changed.</en>
   *
   * @param shape IChartShape <zh>ͼζ.</zh>
   * <en>The chart shape object.</en>
   */
  protected void fireChartShapeChanged(IChartShape shape) {
    notifyListeners(new ChartShapeChangeEvent(this, shape)) ;
  }

  /**
   * <zh>յͼζ仯֪ͨ.</zh>
   * <en>Receives notification of an {@link com.ijchart.xychart.shape.IChartShape} change event.</en>
   *
   * @param event ChartShapeChangeEvent <zh>ͼζ仯Ķ</zh>
   * <en>Information about the event.</en>
   */
  public void chartShapeChanged(ChartShapeChangeEvent event) {
    notifyListeners(event) ;
  }

  /**
   * <zh>ƶͼ(axisLine3DShape)ԭ㵽(x,y).</zh>
   * <en>Translates the origin of the all shapes(not include axisLine3DShape) in this chart to the
   * point (x,&nbsp;y) in the current coordinate system. </en>
   *
   * @param  x double <zh>x.</zh>
   * <en>The x coordinate.</en>
   * @param  y double <zh>y.</zh>
   * <en>The y coordinate.</en>
   */
  public void translate(double x, double y) {
    if (x == 0 && y == 0) {
      return ;
    }

    if (this.drawArea != null) {
      this.drawArea.setRect(this.drawArea.getX() + x, this.drawArea.getY() + y,
                            this.drawArea.getWidth(), this.drawArea.getHeight()) ;
    }
  }

  /**
   * <zh>ͼεȵArea.</zh>
   * <en>Returns the hotspot areas.</en>
   *
   * @param drawTypes List <zh>ʾȵͼ (Ϊnull).</zh>
   * <en>The types of hotspot shape (null not permitted).</en>
   * @param fragment IAreaFragment <zh>ʵֽӿ{@link IAreaFragment}Ķ,(Ϊnull)</zh>
   * <en>The IAreaFragment object (maybe null).</en>
   *
   * @return List <zh>ͼεȵArea (Ϊnull).</zh>
   * <en>The hotspot areas (never null).</en>
   */
  public List getHotspotAreas(List drawTypes, IAreaFragment fragment) {
    return Collections.EMPTY_LIST ;
  }

  /**
   * <zh>ü.</zh>
   * <en>Set clip area.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   */
  protected void clip(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (this.drawArea == null) {
      throw new RuntimeException("'drawArea' can not be null.") ;
    }

    this.state = new Graphics2DState(g2) ;
    this.state.saveOldState() ;
    this.oldClip = g2.getClip() ;
    g2.setClip(this.drawArea) ;
  }

  /**
   * <zh>ָclip.</zh>
   * <en>Restore clip area.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   */
  protected void restoreClip(Graphics2D g2) {
    if (this.state != null) {
      g2.setClip(this.oldClip) ;
      this.state.restoreOldState() ;
    }
  }

  /**
   * <zh>ͼμshapesѰҰpointͼ.</zh>
   * <en>Find a shape which contains the specified point.</en>
   *
   * @param shapes List <zh>ͼμ (Ϊnull).</zh>
   * <en>The collection of shapes (null permitted).</en>
   * @param point Point2D <zh> (Ϊnull).</zh>
   * <en>A point (null not permitted).</en>
   * @param drawType ChartShapeDrawType <zh>ͼεĻ (Ϊnull).</zh>
   * <en>The shape draw shape (null permitted).</en>
   *
   * @return IChartShape
   */
  protected IChartShape findShape(List shapes, Point2D point, ChartShapeDrawType drawType) {
    if (shapes == null || point == null || drawType == null) {
      return null ;
    }

    for (int i = shapes.size() - 1 ; i >= 0 ; i--) {
      Object obj = shapes.get(i) ;
      if (obj == null) {
        continue ;
      }

      if (obj instanceof IChartShape) {
        IChartShape shape = (IChartShape) obj ;
        if (shape.getShapeDrawType().equals(ChartShapeDrawType.ANY_SHAPE)) { //
          if (shape.contains(point)) {
            return shape ;
          }
        } else if (shape.getShapeDrawType().equals(drawType) && shape.contains(point)) {
          return shape ;
        }
      } else if (obj instanceof IShapeCollection) {
        IShapeCollection collection = (IShapeCollection) obj ;

        IChartShape shape = collection.findChartShape(point, drawType) ;
        if (shape != null) {
          return shape ;
        }
      }
    }

    return null ;
  }

  /**
   * <zh>λƼÿһͼ.</zh>
   * <en>Drawing all ChartShape in this collection.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param shapes List <zh>ͼμ (Ϊnull).</zh>
   * <en>The collection of shapes (null permitted).</en>
   */
  protected void draw(Graphics2D g2, List shapes) {
    if (g2 == null || shapes == null) {
      return ;
    }

    int size = shapes.size() ;
    for (int i = 0 ; i < size ; i++) {
      Object obj = shapes.get(i) ;
      if (obj != null && obj instanceof IChartShape) {
        ((IChartShape) obj).draw(g2) ;
      }
    }
  }

  /**
   * <zh>ƶͼ(axisLine3DShape)ԭ㵽(x,y).</zh>
   * <en>Translates the origin of the all shapes(not include axisLine3DShape) in this chart to the
   * point (x,&nbsp;y) in the current coordinate system. </en>
   *
   * @param  x double <zh>x.</zh>
   * <en>The x coordinate.</en>
   * @param  y double <zh>y.</zh>
   * <en>The y coordinate.</en>
   * @param shapes List <zh>ͼμ (Ϊnull).</zh>
   * <en>The collection of shapes (null permitted).</en>
   */
  protected void translate(double x, double y, List shapes) {
    if ((x == 0 && y == 0) || shapes == null) {
      return ;
    }

    int size = shapes.size() ;
    for (int i = 0 ; i < size ; i++) {
      Object obj = shapes.get(i) ;
      if (obj != null && obj instanceof IChartShape) {
        ((IChartShape) obj).translate(x, y) ;
      }
    }
  }

  /**
   * <zh>еͼεļ.</zh>
   * <en>Remove all the shapes of the listener.</en>
   *
   * @param shapes List <zh>ͼμ (Ϊnull).</zh>
   * <en>The collection of shapes (null permitted).</en>
   */
  protected void clearShapesListener(List shapes) {
    if (shapes == null) {
      return ;
    }

    int size = shapes.size() ;
    for (int i = 0 ; i < size ; i++) {
      Object obj = shapes.get(i) ;
      if (obj != null && obj instanceof IChartShape) {
        ((IChartShape) obj).removeAllChangeListener() ;
      }
    }
  }

  /**
   * <zh>ͼεѡ״̬.</zh>
   * <en>Clear the selected shape.</en>
   *
   * @param shape IChartShape <zh>IChartShape (Ϊnull).</zh>
   * <en>The IChartShape object (null permitted).</en>
   */
  protected void clearSelectedShape(IChartShape shape) {
    if (shape == null) {
      return ;
    }

    shape.setSelected(false) ;
  }

  /**
   * <zh>еͼεѡ״̬.</zh>
   * <en>Clear all the selected shapes.</en>
   *
   * @param shapes List <zh>ͼμ (Ϊnull).</zh>
   * <en>The collection of shapes (null permitted).</en>
   */
  protected void clearSelectedShape(List shapes) {
    if (shapes == null) {
      return ;
    }

    int size = shapes.size() ;
    for (int i = 0 ; i < size ; i++) {
      Object obj = shapes.get(i) ;
      if (obj != null && obj instanceof IChartShape) {
        ((IChartShape) obj).setSelected(false) ;
      }
    }
  }

  /**
   * <zh>еͼεļ.</zh>
   * <en>Remove all the shapes of the listener.</en>
   *
   * @param shapes List <zh>ͼμ (Ϊnull).</zh>
   * <en>The collection of shapes (null permitted).</en>
   * @param listener IChartShapeChangeListener <zh>listener (Ϊnull).</zh>
   * <en>The listener object (null permitted).</en>
   */
  protected void removeShapesListener(List shapes, IChartShapeChangeListener listener) {
    if (shapes == null || listener == null) {
      return ;
    }

    int size = shapes.size() ;
    for (int i = 0 ; i < size ; i++) {
      Object obj = shapes.get(i) ;
      if (obj != null && obj instanceof IChartShape) {
        ((IChartShape) obj).removeChangeListener(listener) ;
      }
    }
  }

  /**
   * <zh>ͼεȵarea.</zh>
   * <en>Calculate the area of hotspot shape.</en>
   *
   * @param result <zh>صĽ (Ϊnull).</zh>
   * <en>The result (null not permitted).</en>
   * @param shape List <zh>ͼ (Ϊnull).</zh>
   * <en>The chart shape (null permitted).</en>
   * @param fragment IAreaFragment <zh>ʵֽӿ{@link IAreaFragment}Ķ,(Ϊnull)</zh>
   * <en>The {@link IAreaFragment} object (maybe null).</en>
   * @param drawType ChartShapeDrawType <zh>ͼ (Ϊnull).</zh>
   * <en>The shape draw type (null permitted).</en>
   */
  protected void addHotspotArea(List result, IChartShape shape, IAreaFragment fragment,
                                ChartShapeDrawType drawType) {
    if (result == null) {
      throw new IllegalArgumentException("Null 'result' argument.") ;
    }

    if (shape != null) {
      String hotspotType = shape.getHotspotType() ;
      String area = "" ;
      if (fragment != null) {
        fragment.getFragment(0, shape.getData()) ;
      }

      String areaText = null ;
      if (shape instanceof AbstractTextChartShape) {
        areaText = ((AbstractTextChartShape) shape).getText() ;
      }
      if (!StringUtil.isEmpty(areaText)) {
        areaText = " areaText=\"" + StringUtil.javascriptEscape(areaText) + "\"" ;
      } else {
        areaText = "" ;
      }

      result.add("<AREA shape=\"" + hotspotType
                 + "\" shape_type=\"" + ((drawType == null) ? "" : drawType.toString())
                 + "\" COORDS=\"" + shape.getHotspotPolyCoords(this.drawArea)
                 + "\"" + area + areaText + ">") ;
    }
  }

  /**
   * <zh>ͼεȵarea.</zh>
   * <en>Calculate the area of hotspot shape.</en>
   *
   * @param result <zh>صĽ (Ϊnull).</zh>
   * <en>The result (null not permitted).</en>
   * @param shapes List <zh>ͼμ (Ϊnull).</zh>
   * <en>The collection of shapes (null permitted).</en>
   * @param fragment IAreaFragment <zh>ʵֽӿ{@link IAreaFragment}Ķ,(Ϊnull)</zh>
   * <en>The {@link IAreaFragment} object (maybe null).</en>
   * @param drawType ChartShapeDrawType <zh>ͼ (Ϊnull).</zh>
   * <en>The shape draw type (null permitted).</en>
   */
  protected void addHotspotArea(List result, List shapes, IAreaFragment fragment,
                                ChartShapeDrawType drawType) {
    if (result == null) {
      throw new IllegalArgumentException("Null 'result' argument.") ;
    }

    if (shapes != null) {
      int size = shapes.size() ;
      for (int i = 0 ; i < size ; i++) {
        Object obj = shapes.get(i) ;
        if (obj != null && obj instanceof IChartShape) {
          IChartShape chartShape = (IChartShape) obj ;
          String hotspotType = chartShape.getHotspotType() ;
          String area = "" ;
          if (fragment != null) {
            area = fragment.getFragment(i, chartShape.getData()) ;
          }

          String areaText = null ;
          if (chartShape instanceof AbstractTextChartShape) {
            areaText = ((AbstractTextChartShape) chartShape).getText() ;
          }
          if (!StringUtil.isEmpty(areaText)) {
            areaText = " areaText=\"" + StringUtil.javascriptEscape(areaText) + "\"" ;
          } else {
            areaText = "" ;
          }

          result.add("<AREA shape=\"" + hotspotType
                     + "\" shape_type=\"" + ((drawType == null) ? "" : drawType.toString())
                     + "\" COORDS=\"" + chartShape.getHotspotPolyCoords(this.drawArea)
                     + "\"" + area + areaText + ">") ;
        }
      }
    }
  }

  /**
   * <zh>clone.</zh>
   * <en>Returns a clone of the axis.</en>
   *
   * @return A clone.
   * @throws CloneNotSupportedException if some component of the axis does
   *         not support cloning.
   */
  public Object clone() throws CloneNotSupportedException {
    AbstractShapeCollection clone = (AbstractShapeCollection)super.clone() ;
    clone.listenerList = new EventListenerList() ;

    return clone ;

  }

  /**
   * <zh>hashCode.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    return HashcodeUtil.hashCode(this.drawArea) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this axis for equality with another object.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>The object (null permitted).</en>
   *
   * @return true or false.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof AbstractShapeCollection)) {
      return false ;
    }

    AbstractShapeCollection that = (AbstractShapeCollection) obj ;
    if (!EqualsUtil.equals(this.drawArea, that.drawArea)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  A string representation of the object.
   */
  public String toString() {
    return "[drawArea=" + this.drawArea + "]" ;
  }

  /**
   * <zh>л</zh>
   * <en>Provides serialization support</en>
   * @param stream <zh></zh>
   * <en>the output stream</en>
   * @throws IOException  if there is an I/O error.
   */
  private void writeObject(ObjectOutputStream stream) throws IOException {
    stream.defaultWriteObject() ;
    SerialUtil.writeShape(stream, this.drawArea) ;
  }

  /**
   * Provides serialization support.
   *
   * @param stream  the input stream.
   *
   * @throws IOException  if there is an I/O error.
   * @throws ClassNotFoundException  if there is a classpath problem.
   */
  private void readObject(ObjectInputStream stream) throws IOException, ClassNotFoundException {
    stream.defaultReadObject() ;
    this.drawArea = (Rectangle2D) SerialUtil.readShape(stream) ;
    this.listenerList = new EventListenerList() ;
  }

}
