package com.ijchart.xychart.data ;

import java.io.Serializable ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>XYBoxAndWhisker(ͼ)ͼݶ,ⲿǲ޸ĵ.</zh>
 * <en>XYBoxAndWhisker ChartData object,this class is immutable.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class IntervalXBoxAndWhiskerData extends BoxAndWhiskerData implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -8610961756854160043L ;

  /**
   * <zh>xʼֵ.</zh>
   * <en>The start x value.</en>
   */
  private double startX ;

  /**
   * <zh>xֵ.</zh>
   * <en>The end x value.</en>
   */
  private double endX ;

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param startX double <zh>xʼֵ.</zh>
   * <en>The start x value.</en>
   * @param endX double <zh>xֵ.</zh>
   * <en>The end x value.</en>
   */
  public IntervalXBoxAndWhiskerData(double startX, double endX) {
    super() ;
    this.startX = startX ;
    this.endX = endX ;
  }

  /**
   * <zh>xʼֵ.</zh>
   * <en>Returns the start x value.</en>
   *
   * @return double <zh>xʼֵ.</zh>
   * <en>The start x value.</en>
   */
  public double getStartX() {
    return this.startX ;
  }

  /**
   * <zh>xֵ.</zh>
   * <en>Returns the end x value.</en>
   *
   * @return double <zh>xֵ.</zh>
   * <en>The end x value.</en>
   */
  public double getEndX() {
    return this.endX ;
  }

  /**
   * <zh>رǩַ.</zh>
   * <en>Returns the label text.</en>
   *
   * @return String <zh>ǩַ (Ϊnull).</zh>
   * <en>The label text (maybe null).</en>
   */
  public String getLabelText() {
    return "(" + formatNumber(this.startX)
      + "," + formatNumber(this.endX)
      + "," + formatNumber(getMedian())
      + "," + formatNumber(getMean())
      + "," + formatNumber(getMinRegularValue())
      + "," + formatNumber(getMaxRegularValue())
      + ", " + formatNumber(getQ1())
      + "," + formatNumber(getQ3()) + ")" ;
  }

  /**
   * <zh>ͼȵareaĺ.</zh>
   * <en>Returns the function arguments of hotspot area.</en>
   *
   * @return String <zh> (Ϊnull).</zh>
   * <en>The function arguments (maybe null).</en>
   */
  public String getAreaFunctionArguments() {
    return formatNumberForFunction(this.startX)
      + "," + formatNumberForFunction(this.endX)
      + "," + formatNumberForFunction(getMedian())
      + "," + formatNumberForFunction(getMean())
      + "," + formatNumberForFunction(getMinRegularValue())
      + "," + formatNumberForFunction(getMaxRegularValue())
      + ", " + formatNumberForFunction(getQ1())
      + "," + formatNumberForFunction(getQ3()) ;
  }

  /**
   * <zh>ͼȵareaurl.</zh>
   * <en>Returns the url for hotspot area.</en>
   *
   * @return String <zh>url (Ϊnull).</zh>
   * <en>The url fragment (maybe null).</en>
   */
  public String getAreaURLFragment() {
    return "startX=" + formatNumber(this.startX)
      + "&endX=" + formatNumber(this.endX)
      + "&median=" + formatNumber(getMedian())
      + "&mean=" + formatNumber(getMean())
      + "&minRegularValue=" + formatNumber(getMinRegularValue())
      + "&maxRegularValue=" + formatNumber(getMaxRegularValue())
      + "&q1= " + formatNumber(getQ1())
      + "&q3=" + formatNumber(getQ3()) ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof IntervalXBoxAndWhiskerData)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    IntervalXBoxAndWhiskerData that = (IntervalXBoxAndWhiskerData) obj ;
    if (this.startX != that.startX) {
      return false ;
    }
    if (this.endX != that.endX) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    double result = super.hashCode() ;
    result = result * 37 + this.startX ;
    result = result * 37 + this.endX ;

    return (int) result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[startX=" + this.startX
      + ",endX=" + this.endX
      + ",mean=" + getMean()
      + ",median=" + getMedian()
      + ",q1=" + getQ1()
      + ",q3=" + getQ3()
      + "]" ;
  }

}
