package com.ijchart.xychart.dataset ;

import java.io.Serializable ;

import com.ijchart.xychart.constant.DatasetSlidingType ;
import com.ijchart.xychart.constant.SortType ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.util.EqualsUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ɹݼ.</zh>
 * <en>A dataset implementation that presents a subset of the underlying dataset.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class SlidingDataset extends AbstractDataset implements IMatrixDataset, Serializable,
  Cloneable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 5983215559558424039L ;

  /**
   * <zh>ڲݼ.</zh>
   * <en>The underlying dataset.</en>
   */
  private IMatrixDataset underlyingDataset ;

  /**
   * <zh>ݼ.</zh>
   * <en>The sliding type of dataset.</en>
   */
  private DatasetSlidingType slidingType ;

  /**
   * <zh>ʼֵ.</zh>
   * <en>The start index.</en>
   */
  private int startIndex ;

  /**
   * <zh>,.</zh>
   * <en>The max data row or column count.</en>
   */
  private int maxCount ;

  /**
   * <zh>.</zh>
   * <en>A constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ڲݼ (Ϊnull).</zh>
   * <en>The underlying dataset (null not permitted).</en>
   * @param slidingType DatasetSlidingType <zh>ݼ (Ϊnull).</zh>
   * <en>The sliding type of dataset (null not permitted).</en>
   * @param startIndex int <zh>ʼֵ.</zh>
   * <en>The start index.</en>
   * @param maxCount int <zh>,.</zh>
   * <en>The max data row or column count.</en>
   */
  public SlidingDataset(IMatrixDataset dataset, DatasetSlidingType slidingType, int startIndex,
                        int maxCount) {
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument") ;
    }
    if (slidingType == null) {
      throw new IllegalArgumentException("Null 'slidingType' argument") ;
    }
    this.underlyingDataset = dataset ;
    this.slidingType = slidingType ;

    if (startIndex < 0) {
      throw new IllegalArgumentException("Argument 'startIndex' must be >=0") ;
    }
    if (this.slidingType.equals(DatasetSlidingType.ROW)
        && startIndex >= this.underlyingDataset.getRowCount()) {
      throw new IllegalArgumentException("Argument 'startIndex' must be <"
                                         + this.underlyingDataset.getRowCount()) ;
    } else if (this.slidingType.equals(DatasetSlidingType.COLUMN)
               && startIndex >= this.underlyingDataset.getColumnCount()) {
      throw new IllegalArgumentException("Argument 'startIndex' must be <"
                                         + this.underlyingDataset.getColumnCount()) ;
    }
    this.startIndex = startIndex ;

    if (maxCount < 0) {
      throw new IllegalArgumentException("Argument 'maxCount' must be >=0.") ;
    }
    this.maxCount = maxCount ;
  }

  /**
   * <zh>ڲݼ.</zh>
   * <en>Returns the underlying dataset.</en>
   *
   * @return IMatrixDataset <zh>ڲݼ (Ϊnull).</zh>
   * <en>The underlying dataset (never null).</en>
   */
  public IMatrixDataset getUnderlyingDataset() {
    return this.underlyingDataset ;
  }

  /**
   * <zh>ݼ.</zh>
   * <en>Returns the sliding type of dataset.</en>
   *
   * @return DatasetSlidingType <zh> (Ϊnull).</zh>
   * <en>The sliding type (never null).</en>
   */
  public DatasetSlidingType getSlidingType() {
    return this.slidingType ;
  }

  /**
   * <zh>ݼ,м
   * {@link com.ijchart.xychart.event.DatasetChangeEvent}¼.</zh>
   * <en>Sets the sliding type of dataset and sends a
   * {@link com.ijchart.xychart.event.DatasetChangeEvent} to all registered listeners.</en>
   *
   * @param slidingType DatasetSlidingType <zh>.</zh>
   * <en>The sliding type.</en>
   */
  public void setSlidingType(DatasetSlidingType slidingType) {
    if (slidingType == null) {
      throw new IllegalArgumentException("Null 'slidingType' argument") ;
    }

    if (!this.slidingType.equals(slidingType)) {
      this.slidingType = slidingType ;
      fireDatasetChanged() ;
    }
  }

  /**
   * <zh>ؿʼֵ.</zh>
   * <en>Returns the start index.</en>
   *
   * @return int <zh>ֵ.</zh>
   * <en>The start index.</en>
   */
  public int getStartIndex() {
    return this.startIndex ;
  }

  /**
   * <zh>ÿʼֵ,м
   * {@link com.ijchart.xychart.event.DatasetChangeEvent}¼.</zh>
   * <en>Sets the start index and sends a
   * {@link com.ijchart.xychart.event.DatasetChangeEvent} to all registered listeners.</en>
   *
   * @param startIndex int <zh>ʼֵ.</zh>
   * <en>The start index.</en>
   */
  public void setStartIndex(int startIndex) {
    if (startIndex < 0) {
      throw new IllegalArgumentException("Argument 'startIndex' must be >=0") ;
    }

    if (this.slidingType.equals(DatasetSlidingType.ROW)
        && startIndex >= this.underlyingDataset.getRowCount()) {
      throw new IllegalArgumentException("Argument 'startIndex' must be <"
                                         + this.underlyingDataset.getRowCount()) ;
    } else if (this.slidingType.equals(DatasetSlidingType.COLUMN)
               && startIndex >= this.underlyingDataset.getColumnCount()) {
      throw new IllegalArgumentException("Argument 'startIndex' must be <"
                                         + this.underlyingDataset.getColumnCount()) ;
    }

    if (this.startIndex != startIndex) {
      this.startIndex = startIndex ;
      fireDatasetChanged() ;
    }
  }

  /**
   * <zh>,.</zh>
   * <en>Returns the max data row or column count.</en>
   *
   * @return int <zh>.</zh>
   * <en>The max data row or column count.</en>
   */
  public int getMaxCount() {
    return this.maxCount ;
  }

  /**
   * <zh>,,м
   * {@link com.ijchart.xychart.event.DatasetChangeEvent}¼.</zh>
   * <en>Sets the max data row or column count and sends a
   * {@link com.ijchart.xychart.event.DatasetChangeEvent} to all registered listeners.</en>
   *
   * @param maxCount int <zh>,.</zh>
   * <en>The max data row or column count.</en>
   */
  public void setMaxCount(int maxCount) {
    if (maxCount < 0) {
      throw new IllegalArgumentException("Argument 'maxCount' must be >=0.") ;
    }

    if (this.maxCount != maxCount) {
      this.maxCount = maxCount ;
      fireDatasetChanged() ;
    }
  }

  /**
   * <zh>ݼ.</zh>
   * <en>The row count of this dataset.</en>
   *
   * @return int <zh>ݼ.</zh>
   * <en>The row count.</en>
   */
  public int getRowCount() {
    if (this.slidingType.equals(DatasetSlidingType.ROW)) {
      return Math.min(this.maxCount, this.underlyingDataset.getRowCount() - this.startIndex) ;
    } else if (this.slidingType.equals(DatasetSlidingType.COLUMN)) {
      return this.underlyingDataset.getRowCount() ;
    }

    return 0 ;
  }

  /**
   * <zh>indexõеRowKeyֵ,һΪ0,ڶΪ1,.</zh>
   * <en>Get RowKey by index.The first RowKey of the dataset is at index 0,the next at index 1,
   *  and so on, as for array indexing.</en>
   *
   * @param index int <zh>,һΪ0,ڶΪ1.</zh>
   * <en>The row index,first is 0,next is 1.</en>
   * @return Object <zh>еRowKey,û򷵻null.</zh>
   * <en>The dataset RowKey for this index,if not found return null.</en>
   */
  public Object getRowKey(int index) {
    if (this.slidingType.equals(DatasetSlidingType.ROW)) {
      return this.underlyingDataset.getRowKey(this.startIndex + index) ;
    } else if (this.slidingType.equals(DatasetSlidingType.COLUMN)) {
      return this.underlyingDataset.getRowKey(index) ;
    }

    return null ;
  }

  /**
   * <zh>ݼ.</zh>
   * <en>The column count of this dataset.</en>
   *
   * @return int <zh>ݼ.</zh>
   * <en>The column count.</en>
   */
  public int getColumnCount() {
    if (this.slidingType.equals(DatasetSlidingType.ROW)) {
      return this.underlyingDataset.getColumnCount() ;
    } else if (this.slidingType.equals(DatasetSlidingType.COLUMN)) {
      return Math.min(this.maxCount, this.underlyingDataset.getColumnCount() - this.startIndex) ;
    }

    return 0 ;
  }

  /**
   * <zh>ݼ.</zh>
   * <en>The column count of this dataset.</en>
   *
   * @param rowIdx int <zh>.</zh>
   * <en>The row index.</en>
   * @return int <zh>ݼ.</zh>
   * <en>The column count.</en>
   */
  public int getColumnCount(int rowIdx) {
    if (this.slidingType.equals(DatasetSlidingType.ROW)) {
      return this.underlyingDataset.getColumnCount(rowIdx) ;
    } else if (this.slidingType.equals(DatasetSlidingType.COLUMN)) {
      return Math.min(this.maxCount,
                      Math.max(this.underlyingDataset.getColumnCount(rowIdx) - this.startIndex, 0)) ;
    }

    return 0 ;
  }

  /**
   * <zh>indexõеColKeyֵ,һΪ0,ڶΪ1,.</zh>
   * <en>Get ColKey by index.The first ColKey of the dataset is at index 0,the next at index 1,
   *  and so on, as for array indexing.</en>
   *
   * @param index int <zh>,һΪ0,ڶΪ1.</zh>
   * <en>The index,first is 0,next is 1.</en>
   * @return Object <zh>еColKey,û򷵻null.</zh>
   * <en>The dataset ColKey for this index,if not found return null.</en>
   */
  public Object getColKey(int index) {
    if (this.slidingType.equals(DatasetSlidingType.ROW)) {
      return this.underlyingDataset.getColKey(index) ;
    } else if (this.slidingType.equals(DatasetSlidingType.COLUMN)) {
      return this.underlyingDataset.getColKey(this.startIndex + index) ;
    }

    return null ;
  }

  /**
   * <zh>õάеĳֵ,rowIndexcolIndexΧ׳쳣.</zh>
   * <en>Get the data of matrix by rowIndex and colIndex,an Exception will be throw when index out
   * of bound.</en>
   *
   * @param rowIndex int <zh>ֵ,һ0,ڶ1,.</zh>
   * <en>The row index.The first is at index 0,the next is at 1, and so on.</en>
   * @param colIndex int <zh>ֵ,һ0,ڶ1,.</zh>
   * <en>The column index.The first is at index 0,the next is at 1, and so on.</en>
   * @return IChartData <zh>иõֵ(Ϊnull).</zh>
   * <en>The data of matrix (maybe null).</en>
   */
  public IChartData getData(int rowIndex, int colIndex) {
    if (this.slidingType.equals(DatasetSlidingType.ROW)) {
      return this.underlyingDataset.getData(rowIndex + this.startIndex, colIndex) ;
    } else if (this.slidingType.equals(DatasetSlidingType.COLUMN)) {
      return this.underlyingDataset.getData(rowIndex, this.startIndex + colIndex) ;
    }

    return null ;
  }

  /**
   * <zh>ݼǷΪ.</zh>
   * <en>Returns whether or not the dataset is empty.</en>
   *
   * @return boolean <zh>յݼtrue,Ϊfalse.</zh>
   * <en>Returns true if the axis dataset is empty,otherwise false.</en>
   */
  public boolean isEmpty() {
    if (this.slidingType.equals(DatasetSlidingType.ROW)) {
      int size = this.underlyingDataset.getRowCount() ;
      if (size == 0) {
        return true ;
      }

      for (int i = this.startIndex ; i < size ; i++) {
        if (this.underlyingDataset.getColumnCount(i) > 0) {
          return false ;
        }
      }
    } else if (this.slidingType.equals(DatasetSlidingType.COLUMN)) {
      int cols = this.underlyingDataset.getColumnCount() ;
      if (cols == 0) {
        return true ;
      }

      int rows = this.underlyingDataset.getRowCount() ;
      for (int i = 0 ; i < rows ; i++) {
        if (this.underlyingDataset.getColumnCount(i) > this.startIndex) {
          return false ;
        }
      }
    }

    return true ;
  }

  /**
   * <zh>жеֵǷc.</zh>
   * <en>Test whether or not all data is Class c in this dataset.</en>
   *
   * @param c Class <zh>Ҫжϵclass (Ϊnull).</zh>
   * <en>A class (null not permitted).</en>
   *
   * @return boolean <zh>ֵctrue,򷵻false.</zh>
   * <en>Returns true if all data equals Class c in this dataset,false otherwise.</en>
   */
  public boolean isThisDataType(Class c) {
    return this.underlyingDataset.isThisDataType(c) ;
  }

  /**
   * <zh>ݼеһݽ,ݼеݱʵ{@link java.lang.Comparable}ӿ.</zh>
   * <en>Sort the datas of a dataset row,the data must implements
   *  {@link java.lang.Comparable} interface.</en>
   *
   * @param rowIndex int <zh>ֵ,һ0,ڶ1,.</zh>
   * <en>The row index.The first is at index 0,the next is at 1, and so on.</en>
   * @param sortType SortType <zh> (Ϊnull).</zh>
   * <en>The sort type,ascending or descending.</en>
   */
  public void sortRowData(int rowIndex, SortType sortType) {
    this.underlyingDataset.sortRowData(rowIndex, sortType) ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[slidingType=" + this.slidingType + ",startIndex=" + this.startIndex + ",maxCount="
      + this.maxCount + "]" ;
  }

  /**
   * <zh>hascodeֵ.</zh>
   * <en>Get hascode.</en>
   * @return int
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = 29 * result + this.slidingType.hashCode() ;
    result = 29 * result + this.startIndex ;
    result = 29 * result + this.maxCount ;

    return result ;
  }

  /**
   * <zh>equals.</zh>
   * <en>Override equals method.</en>
   * @param obj Object
   * @return boolean
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof SlidingDataset)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    SlidingDataset other = (SlidingDataset) obj ;
    if (!EqualsUtil.equals(this.underlyingDataset, other.underlyingDataset)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.slidingType, other.slidingType)) {
      return false ;
    }
    if (this.startIndex != other.startIndex) {
      return false ;
    }
    if (this.maxCount != other.maxCount) {
      return false ;
    }

    return true ;
  }

}
