package com.ijchart.xychart.format ;

import java.io.Serializable ;
import java.text.DecimalFormat ;
import java.text.NumberFormat ;

import com.ijchart.xychart.data.XYNumberChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ʽ{@link com.ijchart.xychart.data.XYNumberChartData}͵.</zh>
 * <en>A formatter class for {@link com.ijchart.xychart.data.XYNumberChartData} chart data.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYNumberChartDataFormatter extends AbstractChartDataFormatterAdapter implements
  IChartDataFormatter, Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -7352453872138664734L ;

  /**
   * <zh>ֻʾxֵ.</zh>
   * <en>Show x value only.</en>
   */
  public static final int XYNUMBER_FORMATTER_TYPE_X = 1 ;

  /**
   * <zh>ֻʾyֵ.</zh>
   * <en>Show y value only.</en>
   */
  public static final int XYNUMBER_FORMATTER_TYPE_Y = 2 ;

  /**
   * <zh>ʾ(x,y)ֵ.</zh>
   * <en>Show (x,y) value.</en>
   */
  public static final int XYNUMBER_FORMATTER_TYPE_XY = 3 ;

  /**
   * <zh>ʽ.</zh>
   * <en>The format type;</en>
   */
  private int formatType ;

  /**
   * <zh>ʽxֵ.</zh>
   * <en>The x-value format object.</en>
   */
  private DecimalFormat xFormat ;

  /**
   * <zh>ʽyֵ.</zh>
   * <en>The y-value format object.</en>
   */
  private DecimalFormat yFormat ;

  /**
   * <zh>.</zh>
   * <en>Constructor.</en>
   *
   * @param xMask String <zh>xֵ (Ϊnull).</zh>
   * <en>The x-value format mask (null not permitted).</en>
   * @param yMask String <zh>yֵ (Ϊnull).</zh>
   * <en>The y-value format mask (null not permitted).</en>
   * @param formatType int <zh>ʽ,ֵܵΪ{@link #XYNUMBER_FORMATTER_TYPE_X},
   * {@link #XYNUMBER_FORMATTER_TYPE_Y},{@link #XYNUMBER_FORMATTER_TYPE_XY}.</zh>
   * <en>The format type,see {@link #XYNUMBER_FORMATTER_TYPE_X},
   * {@link #XYNUMBER_FORMATTER_TYPE_Y},{@link #XYNUMBER_FORMATTER_TYPE_XY}</en>
   */
  public XYNumberChartDataFormatter(String xMask, String yMask, int formatType) {
    if (xMask == null) {
      throw new IllegalArgumentException("Null 'xMask' argument.") ;
    }
    if (yMask == null) {
      throw new IllegalArgumentException("Null 'yMask' argument.") ;
    }

    this.xFormat = (DecimalFormat) NumberFormat.getInstance() ;
    this.xFormat.applyPattern(xMask) ;

    this.yFormat = (DecimalFormat) NumberFormat.getInstance() ;
    this.yFormat.applyPattern(yMask) ;

    this.formatType = formatType ;
  }

  /**
   * <zh>ظʽ.</zh>
   * <en>Returns the format type;</en>
   *
   * @return int <zh>ʽ.</zh>
   * <en>The format type;</en>
   */
  public int getFormatType() {
    return this.formatType ;
  }

  /**
   * <zh>ظʽxֵ.</zh>
   * <en>Returns the x-value format object.</en>
   *
   * @return DecimalFormat <zh>ʽxֵ.</zh>
   * <en>The x-value format object.</en>
   */
  public DecimalFormat getXFormat() {
    return this.xFormat ;
  }

  /**
   * <zh>ظʽyֵ.</zh>
   * <en>Returns the y-value format object.</en>
   *
   * @return DecimalFormat <zh>ʽyֵ.</zh>
   * <en>The y-value format object.</en>
   */
  public DecimalFormat getYFormat() {
    return this.yFormat ;
  }

  /**
   * <zh>ʽݼĳ{@link com.ijchart.xychart.data.IChartData}Ϊַ.</zh>
   * <en>Formatted an {@link com.ijchart.xychart.data.IChartData} object to a string.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   * @param row int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset row number,must be greater than or equal to 0.</en>
   * @param column int <zh>ݼ,ڵ0.</zh>
   * <en>The dataset column number,must be greater than or equal to 0.</en>
   *
   * @return String <zh>ַ (Ϊnull).</zh>
   * <en>A string (maybe null).</en>
   */
  public String format(IMatrixDataset dataset, int row, int column) {
    if (dataset == null) {
      throw new IllegalArgumentException("Null 'dataset' argument.") ;
    }
    if (row < 0) {
      throw new IllegalArgumentException("Must be 'row'>=0.") ;
    }
    if (column < 0) {
      throw new IllegalArgumentException("Must be 'column'>=0.") ;
    }

    Object obj = dataset.getData(row, column) ;
    if (obj != null && obj instanceof XYNumberChartData) {
      XYNumberChartData data = (XYNumberChartData) obj ;
      if (this.formatType == XYNUMBER_FORMATTER_TYPE_X) {
        return this.xFormat.format(data.getX()) ;
      } else if (this.formatType == XYNUMBER_FORMATTER_TYPE_Y) {
        return this.yFormat.format(data.getY()) ;
      } else if (this.formatType == XYNUMBER_FORMATTER_TYPE_XY) {
        return "(" + this.xFormat.format(data.getX()) + ","
          + this.yFormat.format(data.getY()) + ")" ;
      }
    }

    return null ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this object for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪȽϵһ (Ϊnull).</zh>
   * <en>The object to test against (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof XYNumberChartDataFormatter)) {
      return false ;
    }

    XYNumberChartDataFormatter that = (XYNumberChartDataFormatter) obj ;
    if (!EqualsUtil.equals(this.xFormat, that.xFormat)) {
      return false ;
    }
    if (!EqualsUtil.equals(this.yFormat, that.yFormat)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Creates and returns a copy of this object.</en>
   *
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the <code>Cloneable</code> interface. Subclasses
   *               that override the <code>clone</code> method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  protected Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code.</en>
   *
   * @return A hash code.
   */
  public int hashCode() {
    int hashCode = HashcodeUtil.hashCode(this.xFormat) ;
    hashCode = 37 * hashCode + HashcodeUtil.hashCode(this.yFormat) ;

    return hashCode ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of this Range.</en>
   *
   * @return A String.
   */
  public String toString() {
    return "[xFormat=" + this.xFormat + ",yFormat=" + this.yFormat + "]" ;
  }

}
