package com.ijchart.xychart.function ;

import java.io.Serializable ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ֲ̬.</zh>
 * <en>The normal distribution function.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class NormalDistributionFunction implements IFunction, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 4381270345160495350L ;

  /**
   * <zh>ƽ.</zh>
   * <en>The mean.</en>
   */
  private double mean ;

  /**
   * <zh>׼.</zh>
   * <en>The standard deviation.</en>
   */
  private double stDev ;

  /**
   * <zh>һ׼.</zh>
   * <en>Create a standard diviation function.</en>
   *
   * @param mean double <zh>ƽ.</zh>
   * <en>The mean.</en>
   * @param stDev double <zh>׼.</zh>
   * <en>The standard deviation.</en>
   */
  public NormalDistributionFunction(double mean, double stDev) {
    this.mean = mean ;
    this.stDev = stDev ;
  }

  /**
   * <zh>ƽ.</zh>
   * <en>Returns the mean.</en>
   *
   * @return double <zh>ƽ.</zh>
   * <en>The mean.</en>
   */
  public double getMean() {
    return this.mean ;
  }

  /**
   * <zh>ر׼.</zh>
   * <en>Returns the standard deviation.</en>
   *
   * @return double <zh>׼.</zh>
   * <en>The standard deviation.</en>
   */
  public double getStDev() {
    return this.stDev ;
  }

  /**
   * <zh>xֵyֵ.</zh>
   * <en>Calculate the y value for a given inpu x.</en>
   *
   * @param x double <zh>xֵ.</zh>
   * <en>The x value.</en>
   * @return double <zh>yֵ.</zh>
   * <en>The y value.</en>
   */
  public double getY(double x) {
    return Math.exp(-1.0 * (x - this.mean) * (x - this.mean)
                / (2 * this.stDev * this.stDev)) / Math.sqrt(2 * Math.PI
                * this.stDev * this.stDev);
  }

  /**
   * <zh>equals.</zh>
   * <en>Override equals method.</en>
   * @param obj Object
   * @return boolean
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }

    if (!(obj instanceof NormalDistributionFunction)) {
      return false ;
    }

    NormalDistributionFunction that = (NormalDistributionFunction) obj ;

    return this.mean == that.mean && this.stDev == that.stDev ;
  }

  /**
   * <zh>toString().</zh>
   * <en>Override toString method.</en>
   * @return String
   */
  public String toString() {
    return "[mean=" + this.mean + ",stDev=" + this.stDev + "]" ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object. </en>
   * @return  a hash code value for this object.
   */
  public int hashCode() {
    return (int) ((this.mean + this.stDev) * 1000) ;
  }

}
