package com.ijchart.xychart.plot.combine ;

import java.awt.Graphics2D ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.category.CategoryAxis ;
import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.axis.value.NumberValueAxis ;
import com.ijchart.xychart.chartinfo.ChartShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.PlotType ;
import com.ijchart.xychart.event.RenderChangeEvent ;
import com.ijchart.xychart.plot.CategoryPlot ;
import com.ijchart.xychart.range.NumberRange ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.RectangleInsets ;
import com.ijchart.xychart.ui.supplier.IDrawingSupplier ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>CategoryPlot֮乲value.</zh>
 * <en>A combined category plot where the value axis is shared.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class CombineValueAxisCategoryPlot extends AbstractCombineCategoryPlot implements
  Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 3287850791965823558L ;

  /**
   * <zh>ͼplot,Ҫʵִ˷.</zh>
   * <en>Draws the plot within the specified area.Subclasses need to provide an
   * implementation of this method, obviously.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param plotArea Rectangle2D <zh>plot (Ϊnull).</zh>
   * <en>The plot draw area (null not permitted).</en>
   * @param chartShapes ChartShapeCollection <zh>chartͼμ (Ϊnull).</zh>
   * <en>Collects information about the chart (null not permitted).</en>
   */
  public void draw(Graphics2D g2, Rectangle2D plotArea, ChartShapeCollection chartShapes) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (plotArea == null) {
      throw new IllegalArgumentException("Null 'plotArea' argument.") ;
    }
    if (chartShapes == null) {
      throw new IllegalArgumentException("Null 'chartShapes' argument.") ;
    }

    this.dataArea = plotArea ;
    int plotCount = this.subPlots.size() ;
    if (plotCount == 0) {
      return ;
    }

    CategoryPlot firstPlot = (CategoryPlot)this.subPlots.get(0) ;
    ChartOrientation orientation = firstPlot.getOrientation() ;
    for (int i = 0 ; i < plotCount ; i++) {
      CategoryPlot subPlot = (CategoryPlot)this.subPlots.get(i) ;
      if (!subPlot.getOrientation().equals(orientation)) {
        subPlot.setOrientation(orientation) ;
      }
    }

    double maxSpace = calculateMaxCategoryAxisSpace(g2, plotArea) ;
    NumberRange maxRange = calculateMaxValueAxisRange() ;
    changetPlotInsets(maxSpace) ;

    double totalWight = 0 ; //Ȩ֮
    for (int i = 0 ; i < plotCount ; i++) {
      Double wight = (Double)this.subPlotWight.get(i) ;
      totalWight += wight.doubleValue() ;
    }

    IDrawingSupplier supplier = firstPlot.getDrawingSupplier() ;
    supplier.setCanReset(true) ;

    double totalGap = (plotCount - 1) * this.plotGap ;
    if (orientation.equals(ChartOrientation.VERTICAL)) {
      if (totalGap >= plotArea.getHeight()) {
        throw new RuntimeException("Total gap size is greater than the height of plotArea.") ;
      }
    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      if (totalGap >= plotArea.getHeight()) {
        throw new RuntimeException("Total gap size is greater than the width of plotArea.") ;
      }
    }

    double plotX = plotArea.getX() ;
    double plotY = plotArea.getY() ;
    double plotMaxY = plotArea.getMaxY() ;
    double plotWidth = plotArea.getWidth() ;
    double plotHeight = plotArea.getHeight() ;
    double totalPlotHeight = 0 ;
    for (int i = 0 ; i < this.subPlots.size() ; i++) {
      CategoryPlot subPlot = (CategoryPlot)this.subPlots.get(i) ;
      double plotWight = ((Double)this.subPlotWight.get(i)).doubleValue() ;

      if (i > 0) {
        subPlot.setDrawingSupplier(supplier) ;
        supplier.setCanReset(false) ;

        int valueAxisCount = subPlot.getValueAxisCount() ;
        for (int j = 0 ; j < valueAxisCount ; j++) {
          AbstractValueAxis valueAxis = subPlot.getValueAxis(j) ;
          valueAxis.setVisible(false) ;
          if (valueAxis instanceof NumberValueAxis) {
            ((NumberValueAxis) valueAxis).setUserDefineDatasetRange(maxRange) ;
          }
        }
      } else {
        int valueAxisCount = subPlot.getValueAxisCount() ;
        if (valueAxisCount > 0) {
          for (int j = 0 ; j < valueAxisCount ; j++) {
            AbstractValueAxis valueAxis = subPlot.getValueAxis(j) ;
            if (valueAxis instanceof NumberValueAxis) {
              ((NumberValueAxis) valueAxis).setUserDefineDatasetRange(maxRange) ;
            }
          }
        }
      }

      int categoryAxisCount = subPlot.getCategoryAxisCount() ;
      for (int j = 0 ; j < categoryAxisCount ; j++) {
        subPlot.getCategoryAxis(j).setFixedAxisSpace(maxSpace) ;
      }

      if (orientation.equals(ChartOrientation.VERTICAL)) {
        double subPlotWidth = (plotWidth - totalGap) * plotWight / totalWight ;
        Rectangle2D subPlotArea = new Rectangle2D.Double(plotX + totalPlotHeight, plotY,
                                                         subPlotWidth, plotHeight) ;
        subPlot.draw(g2, subPlotArea, chartShapes) ;

        totalPlotHeight += subPlotWidth + this.plotGap ;
      } else if (orientation.equals(ChartOrientation.HORIZONTAL)) {
        double subPlotHeight = (plotHeight - totalGap) * plotWight / totalWight ;
        totalPlotHeight += subPlotHeight ;
        Rectangle2D subPlotArea = new Rectangle2D.Double(plotX, plotMaxY - totalPlotHeight,
                                                         plotWidth, subPlotHeight) ;
        totalPlotHeight += this.plotGap ;
        subPlot.draw(g2, subPlotArea, chartShapes) ;
      }
    }
  }

  /**
   * <zh>յrender仯¼.</zh>
   * <en>Receives notification of a render change event.</en>
   *
   * @param event RenderChangeEvent <zh>render仯¼.</zh>
   * <en>The render change event.</en>
   */
  public void renderChanged(RenderChangeEvent event) {
    for (int i = 0 ; i < this.subPlots.size() ; i++) {
      CategoryPlot subPlot = (CategoryPlot)this.subPlots.get(i) ;

      int valueAxisCount = subPlot.getValueAxisCount() ;
      for (int j = 0 ; j < valueAxisCount ; j++) {
        AbstractValueAxis valueAxis = subPlot.getValueAxis(j) ;
        if (valueAxis instanceof NumberValueAxis) {
          ((NumberValueAxis) valueAxis).setUserDefineDatasetRange(null) ;
        }
      }
    }

    super.renderChanged(event) ;
  }

  /**
   * <zh>ıplotinsetĴС,ʹplotжʱҲܶ.</zh>
   * <en>Changed the plot insets.</en>
   *
   * @param maxSpace double <zh>ռ,Ȼ߶.</zh>
   * <en>The max width or height for all axes.</en>
   */
  protected void changetPlotInsets(double maxSpace) {
    int plotCount = this.subPlots.size() ;
    if (plotCount == 0) {
      return ;
    }

    CategoryPlot firstPlot = (CategoryPlot)this.subPlots.get(0) ;
    ChartOrientation orientation = firstPlot.getOrientation() ;
    if (orientation.equals(ChartOrientation.VERTICAL)) {
      int maxTopCount = 0, maxBottomCount = 0 ;
      int[] topAxisCount = new int[plotCount] ;
      int[] bottomAxisCount = new int[plotCount] ;
      for (int i = 0 ; i < plotCount ; i++) {
        CategoryPlot subPlot = (CategoryPlot)this.subPlots.get(i) ;
        topAxisCount[i] = getAxisCountByPosition(subPlot, orientation, RectangleEdge.TOP) ;
        bottomAxisCount[i] = getAxisCountByPosition(subPlot, orientation, RectangleEdge.BOTTOM) ;

        maxTopCount = Math.max(maxTopCount, topAxisCount[i]) ;
        maxBottomCount = Math.max(maxBottomCount, bottomAxisCount[i]) ;
      }

      for (int i = 0 ; i < plotCount ; i++) {
        double topSpace = (maxTopCount - topAxisCount[i]) * maxSpace ;
        double bottomSpace = (maxBottomCount - bottomAxisCount[i]) * maxSpace ;
        CategoryPlot subPlot = (CategoryPlot)this.subPlots.get(i) ;
        RectangleInsets insets = subPlot.getInsets() ;
        subPlot.setInsets(new RectangleInsets(insets.getUnitType(), topSpace,
                                              insets.getLeft(), bottomSpace, insets.getRight())) ;
      }
    } else if (orientation.equals(ChartOrientation.HORIZONTAL)) {
      int maxLeftCount = 0, maxRightCount = 0 ;
      int[] leftAxisCount = new int[plotCount] ;
      int[] rightAxisCount = new int[plotCount] ;
      for (int i = 0 ; i < plotCount ; i++) {
        CategoryPlot subPlot = (CategoryPlot)this.subPlots.get(i) ;
        leftAxisCount[i] = getAxisCountByPosition(subPlot, orientation, RectangleEdge.LEFT) ;
        rightAxisCount[i] = getAxisCountByPosition(subPlot, orientation, RectangleEdge.RIGHT) ;

        maxLeftCount = Math.max(maxLeftCount, leftAxisCount[i]) ;
        maxRightCount = Math.max(maxRightCount, rightAxisCount[i]) ;
      }

      for (int i = 0 ; i < plotCount ; i++) {
        double leftSpace = (maxLeftCount - leftAxisCount[i]) * maxSpace ;
        double rightSpace = (maxRightCount - rightAxisCount[i]) * maxSpace ;
        CategoryPlot subPlot = (CategoryPlot)this.subPlots.get(i) ;
        RectangleInsets insets = subPlot.getInsets() ;
        subPlot.setInsets(new RectangleInsets(insets.getUnitType(), insets.getTop(),
                                              leftSpace, insets.getBottom(), rightSpace)) ;
      }
    }
  }

  /**
   * <zh>categoryռ,Ȼ߶.</zh>
   * <en>Calculate the max width or height for all category axes.</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(ӡĻ) (Ϊnull).</zh>
   * <en>The graphics device (null not permitted).</en>
   * @param plotArea Rectangle2D <zh>plot (Ϊnull).</zh>
   * <en>The chart draw orientation.</en>
   *
   * @return double <zh>ĿȻ߶.</zh>
   * <en>The max width or height.</en>
   */
  protected double calculateMaxCategoryAxisSpace(Graphics2D g2, Rectangle2D plotArea) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (plotArea == null) {
      throw new IllegalArgumentException("Null 'plotArea' argument.") ;
    }

    double maxSpace = 0 ;
    Rectangle2D clonePlotArea = (Rectangle2D) plotArea.clone() ;
    int plotSize = this.subPlots.size() ;
    CategoryPlot firstPlot = (CategoryPlot)this.subPlots.get(0) ;
    ChartOrientation orientation = firstPlot.getOrientation() ;
    double plotWidth = plotArea.getWidth() ;
    double plotHeight = plotArea.getHeight() ;
    double totalGap = this.plotGap * (plotSize - 1) ;

    double totalWight = 0 ; //Ȩ֮
    for (int i = 0 ; i < plotSize ; i++) {
      Double wight = (Double)this.subPlotWight.get(i) ;
      totalWight += wight.doubleValue() ;
    }

    for (int i = 0 ; i < plotSize ; i++) {
      CategoryPlot plot = (CategoryPlot)this.subPlots.get(i) ;
      Double wight = (Double)this.subPlotWight.get(i) ;
      double plotWidthOrHeight = 0 ;
      if (orientation.equals(ChartOrientation.VERTICAL)) {
        plotWidthOrHeight = (plotHeight - totalGap) * wight.doubleValue() / totalWight ;
        clonePlotArea.setRect(clonePlotArea.getX(), clonePlotArea.getY(), clonePlotArea.getWidth(),
                              plotWidthOrHeight) ;
      } else if (orientation.equals(ChartOrientation.HORIZONTAL)) {
        plotWidthOrHeight = (plotWidth - totalGap) * wight.doubleValue() / totalWight ;
        clonePlotArea.setRect(clonePlotArea.getX(), clonePlotArea.getY(), plotWidthOrHeight,
                              clonePlotArea.getHeight()) ;
      }

      int axisCnt = plot.getCategoryAxisCount() ;
      for (int j = 0 ; j < axisCnt ; j++) {
        CategoryAxis categoryAxis = plot.getCategoryAxis(j) ;
        double axisSpace = categoryAxis.calculateAxisSpace(g2, plotArea, clonePlotArea,
                                                           plot.getCategoryAxisEdge(j)) ;
        maxSpace = Math.max(maxSpace, axisSpace) ;
      }
    }

    return maxSpace ;
  }

  /**
   * <zh>value᷶ΧݷΧ.</zh>
   * <en>Calculate the max NumberRange for all value axes.</en>
   *
   * @return NumberRange <zh>NumberRange (Ϊnull).</zh>
   * <en>The NumberRange object (never null).</en>
   */
  protected NumberRange calculateMaxValueAxisRange() {
    double minValue = Double.NaN ;
    double maxValue = Double.NaN ;

    boolean first = true ;
    for (int i = 0 ; i < this.subPlots.size() ; i++) {
      CategoryPlot plot = (CategoryPlot)this.subPlots.get(i) ;
      int axisCnt = plot.getValueAxisCount() ;
      for (int j = 0 ; j < axisCnt ; j++) {
        AbstractValueAxis valueAxis = plot.getValueAxis(j) ;
        if (valueAxis instanceof NumberValueAxis) {
          NumberRange range = ((NumberValueAxis) valueAxis).getZoomDatasetRange() ;
          if (range == null) {
            range = ((NumberValueAxis) valueAxis).getDatasetRange() ;
          }

          if (first) {
            minValue = range.getLowerBound() ;
            maxValue = range.getUpperBound() ;
            first = false ;
          } else {
            minValue = Math.min(minValue, range.getLowerBound()) ;
            maxValue = Math.max(maxValue, range.getUpperBound()) ;
          }
        }
      }
    }

    return new NumberRange(minValue, maxValue) ;
  }

  /**
   * <zh>plotַ.</zh>
   * <en>Returns a short string describing the plot type.</en>
   *
   * @return <zh>plot.</zh>
   * <en>The plot type.</en>
   */
  public PlotType getPlotType() {
    return PlotType.PLOT_TYPE_COMBINE_VALUE_AXIS_CATEGORY_PLOT ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests the plot for equality with an arbitrary object.</en>
   *
   * @param obj <zh>ҪȽϵĶ (Ϊnull).</zh>
   * <en>The object to test against (null permitted).</en>
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof CombineValueAxisCategoryPlot)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcode.</zh>
   * <en>Returns a hash code for this object.</en>
   *
   * @return <zh>hashcode.</zh>
   * <en>A hash code.</en>
   */
  public int hashCode() {
    return super.hashCode() ;
  }

  /**
   * <zh>¡.</zh>
   * <en>Returns a clone of the plot.</en>
   *
   * @return A clone.
   *
   * @throws CloneNotSupportedException  if the cloning is not supported.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

}
