package com.ijchart.xychart.render.xy ;

import java.awt.geom.GeneralPath ;
import java.awt.geom.Line2D ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.LegendType ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.WindChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.shape.GeneralPathChartShape ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>XYͼ.</zh>
 * <en>Draw xy vector shapes.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class WindRender extends AbstractXYRender implements Cloneable,
  Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 4023049878429210319L ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public WindRender(IMatrixDataset dataset) {
    super(dataset) ;

    setBaseItemDrawBorder(true) ;
    setAutoSelectBorderPaint(true) ;
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param xAxis AbstractValueAxis <zh>xֵ (Ϊnull).</zh>
   * <en>The x value axis (null not permitted).</en>
   * @param xAxisEdge RectangleEdge <zh>xλ(Ϊnull).</zh>
   * <en>The x axis edge.</en>
   * @param yAxis AbstractValueAxis <zh>yֵ (Ϊnull).</zh>
   * <en>The y value axis (null not permitted).</en>
   * @param yAxisEdge RectangleEdge <zh>yλ (Ϊnull).</zh>
   * <en>The y axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(AbstractValueAxis xAxis, RectangleEdge xAxisEdge,
                           AbstractValueAxis yAxis, RectangleEdge yAxisEdge,
                           Rectangle2D dataArea, ChartOrientation orientation,
                           RenderShapeCollection renderShapes, float alpha) {
    if (xAxis == null) {
      throw new IllegalArgumentException("Null 'xAxis' argument.") ;
    }
    if (xAxisEdge == null) {
      throw new IllegalArgumentException("Null 'xAxisEdge' argument.") ;
    }
    if (yAxis == null) {
      throw new IllegalArgumentException("Null 'yAxis' argument.") ;
    }
    if (yAxisEdge == null) {
      throw new IllegalArgumentException("Null 'yAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    if (getDataset().isEmpty()) {
      return ;
    }

    IMatrixDataset dataset = getDataset() ;
    int rows = dataset.getRowCount() ;

    this.fillPaints = getFillPaints(rows) ;
    this.borderPaints = getBorderPaints(rows) ;
    this.borderStrokes = getBorderStrokes(rows) ;

    double zeroYPos = yAxis.valueToJava2D(dataArea, yAxisEdge, 0.0, false) ;
    for (int i = 0 ; i < rows ; i++) {
      int currRowCols = dataset.getColumnCount(i) ;
      for (int j = 0 ; j < currRowCols ; j++) {
        IChartData data = dataset.getData(i, j) ;
        if (data != null && data instanceof WindChartData) {
          WindChartData windData = (WindChartData) data ;
          double date = windData.getDateValue() ;
          double direction = windData.getWindDirection() ;
          double force = windData.getWindForce() ;

          double directionRad = Math.toRadians(direction * ( -30.0) - 90.0) ;
          double ax1 = 0, ax2 = 0, ay2 = 0, rax2 = 0, ray2 = 0 ;

          ax1 = xAxis.valueToJava2D(dataArea, xAxisEdge, date, false) ;

          rax2 = date + (force * Math.cos(directionRad) * 8000000.0) ;
          ray2 = force * Math.sin(directionRad) ;

          ax2 = xAxis.valueToJava2D(dataArea, xAxisEdge, rax2, false) ;
          ay2 = yAxis.valueToJava2D(dataArea, yAxisEdge, ray2, false) ;

          int diri = windData.getWindDirectionData().intValue() ;
          int forcei = windData.getWindForceData().intValue() ;
          String dirforceText = diri + "-" + forcei ;

          GeneralPath hotspot = new GeneralPath() ;
          final float delta = 1.0F ;
          Line2D line = null ;
          if (orientation.equals(ChartOrientation.VERTICAL)) {
            line = new Line2D.Double(ax1, zeroYPos, ax2, ay2) ;

            hotspot.moveTo((float) ax2, (float) ay2) ;
            hotspot.lineTo((float) ax1, (float) zeroYPos) ;
            hotspot.lineTo((float) ax1 - delta, (float) zeroYPos) ;
            hotspot.lineTo((float) ax2 - delta, (float) ay2) ;
            hotspot.lineTo((float) ax2, (float) ay2) ;
          } else {
            line = new Line2D.Double(zeroYPos, ax1, ay2, ax2) ;

            hotspot.moveTo((float) ay2, (float) ax2) ;
            hotspot.lineTo((float) zeroYPos, (float) ax1) ;
            hotspot.lineTo((float) zeroYPos, (float) ax1 - delta) ;
            hotspot.lineTo((float) zeroYPos, (float) ax1 - delta) ;
            hotspot.lineTo((float) ay2, (float) ax2) ;
          }

          double alx2 = 0, aly2 = 0, arx2 = 0, ary2 = 0 ;
          double ralx2 = 0, raly2 = 0, rarx2 = 0, rary2 = 0 ;

          double aldir = Math.toRadians(direction * ( -30.0) - 90.0 - 5.0) ;
          ralx2 = force * Math.cos(aldir) * 8000000 * 0.8 + date ;
          raly2 = force * Math.sin(aldir) * 0.8 ;

          alx2 = xAxis.valueToJava2D(dataArea, xAxisEdge, ralx2, false) ;
          aly2 = yAxis.valueToJava2D(dataArea, yAxisEdge, raly2, false) ;

          Line2D line1 = null ;
          if (orientation.equals(ChartOrientation.VERTICAL)) {
            line1 = new Line2D.Double(alx2, aly2, ax2, ay2) ;

            hotspot.lineTo((float) alx2, (float) aly2) ;
          } else {
            line1 = new Line2D.Double(aly2, alx2, ay2, ax2) ;

            hotspot.lineTo((float) aly2, (float) alx2) ;
          }

          double ardir = Math.toRadians(direction * ( -30.0) - 90.0 + 5.0) ;
          rarx2 = force * Math.cos(ardir) * 8000000 * 0.8 + date ;
          rary2 = force * Math.sin(ardir) * 0.8 ;

          arx2 = xAxis.valueToJava2D(dataArea, xAxisEdge, rarx2, false) ;
          ary2 = yAxis.valueToJava2D(dataArea, yAxisEdge, rary2, false) ;

          Line2D line2 = null ;
          if (orientation.equals(ChartOrientation.VERTICAL)) {
            line2 = new Line2D.Double(arx2, ary2, ax2, ay2) ;

            hotspot.lineTo((float) ax2, (float) ay2) ;
            hotspot.lineTo((float) arx2, (float) ary2) ;
          } else {
            line2 = new Line2D.Double(ary2, arx2, ay2, ax2) ;

            hotspot.lineTo((float) ay2, (float) ax2) ;
            hotspot.lineTo((float) ary2, (float) arx2) ;
          }

          if (orientation.equals(ChartOrientation.VERTICAL)) {
            hotspot.lineTo((float) alx2, (float) aly2) ;
            hotspot.lineTo((float) ax2, (float) ay2) ;
          } else {
            hotspot.lineTo((float) aly2, (float) alx2) ;
            hotspot.lineTo((float) ay2, (float) ax2) ;
          }
          hotspot.closePath() ;

          //ֵ᷶Χֵ
          if (ax1 == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || zeroYPos == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || ax2 == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || ay2 == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || alx2 == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || aly2 == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || arx2 == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || ary2 == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
            renderShapes.addDataItemShape(null) ;
            continue ;
          }

          GeneralPath path = new GeneralPath() ;
          path.append(line, false) ;
          path.append(line1, false) ;
          path.append(line2, false) ;

          Rectangle2D rect = null ; //ַλ
          if (orientation.equals(ChartOrientation.VERTICAL)) {
            rect = new Rectangle2D.Double(ax1, zeroYPos, 1, 1) ;
          } else {
            rect = new Rectangle2D.Double(zeroYPos, ax1, 1, 1) ;
          }

          GeneralPathChartShape chartShape = new GeneralPathChartShape(path, hotspot, rect,
                                                                       dirforceText) ;
          chartShape.setForceInsideTextNoWrap(true) ;
          IChartShape pathShape = createItemShape(orientation, yAxis, i, j, null, fillPaints[i],
                                                  fillPaints[i], borderStrokes[i], alpha,
                                                  chartShape) ;
          renderShapes.addDataItemShape(pathShape) ;
        } else {
          renderShapes.addDataItemShape(null) ;
        }
      }
    }
  }

  /**
   * <zh>ͼ.</zh>
   * <en>Return the legend type.</en>
   *
   * @return LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  public LegendType getLegendType() {
    return LegendType.ROW ;
  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String text,
                                            Object appendParam) {
    if (appendParam == null) {
      throw new IllegalArgumentException("Null 'appendParam' argument.") ;
    }

    return (IChartShape) appendParam ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates if some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument; false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return true ;
    }
    if (!(obj instanceof WindRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    return super.hashCode() ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "WindRender" ;
  }

}
