package com.ijchart.xychart.render.xy ;

import java.awt.Shape ;
import java.awt.geom.Line2D ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;
import java.util.ArrayList ;
import java.util.Hashtable ;
import java.util.List ;

import com.ijchart.xychart.axis.value.AbstractValueAxis ;
import com.ijchart.xychart.chartinfo.RenderShapeCollection ;
import com.ijchart.xychart.constant.ChartOrientation ;
import com.ijchart.xychart.constant.LegendType ;
import com.ijchart.xychart.constant.SortType ;
import com.ijchart.xychart.data.IChartData ;
import com.ijchart.xychart.data.IXYChartData ;
import com.ijchart.xychart.dataset.IMatrixDataset ;
import com.ijchart.xychart.exception.RowXValueDuplicateException ;
import com.ijchart.xychart.shape.IChartShape ;
import com.ijchart.xychart.shape.LineAndShapeChartShape ;
import com.ijchart.xychart.ui.RectangleEdge ;
import com.ijchart.xychart.ui.supplier.IDrawingSupplier ;
import com.ijchart.xychart.util.EqualsUtil ;
import com.ijchart.xychart.util.HashcodeUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>XYͼͼ.</zh>
 * <en>A render that draws shapes for each data item, and lines between data
 * items (for use with the {@link com.ijchart.xychart.plot.XYPlot} class).</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class XYLineAndShapeRender extends AbstractXYRender implements Cloneable, Serializable {
  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = -327114058266793685L ;

  /**
   * <zh>ȱʡеǷʾ.</zh>
   * <en>A default flag indicating whether or not lines are drawn between non-null points.</en>
   */
  public static final boolean DEFAULT_BASE_ITEM_LINE_VISIBLE = true ;

  /**
   * <zh>ȱʡеͼǷʾ.</zh>
   * <en>A default flag indicating whether or not shapes are drawn for all points.</en>
   */
  public static final boolean DEFAULT_BASE_ITEM_SHAPE_VISIBLE = true ;

  /**
   * <zh>ȱʡеͼǷ.</zh>
   * <en>A default flag indicating whether or not shapes are drawn for all points.</en>
   */
  public static final boolean DEFAULT_BASE_ITEM_SHAPE_FILLED = true ;

  /**
   ** <zh>ȱʡеԶ.</zh>
   * <en>A default flag indicating whether or not items are automatically sorted.</en>
   */
  public static final boolean DEFAULT_BASE_ITEM_AUTO_SORT = true ;

  /**
   * <zh>ȱʡеǷظ.</zh>
   * <en>A default flag that control (all rows) whether or not items duplicate x-values
   *  are allowed.</en>
   */
  public static final boolean DEFAULT_BASE_ALLOW_DUPLICATE_XVALUE = true ;

  /**
   * <zh>ÿеǷʾ.</zh>
   * <en>A table of flags that control (per row) whether or not lines are visible.</en>
   */
  private Hashtable rowItemLineVisible ;

  /**
   * <zh>еǷʾ.</zh>
   * <en>A flag indicating whether or not lines are drawn between non-null points.</en>
   */
  private boolean baseItemLineVisible ;

  /**
   * <zh>ÿеͼǷʾ.</zh>
   * <en>A table of flags that control (per row) whether or not shapes are visible.</en>
   */
  private Hashtable rowItemShapeVisible ;

  /**
   * <zh>еͼǷʾ.</zh>
   * <en>A flag indicating whether or not shapes are drawn for all points.</en>
   */
  private boolean baseItemShapeVisible ;

  /**
   * <zh>ÿеͼǷ.</zh>
   * <en>A table of flags that control (per row) whether or not shapes are filled.</en>
   */
  private Hashtable rowItemShapeFilled ;

  /**
   * <zh>еͼǷ.</zh>
   * <en>A flag indicating whether or not shapes are drawn for all points.</en>
   */
  private boolean baseItemShapeFilled ;

  /**
   * <zh>ÿеǷԶ.</zh>
   * <en>A table of flags that control (per row) whether or not items
   *  are automatically sorted.</en>
   */
  private Hashtable rowItemAutoSort ;

  /**
   ** <zh>еԶ.</zh>
   * <en>A flag indicating whether or not items are automatically sorted.</en>
   */
  private boolean baseItemAutoSort ;

  /**
   * <zh>ÿеǷظ.</zh>
   * <en>A table of flags that control (per row) whether or not items
   *  duplicate x-values are allowed.</en>
   */
  private Hashtable rowItemAllowDuplicateXvalue ;

  /**
   * <zh>еǷظ.</zh>
   * <en>A flag that control (all rows) whether or not items duplicate x-values are allowed.</en>
   */
  private boolean baseItemAllowDuplicateXvalues ;

  /**
   * <zh>.</zh>
   * <en>constructor.</en>
   *
   * @param dataset IMatrixDataset <zh>ݼ (Ϊnull).</zh>
   * <en>The dataset (null not permitted).</en>
   */
  public XYLineAndShapeRender(IMatrixDataset dataset) {
    super(dataset) ;

    this.rowItemLineVisible = new Hashtable() ;
    this.baseItemLineVisible = DEFAULT_BASE_ITEM_LINE_VISIBLE ;
    this.rowItemShapeVisible = new Hashtable() ;
    this.baseItemShapeVisible = DEFAULT_BASE_ITEM_SHAPE_VISIBLE ;
    this.rowItemShapeFilled = new Hashtable() ;
    this.baseItemShapeFilled = DEFAULT_BASE_ITEM_SHAPE_FILLED ;

    this.rowItemAutoSort = new Hashtable() ;
    this.baseItemAutoSort = DEFAULT_BASE_ITEM_AUTO_SORT ;
    this.rowItemAllowDuplicateXvalue = new Hashtable() ;
    this.baseItemAllowDuplicateXvalues = DEFAULT_BASE_ALLOW_DUPLICATE_XVALUE ;

    setBaseItemDrawBorder(true) ;
    setAutoSelectBorderPaint(true) ;
  }

  /**
   * <zh>ÿеǷʾ.</zh>
   * <en>Returns the flag used to control whether or not the lines for a row
   * is visible.</en>
   *
   * @param row int <zh>һе,0ʼ.</zh>
   * <en>The row index (zero-based).</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The flag (possibly null).</en>
   */
  public boolean isRowItemLineVisible(int row) {
    Boolean bool = (Boolean)this.rowItemLineVisible.get(intToKey(row)) ;
    if (bool != null) {
      return bool.booleanValue() ;
    }

    return this.baseItemLineVisible ;
  }

  /**
   * <zh>һеǷʾ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the 'lines visible' flag for a row and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>еֵ,0ʼ.</zh>
   * <en>The row index (zero-based).</en>
   * @param flag boolean <zh>ǷɼĲֵ (Ϊnull).</zh>
   * <en>The flag (null permitted).</en>
   */
  public void setRowItemLineVisible(int row, boolean flag) {
    this.rowItemLineVisible.put(intToKey(row), Boolean.valueOf(flag)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>еǷʾĲֵ.</zh>
   * <en>Returns the base 'lines visible' attribute.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The base flag.</en>
   */
  public boolean isBaseItemLineVisible() {
    return this.baseItemLineVisible ;
  }

  /**
   * <zh>еǷʾ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the base 'lines visible' flag and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public void setBaseItemLineVisible(boolean flag) {
    if (this.baseItemLineVisible != flag) {
      this.baseItemLineVisible = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ĳеͼǷʾĲֵ.</zh>
   * <en>Returns the flag used to control whether or not the shapes for a row is visible.</en>
   *
   * @param row int <zh>ек,0ʼ.</zh>
   * <en>The row index (zero-based).</en>
   *
   * @return boolean <zh>ֵ(Ϊnull).</zh>
   * <en>A boolean (maybe null).</en>
   */
  public boolean isRowItemShapeVisible(int row) {
    Boolean bool = (Boolean)this.rowItemShapeVisible.get(intToKey(row)) ;
    if (bool != null) {
      return bool.booleanValue() ;
    }

    return this.baseItemShapeVisible ;
  }

  /**
   * <zh>һеͼǷʾ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the 'shapes visible' flag for a row and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent}.</en>
   *
   * @param row int <zh>еֵ,0ʼ.</zh>
   * <en>The row index (zero-based).</en>
   * @param flag boolean <zh>ǷɼĲֵ (Ϊnull).</zh>
   * <en>The flag (null permitted).</en>
   */
  public void setRowItemShapeVisible(int row, boolean flag) {
    this.rowItemShapeVisible.put(intToKey(row), Boolean.valueOf(flag)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>еͼǷʾĲֵ.</zh>
   * <en>Returns the base 'shape visible' attribute.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The base flag.</en>
   */
  public boolean isBaseItemShapeVisible() {
    return this.baseItemShapeVisible ;
  }

  /**
   * <zh>еͼǷʾĲֵ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the base 'shapes visible' flag and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public void setBaseItemShapeVisible(boolean flag) {
    if (this.baseItemShapeVisible != flag) {
      this.baseItemShapeVisible = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ĳеͼǷ.</zh>
   * <en>Returns the flag used to control whether or not the shapes for a row
   * is filled.</en>
   *
   * @param row boolean <zh>еֵ,0ʼ.</zh>
   * <en>The row index (zero-based).</en>
   *
   * @return boolean <zh>ֵ(Ϊnull).</zh>
   * <en>The boolean flag (maybe null).</en>
   */
  public boolean isRowItemShapeFilled(int row) {
    Boolean bool = (Boolean)this.rowItemShapeFilled.get(intToKey(row)) ;
    if (bool != null) {
      return bool.booleanValue() ;
    }

    return this.baseItemShapeFilled ;
  }

  /**
   * <zh>ĳеͼǷ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the 'shapes filled' flag for a row and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param row int <zh>еֵ,0ʼ.</zh>
   * <en>The row index (zero-based).</en>
   * @param filled boolean <zh>ֵ (Ϊnull).</zh>
   * <en>The flag (null permitted).</en>
   */
  public void setRowItemShapeFilled(int row, boolean filled) {
    this.rowItemShapeFilled.put(intToKey(row), Boolean.valueOf(filled)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>еͼǷĲֵ.</zh>
   * <en>Returns the base 'shape filled' attribute.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>The base flag.</en>
   */
  public boolean isBaseItemShapeFilled() {
    return this.baseItemShapeFilled ;
  }

  /**
   * <zh>еͼǷĲֵ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets the base 'shapes filled' flag and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>The flag.</en>
   */
  public void setBaseItemShapeFilled(boolean flag) {
    if (this.baseItemShapeFilled != flag) {
      this.baseItemShapeFilled = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ÿÿеǷԶ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>A table of flags that control (per row) whether or not items
   *  are automatically sorted,and sends a {@link com.ijchart.xychart.event.RenderChangeEvent}
   *  to all registered listeners.</en>
   *
   * @param row int <zh>еֵ,0ʼ.</zh>
   * <en>The row index (zero-based).</en>
   * @param autoSort boolean <zh>ֵ (Ϊnull).</zh>
   * <en>The flag (null permitted).</en>
   */
  public void setRowItemAutoSort(int row, boolean autoSort) {
    this.rowItemAutoSort.put(intToKey(row), Boolean.valueOf(autoSort)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ؿÿеǷԶ.</zh>
   * <en>Returns a flags that control (per row) whether or not items
   *  are automatically sorted.</en>
   *
   * @param row boolean <zh>еֵ,0ʼ.</zh>
   * <en>The row index (zero-based).</en>
   *
   * @return boolean <zh>ֵ(Ϊnull).</zh>
   * <en>The boolean flag (maybe null).</en>
   */
  public boolean isRowItemAutoSort(int row) {
    Boolean bool = (Boolean)this.rowItemAutoSort.get(intToKey(row)) ;
    if (bool != null) {
      return bool.booleanValue() ;
    }

    return this.baseItemAutoSort ;
  }

  /**
   * <zh>ؿеԶ.</zh>
   * <en>Returns a flag indicating whether or not items are automatically sorted.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isBaseItemAutoSort() {
    return this.baseItemAutoSort ;
  }

  /**
   * <zh>ÿеԶ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>A flag indicating whether or not items are automatically sorted and sends a
   * {@link com.ijchart.xychart.event.RenderChangeEvent} to all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setBaseItemAutoSort(boolean flag) {
    if (this.baseItemAutoSort != flag) {
      this.baseItemAutoSort = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>ÿÿеǷظ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets a flag that control (per row) whether or not items duplicate x-values are allowed,
   * and sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to
   *  all registered listeners.</en>
   *
   * @param row int <zh>еֵ,0ʼ.</zh>
   * <en>The row index (zero-based).</en>
   * @param allowDuplicate boolean <zh>ֵ (Ϊnull).</zh>
   * <en>The flag (null permitted).</en>
   */
  public void setRowItemAllowDuplicateXvalue(int row, boolean allowDuplicate) {
    this.rowItemAllowDuplicateXvalue.put(intToKey(row), Boolean.valueOf(allowDuplicate)) ;
    fireRenderChanged() ;
  }

  /**
   * <zh>ÿеǷظ.</zh>
   * <en>A table of flags that control (per row) whether or not items duplicate x-values
   * are allowed.</en>
   *
   * @param row boolean <zh>еֵ,0ʼ.</zh>
   * <en>The row index (zero-based).</en>
   *
   * @return boolean <zh>ֵ(Ϊnull).</zh>
   * <en>The boolean flag (maybe null).</en>
   */
  public boolean isRowItemAllowDuplicateXvalue(int row) {
    Boolean bool = (Boolean)this.rowItemAllowDuplicateXvalue.get(intToKey(row)) ;
    if (bool != null) {
      return bool.booleanValue() ;
    }

    return this.baseItemAllowDuplicateXvalues ;
  }

  /**
   * <zh>ؿеǷظ.</zh>
   * <en>Returns a flag that control (all rows) whether or not items
   *  duplicate x-values are allowed.</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public boolean isBaseItemAllowDuplicateXvalues() {
    return this.baseItemAllowDuplicateXvalues ;
  }

  /**
   * <zh>ÿеǷظ,м
   * {@link com.ijchart.xychart.event.RenderChangeEvent}¼.</zh>
   * <en>Sets a boolean flag that control (all rows) whether or not items duplicate x-values are
   *  allowed and sends a {@link com.ijchart.xychart.event.RenderChangeEvent} to
   *  all registered listeners.</en>
   *
   * @param flag boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  public void setBaseItemAllowDuplicateXvalues(boolean flag) {
    if (this.baseItemAllowDuplicateXvalues != flag) {
      this.baseItemAllowDuplicateXvalues = flag ;
      fireRenderChanged() ;
    }
  }

  /**
   * <zh>еͼ.</zh>
   * <en>Draws all data items.</en>
   *
   * @param xAxis AbstractValueAxis <zh>xֵ (Ϊnull).</zh>
   * <en>The x value axis (null not permitted).</en>
   * @param xAxisEdge RectangleEdge <zh>xλ(Ϊnull).</zh>
   * <en>The x axis edge.</en>
   * @param yAxis AbstractValueAxis <zh>yֵ (Ϊnull).</zh>
   * <en>The y value axis (null not permitted).</en>
   * @param yAxisEdge RectangleEdge <zh>yλ (Ϊnull).</zh>
   * <en>The y axis edge (null not permitted).</en>
   * @param dataArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The chart data area (null not permitted).</en>
   * @param orientation ChartOrientation <zh>ͼķ (Ϊnull).</zh>
   * <en>The chart orientation (null not permitted).</en>
   * @param renderShapes RenderShapeCollection <zh>renderͼμ (Ϊnull).</zh>
   * <en>Collects information about the render (null not permitted).</en>
   * @param alpha float <zh>alphaֵ,ڱ[0.0,1.0].</zh>
   * <en>The alpha transparency (in the range 0.0f to 1.0f, where 0.0f is fully transparent,
   * and 1.0f is fully opaque).</en>
   */
  public void drawAllItems(AbstractValueAxis xAxis, RectangleEdge xAxisEdge,
                           AbstractValueAxis yAxis, RectangleEdge yAxisEdge,
                           Rectangle2D dataArea, ChartOrientation orientation,
                           RenderShapeCollection renderShapes, float alpha) {
    if (xAxis == null) {
      throw new IllegalArgumentException("Null 'xAxis' argument.") ;
    }
    if (xAxisEdge == null) {
      throw new IllegalArgumentException("Null 'xAxisEdge' argument.") ;
    }
    if (yAxis == null) {
      throw new IllegalArgumentException("Null 'yAxis' argument.") ;
    }
    if (yAxisEdge == null) {
      throw new IllegalArgumentException("Null 'yAxisEdge' argument.") ;
    }
    if (dataArea == null) {
      throw new IllegalArgumentException("Null 'dataArea' argument.") ;
    }
    if (orientation == null) {
      throw new IllegalArgumentException("Null 'orientation' argument.") ;
    }
    if (renderShapes == null) {
      throw new IllegalArgumentException("Null 'renderShapes' argument.") ;
    }
    if (alpha < 0.0f || alpha > 1.0f) {
      throw new IllegalArgumentException("The 'alpha' value must be in the range 0.0f to 1.0f.") ;
    }

    IMatrixDataset dataset = getDataset() ;
    if (dataset.isEmpty()) {
      return ;
    }
    sortDataAndCheckDuplicateXvalues(dataset) ;
    int rows = dataset.getRowCount() ;

    this.fillPaints = getFillPaints(rows) ;
    getBorderPaints(rows) ;
    this.borderPaints = fillPaints ;
    this.borderStrokes = getBorderStrokes(rows) ;

    //ߺͼ
    double lastX = Double.NaN ; //һXֵ
    double lastY = Double.NaN ; //һYֵ
    IDrawingSupplier supplier = getDrawingSupplier() ;
    for (int i = 0 ; i < rows ; i++) {
      boolean lineVisible = isLineVisible(i) ;
      Shape shape = supplier.getNextShape() ;
      if (!isShapeVisible(i)) {
        shape = null ;
      }

      int currRowCols = dataset.getColumnCount(i) ;
      for (int j = 0 ; j < currRowCols ; j++) {
        IChartData data = dataset.getData(i, j) ;
        if (data != null && data instanceof IXYChartData) {
          double xValue = ((IXYChartData) data).getX() ;
          double x = xAxis.valueToJava2D(dataArea, xAxisEdge, xValue, false) ;
          double yValue = ((IXYChartData) data).getY() ;
          double y = yAxis.valueToJava2D(dataArea, yAxisEdge, yValue, false) ;

          if (orientation.equals(ChartOrientation.HORIZONTAL)) { //ֱ
            double temp = y ;
            y = x ;
            x = temp ;
          }

          //ֵ᷶Χֵ
          if (x == AbstractValueAxis.INVALID_COORDINATE_VALUE
              || y == AbstractValueAxis.INVALID_COORDINATE_VALUE) {
            lastX = Double.NaN ;
            lastY = Double.NaN ;
            renderShapes.addDataItemShape(null) ;
            continue ;
          }

          Line2D line = null ;
          if (j == 0) {
            line = null ;
          } else {
            if (!Double.isNaN(lastX) && !Double.isNaN(lastY) && lineVisible) {
              line = new Line2D.Double(lastX - x, lastY - y, 0, 0) ;
            }
          }

          LineAndShapeChartShape lineChartShape = new LineAndShapeChartShape(x, y, shape, line) ;
          IChartShape chartShape = createItemShape(orientation, yAxis, i, j, null,
                                                   fillPaints[i], borderPaints[i],
                                                   borderStrokes[i], alpha, lineChartShape) ;
          renderShapes.addDataItemShape(chartShape) ;

          lastX = x ;
          lastY = y ;
        } else {
          lastX = Double.NaN ;
          lastY = Double.NaN ;
          renderShapes.addDataItemShape(null) ;
        }
      }
    }

  }

  /**
   * <zh>ͼ.</zh>
   * <en>Return the legend type.</en>
   *
   * @return LegendType <zh>ͼ.</zh>
   * <en>The legend type.</en>
   */
  public LegendType getLegendType() {
    return LegendType.ROW ;
  }

  /**
   * <zh>ÿͼͼ.</zh>
   * <en>Returns the shape array of legend items.</en>
   *
   * @param legendCount int <zh>ͼĸ.</zh>
   * <en>The legend items count.</en>
   * @return IChartShape[] <zh>ͼ.</zh>
   * <en>The shapes array.</en>
   */
  protected IChartShape[] getLegendItemShapes(int legendCount) {
    if (legendCount > 0) {
      IDrawingSupplier supplier = getDrawingSupplier() ;
      IChartShape[] shapes = new IChartShape[legendCount] ;
      for (int i = 0 ; i < legendCount ; i++) {
        Shape shape = supplier.getNextShape() ;
        if (!isShapeVisible(i)) {
          shape = LineAndShapeChartShape.ZERO_RECT ;
        }

        Rectangle2D bound = shape.getBounds() ;
        Line2D line = null ;
        if (isLineVisible(i)) {
          line = new Line2D.Double(bound.getMinX() - 6, 0, bound.getMaxX() + 6, 0) ;
        }

        LineAndShapeChartShape lineChartShape = new LineAndShapeChartShape(0, 0, shape, line) ;
        lineChartShape.setDrawBorder(true) ;
        lineChartShape.setFillShape(isItemFill(i, 0)) ;
        lineChartShape.setGradientPaintTransformType(getItemFillPaintTransformer(i, 0)) ;

        if (!isAutoSelectBorderPaint()) {
          lineChartShape.setBorderPaint(getItemBorderPaint(i, 0)) ;
        }

        if (!isAutoSelectBorderStroke()) {
          lineChartShape.setBorderStroke(getItemBorderStroke(i, 0)) ;
        }

        shapes[i] = lineChartShape ;
      }

      return shapes ;
    }

    return null ;
  }

  /**
   * <zh>ֻͼ,ͼε.</zh>
   * <en>Create a shape with no attribute specified.</en>
   *
   * @param rect Rectangle2D <zh>ͼľ (Ϊnull).</zh>
   * <en>The bar rectangle (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param appendParam Object <zh>ӵĲ,ڴݶĶ (Ϊnull).</zh>
   * <en>The append parameter,used for special propose (null permitted).</en>
   *
   * @return IChartShape <zh>ͼ.</zh>
   * <en>The shape.</en>
   */
  protected IChartShape createItemShapeOnly(Rectangle2D rect, String text,
                                            Object appendParam) {
    if (appendParam == null) {
      throw new IllegalArgumentException("Null 'appendParam' argument.") ;
    }

    return (IChartShape) appendParam ;
  }

  /**
   * <zh>һintֵתΪһ.</zh>
   * <en>Convert a int value to an object.</en>
   *
   * @param value int <zh>ֵ.</zh>
   * <en>A int.</en>
   * @return Object <zh>һ (never null).</zh>
   * <en>An object.</en>
   */
  private static Object intToKey(int value) {
    return String.valueOf(value) ;
  }

  /**
   * <zh>һеǷɼ.</zh>
   * <en>Calculate whether or not the line is visible.</en>
   *
   * @param row int <zh>еֵ,0ʼ.</zh>
   * <en>The row index (zero-based).</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  private boolean isLineVisible(int row) {
    Boolean visible = (Boolean)this.rowItemLineVisible.get(intToKey(row)) ;
    if (visible != null) {
      return visible.booleanValue() ;
    } else {
      return this.baseItemLineVisible ;
    }
  }

  /**
   * <zh>һеͼǷɼ.</zh>
   * <en>Calculate whether or not the shape is visible.</en>
   *
   * @param row int <zh>еֵ,0ʼ.</zh>
   * <en>The row index (zero-based).</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  private boolean isShapeVisible(int row) {
    Boolean visible = (Boolean)this.rowItemShapeVisible.get(intToKey(row)) ;
    if (visible != null) {
      return visible.booleanValue() ;
    } else {
      return this.baseItemShapeVisible ;
    }
  }

  /**
   * <zh>һеǷԶ.</zh>
   * <en>Calculate of a row of datas is automatically sorted.</en>
   *
   * @param row int <zh>еֵ,0ʼ.</zh>
   * <en>The row index (zero-based).</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  private boolean isRowAutoSort(int row) {
    Boolean visible = (Boolean)this.rowItemAutoSort.get(intToKey(row)) ;
    if (visible != null) {
      return visible.booleanValue() ;
    } else {
      return this.baseItemAutoSort ;
    }
  }

  /**
   * <zh>һеǷԶ.</zh>
   * <en>Calculate of a row of datas are automatically sorted.</en>
   *
   * @param row int <zh>еֵ,0ʼ.</zh>
   * <en>The row index (zero-based).</en>
   *
   * @return boolean <zh>ֵ.</zh>
   * <en>A boolean.</en>
   */
  private boolean isRowAllowDuplicate(int row) {
    Boolean visible = (Boolean)this.rowItemAutoSort.get(intToKey(row)) ;
    if (visible != null) {
      return visible.booleanValue() ;
    } else {
      return this.baseItemAutoSort ;
    }
  }

  /**
   * <zh>ݽҼÿеǷظxֵ.</zh>
   * <en>Sort of datas and to check whether or not the x values are duplicate of a row.</en>
   * @param dataset IMatrixDataset <zh>ݼ.</zh>
   * <en>The dataset.</en>
   */
  private void sortDataAndCheckDuplicateXvalues(IMatrixDataset dataset) {
    if (dataset != null) {
      int rows = dataset.getRowCount() ;

      for (int i = 0 ; i < rows ; i++) {
        List datas = new ArrayList() ;
        int currRowCols = dataset.getColumnCount(i) ;
        for (int j = 0 ; j < currRowCols ; j++) {
          datas.add(dataset.getData(i, j)) ;
        }

        if (isRowAutoSort(i)) { //
          dataset.sortRowData(i, SortType.ASCENDING) ;
        }

        if (!isRowAllowDuplicate(i)) { //Ƿظ
          int datasSize = datas.size() ;
          for (int j = 1 ; j < datasSize ; j++) {
            if (EqualsUtil.equals(datas.get(j - 1), datas.get(j))) {
              String msg = "Dataset row " + i + " column " + j + " x value duplicated." ;
              throw new RowXValueDuplicateException(msg) ;
            }
          }
        }

      }
    }
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Tests this instance for equality with an arbitrary object.</en>
   *
   * @param obj  the object (null permitted).
   *
   * @return A boolean.
   */
  public boolean equals(Object obj) {
    if (obj == this) {
      return true ;
    }
    if (!(obj instanceof XYLineAndShapeRender)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    XYLineAndShapeRender that = (XYLineAndShapeRender) obj ;
    if (!EqualsUtil.equals(this.rowItemLineVisible, that.rowItemLineVisible)) {
      return false ;
    }
    if (this.baseItemLineVisible != that.baseItemLineVisible) {
      return false ;
    }
    if (!EqualsUtil.equals(this.rowItemShapeVisible, that.rowItemShapeVisible)) {
      return false ;
    }
    if (this.baseItemShapeVisible != that.baseItemShapeVisible) {
      return false ;
    }
    if (!EqualsUtil.equals(this.rowItemShapeFilled, that.rowItemShapeFilled)) {
      return false ;
    }
    if (this.baseItemShapeFilled != that.baseItemShapeFilled) {
      return false ;
    }
    if (!EqualsUtil.equals(this.rowItemAutoSort, that.rowItemAutoSort)) {
      return false ;
    }
    if (this.baseItemAutoSort != that.baseItemAutoSort) {
      return false ;
    }
    if (!EqualsUtil.equals(this.rowItemAllowDuplicateXvalue, that.rowItemAllowDuplicateXvalue)) {
      return false ;
    }
    if (this.baseItemAllowDuplicateXvalues != that.baseItemAllowDuplicateXvalues) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int result = super.hashCode() ;
    result = result * 37 + HashcodeUtil.hashCode(this.rowItemLineVisible) ;
    result = result * 37 + HashcodeUtil.hashCode(this.baseItemLineVisible) ;
    result = result * 37 + HashcodeUtil.hashCode(this.rowItemShapeVisible) ;
    result = result * 37 + HashcodeUtil.hashCode(this.baseItemShapeVisible) ;
    result = result * 37 + HashcodeUtil.hashCode(this.rowItemShapeFilled) ;
    result = result * 37 + HashcodeUtil.hashCode(this.baseItemShapeFilled) ;
    result = result * 37 + HashcodeUtil.hashCode(this.rowItemAutoSort) ;
    result = result * 37 + HashcodeUtil.hashCode(this.baseItemAutoSort) ;
    result = result * 37 + HashcodeUtil.hashCode(this.rowItemAllowDuplicateXvalue) ;
    result = result * 37 + HashcodeUtil.hashCode(this.baseItemAllowDuplicateXvalues) ;

    return result ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object.</en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "XYLineAndShapeRender" ;
  }

}
