package com.ijchart.xychart.shape ;

import java.awt.Color ;
import java.awt.Font ;
import java.awt.Graphics2D ;
import java.awt.Paint ;
import java.awt.Shape ;
import java.awt.geom.AffineTransform ;
import java.awt.geom.GeneralPath ;
import java.awt.geom.Point2D ;
import java.awt.geom.Rectangle2D ;
import java.io.Serializable ;

import com.ijchart.xychart.constant.ShapeType ;
import com.ijchart.xychart.ui.Graphics2DState ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>ƽͼ.</zh>
 * <en>The Parallelepiped chart shape.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class ParallelepipedTextChartShape extends AbstractRectangleTextChartShape implements
  Cloneable, Serializable {

  /**
   * <zh>л汾ID.</zh>
   * <en>For serialization.</en>
   */
  private static final long serialVersionUID = 4675937910955105575L ;

  /**
   * <zh>ȱʡxƫֵ.</zh>
   * <en>The default x-offset for the 3D effect. </en>
   */
  public static final double DEFAULT_X_OFFSET = 12.0 ;

  /**
   * <zh>ȱʡyƫֵ.</zh>
   * <en>The default y-offset for the 3D effect. </en>
   */
  public static final double DEFAULT_Y_OFFSET = 8.0 ;

  /**
   * <zh>xƫֵ.</zh>
   * <en>The x-offset for the 3D effect. </en>
   */
  private double xOffset ;

  /**
   * <zh>ȱʡyƫֵ.</zh>
   * <en>The default y-offset for the 3D effect. </en>
   */
  private double yOffset ;

  /**
   * <zh>drawAreaʾtext.</zh>
   * <en>Draw text in area drawArea.</en>
   *
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   */
  public ParallelepipedTextChartShape(Rectangle2D drawArea, String text) {
    this(drawArea, text, null) ;
  }

  /**
   * <zh>drawAreaʾtext.</zh>
   * <en>Draw text in area drawArea.</en>
   *
   * @param drawArea Rectangle2D <zh> (Ϊnull).</zh>
   * <en>The draw area (null not permitted).</en>
   * @param text String <zh>Ҫʾ (Ϊnull).</zh>
   * <en>The text (null permitted).</en>
   * @param font <zh> (Ϊnull),fontΪnull,
   * Ϊ{@link com.ijchart.xychart.shape.IChartShape#DEFAULT_TEXT_FONT}.</zh>
   * <en>The specified font (null permitted).If font is null,
   * {@link com.ijchart.xychart.shape.IChartShape#DEFAULT_TEXT_FONT} is used.</en>
   */
  public ParallelepipedTextChartShape(Rectangle2D drawArea, String text, Font font) {
    super(drawArea, text, font) ;
    this.xOffset = DEFAULT_X_OFFSET ;
    this.yOffset = DEFAULT_Y_OFFSET ;
  }

  /**
   * <zh>xƫֵ.</zh>
   * <en>Returns the x-offset for the 3D effect. </en>
   *
   * @return double <zh>ƫֵ.</zh>
   * <en>The x-offset. </en>
   */
  public double getXOffset() {
    return this.xOffset ;
  }

  /**
   * <zh>xƫֵ,м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the x-offset for the 3D effect,and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners. </en>
   *
   * @param offset double <zh>ƫֵ.</zh>
   * <en>The x-offset. </en>
   */
  public void setXOffset(double offset) {
    if (this.xOffset != offset) {
      this.xOffset = offset ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>yƫֵ.</zh>
   * <en>Returns the y-offset for the 3D effect. </en>
   *
   * @return double <zh>ƫֵ.</zh>
   * <en>The y-offset. </en>
   */
  public double getYOffset() {
    return this.yOffset ;
  }

  /**
   * <zh>yƫֵ,м{@link com.ijchart.xychart.event.ChartShapeChangeEvent}¼.</zh>
   * <en>Sets the y-offset for the 3D effect,and sends
   * a {@link com.ijchart.xychart.event.ChartShapeChangeEvent} to all registered listeners. </en>
   *
   * @param offset double <zh>ƫֵ.</zh>
   * <en>The y-offset. </en>
   */
  public void setYOffset(double offset) {
    if (this.yOffset != offset) {
      this.yOffset = offset ;
      fireChartShapeChanged() ;
    }
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻƴͼ</zh>
   * <en>Draw the ChartShape on a Java 2d graphics device(such as the screen or the printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>The graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  public void draw(Graphics2D g2) {
    if (g2 == null) {
      throw new IllegalArgumentException("Null 'g2' argument.") ;
    }
    if (!isVisible()) {
      return ;
    }

    Graphics2DState state = new Graphics2DState(g2) ;
    state.saveOldState() ;

    Rectangle2D cloneArea = new Rectangle2D.Double(0, 0, getDrawArea().getWidth(),
                                                   getDrawArea().getHeight()) ;
    g2.translate(getDrawArea().getX(), getDrawArea().getY()) ;
    g2.rotate(getRotate()) ;
    beforeDraw(g2) ;

    //»ͼ

    //ǰͼ
    fillShape(g2, getFillPaint(), cloneArea) ;
    drawShapeBorder(g2, cloneArea) ;

    //ͼ
    GeneralPath topShape = new GeneralPath() ;
    topShape.moveTo((float) cloneArea.getX(), (float) cloneArea.getY()) ;
    topShape.lineTo((float) (cloneArea.getX() + this.xOffset),
                    (float) (cloneArea.getY() - this.yOffset)) ;
    topShape.lineTo((float) (cloneArea.getMaxX() + this.xOffset),
                    (float) (cloneArea.getY() - this.yOffset)) ;
    topShape.lineTo((float) cloneArea.getMaxX(), (float) cloneArea.getY()) ;
    topShape.closePath() ;

    Paint fillPaint = getFillPaint() ;
    if (fillPaint instanceof Color) {
      fillPaint = ((Color) fillPaint).darker() ;
    }
    fillShape(g2, fillPaint, topShape) ;
    drawShapeBorder(g2, topShape) ;

    //Ҳͼ
    GeneralPath rightShape = new GeneralPath() ;
    rightShape.moveTo((float) cloneArea.getMaxX(), (float) cloneArea.getY()) ;
    rightShape.lineTo((float) (cloneArea.getMaxX() + this.xOffset),
                      (float) (cloneArea.getY() - this.yOffset)) ;
    rightShape.lineTo((float) (cloneArea.getMaxX() + this.xOffset),
                      (float) (cloneArea.getMaxY() - this.yOffset)) ;
    rightShape.lineTo((float) cloneArea.getMaxX(), (float) cloneArea.getMaxY()) ;
    rightShape.closePath() ;

    fillShape(g2, fillPaint, rightShape) ;
    drawShapeBorder(g2, rightShape) ;

    drawTextWithTrimMarginRect(g2, cloneArea) ;
    afterDraw(g2) ;
    g2.rotate( -getRotate()) ;
    state.restoreOldState() ;
    drawSelect(g2) ;
  }

  /**
   * <zh>ͼ豸(Ļӡ)ϻͼεѡв,Ϊͼοܻص,ԻͼʱҪȻеͼκ,
   * ٵô˷ѡв.</zh>
   * <en>Draw the selected ChartShape on a Java 2d graphics device(such as the screen or the
   * printer).</en>
   *
   * @param g2 Graphics2D <zh>ͼ豸(Ļӡ) (Ϊnull)</zh>
   * <en>the graphics device (such as the screen or the printer) (null not permitted).</en>
   */
  protected void drawSelect(Graphics2D g2) {
    if (isSelected()) {
      Rectangle2D drawArea = getDrawArea() ;
      Rectangle2D area = new Rectangle2D.Double(0, 0, drawArea.getWidth(), drawArea.getHeight()) ;
      Point2D.Double[] points = {new Point2D.Double(area.getX(), area.getY()),
                                new Point2D.Double(area.getX() + xOffset, area.getY() - yOffset),
                                new Point2D.Double(area.getMaxX() + xOffset, area.getY() - yOffset),
                                new Point2D.Double(area.getMaxX(), area.getY()),
                                new Point2D.Double(area.getX(), area.getMaxY()),
                                new Point2D.Double(area.getMaxX(), area.getMaxY()),
                                new Point2D.Double(area.getMaxX() + xOffset,
                                                   area.getMaxY() - yOffset)} ;
      drawAnchorByPoints(g2, drawArea.getX(), drawArea.getY(), points) ;
    }
  }

  /**
   * <zh>任ͼ.</zh>
   * <en>Calculate the transformed shape.</en>
   */
  protected void calculateTransformedShape() {
    if (transformedShape == null) {
      Rectangle2D drawArea = getDrawArea() ;
      AffineTransform transform = new AffineTransform() ;
      transform.translate(drawArea.getX(), drawArea.getY()) ;
      transform.rotate(getRotate()) ;

      GeneralPath path = new GeneralPath() ;
      path.moveTo(0, 0) ;
      path.lineTo((float)this.xOffset, (float) - this.yOffset) ;
      path.lineTo((float) (drawArea.getWidth() + this.xOffset), (float) - this.yOffset) ;
      path.lineTo((float) (drawArea.getWidth() + this.xOffset),
                  (float) (drawArea.getHeight() - this.yOffset)) ;
      path.lineTo((float) drawArea.getWidth(), (float) drawArea.getHeight()) ;
      path.lineTo(0, (float) drawArea.getHeight()) ;
      path.closePath() ;

      transformedShape = transform.createTransformedShape(path) ;
    }
  }

  /**
   * <zh>ȵͼ.</zh>
   * <en>Returns the hotspot shape.</en>
   *
   * @return Shape <zh>ȵͼ (Ϊnull).</zh>
   * <en>The hotspot shape (maybe null).</en>
   */
  protected Shape getHotspotShape() {
    calculateTransformedShape() ;
    return transformedShape ;
  }

  /**
   * <zh>ͼε.</zh>
   * <en>The type for describe this shape.</en>
   * @return ShapeType <zh>{@link com.ijchart.xychart.constant.ShapeType}ඨĳ
   * (Ϊnull).</zh>
   * <en>The constant value in class {@link com.ijchart.xychart.constant.ShapeType}.</en>
   */
  public ShapeType getShapeType() {
    return ShapeType.SHAPE_PARALLELEPIPED ;
  }

  /**
   * <zh>Ƿ.</zh>
   * <en>Indicates whether some other object is "equal to" this one.</en>
   *
   * @param obj <zh>ҪȽϵĶ</zh>
   * <en>the reference object with which to compare.</en>
   * @return <zh>ȷtrue,򷵻false.</zh>
   * <en>true if this object is the same as the obj argument;
   * false otherwise.</en>
   */
  public boolean equals(Object obj) {
    if (this == obj) {
      return false ;
    }
    if (!(obj instanceof ParallelepipedTextChartShape)) {
      return false ;
    }
    if (!super.equals(obj)) {
      return false ;
    }

    ParallelepipedTextChartShape that = (ParallelepipedTextChartShape) obj ;
    if (this.xOffset != that.xOffset) {
      return false ;
    }
    if (this.yOffset != that.yOffset) {
      return false ;
    }

    return true ;
  }

  /**
   * <zh>hashcodeֵ.</zh>
   * <en>Returns a hash code value for the object.</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>A hash code value for this object.</en>
   */
  public int hashCode() {
    int hashCode = super.hashCode() ;
    hashCode = 37 * hashCode + (int)this.xOffset ;
    hashCode = 37 * hashCode + (int)this.yOffset ;

    return hashCode ;
  }

  /**
   * <zh>ؿ¡Ķ.</zh>
   * <en>Creates and returns a copy of this object.</en>
   * @return object   a clone of this instance.
   * @exception  CloneNotSupportedException  if the object's class does not
   *               support the Cloneable interface. Subclasses
   *               that override the clone method can also
   *               throw this exception to indicate that an instance cannot
   *               be cloned.
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone() ;
  }

  /**
   * <zh>toString.</zh>
   * <en>Returns a string representation of the object. </en>
   *
   * @return  a string representation of the object.
   */
  public String toString() {
    return "[xOffset=" + this.xOffset
      + ",yOffset=" + this.yOffset
      + "]" ;
  }

}
