package com.ijchart.xychart.util ;

import java.awt.GradientPaint ;
import java.awt.Image ;
import java.awt.Paint ;
import java.util.Collection ;
import java.util.Iterator ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>hashcodeĹ.</zh>
 * <en>A util class for calculate the hashcode of a object.</en>
 *
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public final class HashcodeUtil {
  private HashcodeUtil() {
  }

  /**
   * <zh>һhashcodeֵ,Ϊnull,0.</zh>
   * <en>Returns a hash code for an object, or zero if the object is null.</en>
   *
   * @param object Object <zh>ҪhashcodeĶ (Ϊnull).</zh>
   * <en>The object (null permitted).</en>
   * @return <zh>hashcodeֵ,Ϊnull,0.</zh>
   * <en>The object's hash code (or zero if the object is null).</en>
   */
  public static int hashCode(Object object) {
    int result = 0 ;

    if (object != null) {
      if (object instanceof Collection) {
        Collection col = (Collection) object ;
        Iterator iterator = col.iterator() ;
        while (iterator.hasNext()) {
          Object obj = iterator.next() ;
          if (obj != null) {
            result = 29 * result + obj.hashCode() ;
          }
        }

        return result ;
      } else {
        result = object.hashCode() ;
      }
    }

    return result ;
  }

  /**
   * <zh>һhashcodeֵ,Ϊnull,0.</zh>
   * <en>Returns a hash code for an array, or zero if the object is null.</en>
   *
   * @param object Object <zh>ҪhashcodeĶ (Ϊnull).</zh>
   * <en>The object (null permitted).</en>
   * @return <zh>hashcodeֵ,Ϊnull,0.</zh>
   * <en>The object's hash code (or zero if the object is null).</en>
   */
  public static int hashCode(Object[] object) {
    int result = 0 ;

    if (object != null) {
      for (int i = 0 ; i < object.length ; i++) {
        if (object[i] != null) {
          if (object[i] instanceof Paint) {
            result = 29 * result + hashCode((Paint) object[i]) ;
          } else {
            result = 29 * result + object[i].hashCode() ;
          }
        }
      }
    }

    return result ;
  }

  /**
   * <zh>һImagehashcodeֵ.ͼƬnull,0,򷵻image.width*image.height.</zh>
   * <en>Returns a hashcode for an image,if the image is null return 0 otherwise
   * return image.width*image.height.</en>
   *
   * @param image Image <zh>ҪhashcodeImage (Ϊnull).</zh>
   * <en>The Image object.</en>
   * @return <zh>ͼƬnull,0,򷵻image.width*image.height.</zh>
   * <en>A hashcode.If the image is null return 0 otherwise return image.width*image.height.</en>
   */
  public static int hashCode(Image image) {
    return (image == null) ? 0 : (image.getWidth(null) * image.getHeight(null)) ;
  }

  /**
   * <zh>һhashcodeֵ,bΪ淵1򷵻0.</zh>
   * <en>Returns a hashcode for a boolean value,true 1 otherwise 0.</en>
   *
   * @param b boolean <zh>ҪhashcodeĲֵ (Ϊnull).</zh>
   * <en>The boolean value.</en>
   * @return <zh>bΪ淵1,򷵻0.</zh>
   * <en>A hashcode.If b is true return 1,otherwise return 0.</en>
   */
  public static int hashCode(boolean b) {
    return b ? 1 : 0 ;
  }

  /**
   * <zh>Painthashcodeֵ.</zh>
   * <en>Returns a hash code for a Paint instance.If p is null, this method returns zero.</en>
   *
   * @param paint Paint <zh>Paint (Ϊnull).</zh>
   * <en>The paint (null permitted).</en>
   *
   * @return int <zh>hashcodeֵ.</zh>
   * <en>The hash code.</en>
   */
  public static int hashCode(Paint paint) {
    if (paint == null) {
      return 0 ;
    }

    int result = 0 ;
    if (paint instanceof GradientPaint) {
      GradientPaint gp = (GradientPaint) paint ;
      result = gp.getColor1().hashCode() ;
      result = 37 * result + gp.getPoint1().hashCode() ;
      result = 37 * result + gp.getColor2().hashCode() ;
      result = 37 * result + gp.getPoint2().hashCode() ;
    } else {
      result = paint.hashCode() ;
    }

    return result ;
  }

}
