package com.ijchart.xychart.web ;

import java.util.ArrayList ;
import java.util.Collection ;
import java.util.Collections ;
import java.util.List ;

import javax.servlet.http.HttpServletRequest ;

import com.ijchart.xychart.chartinfo.ChartShapeCollection ;
import com.ijchart.xychart.client.IJChart ;
import com.ijchart.xychart.constant.ChartShapeDrawType ;
import com.ijchart.xychart.constant.ImageType ;
import com.ijchart.xychart.util.StringUtil ;

/**
 * <p>Title: IJChart</p>
 *
 * <p>Description: a chart library for the Java(tm) platform.</p>
 * <zh>webҳʾchartĹ.</zh>
 * <en>A util class for display chart in web page.</en>
 * <p>Copyright: Copyright (c) 2013</p>
 *
 * <p>Company: </p>
 *
 * @author zhang feng min,shanghai china.
 * <a href="mailto:ijchartsh@gmail.com">Contract me by email.</a>
 * @version 1.0
 */
public class WebChart {
  /**
   * <zh>chart id ǰ׺.</zh>
   * <en>The prefix of chart id.</en>
   */
  public static final String PARAM_CHART_ID = "IJCHART_ID" ;

  /**
   * <zh>ͼƬĿȲ.</zh>
   * <en>The width parameter of image.</en>
   */
  public static final String PARAM_CHART_IMAGE_WIDTH = "IMAGE_WIDTH" ;

  /**
   * <zh>ͼƬĸ߶Ȳ.</zh>
   * <en>The height parameter of image.</en>
   */
  public static final String PARAM_CHART_IMAGE_HEIGHT = "IMAGE_HEIGHT" ;

  /**
   * <zh>ͼƬͲ.</zh>
   * <en>The type parameter of image.</en>
   */
  public static final String PARAM_CHART_IMAGE_TYPE = "IMAGE_TYPE" ;

  /**
   * <zh>ȱʡͼƬĿ.</zh>
   * <en>The default image width.</en>
   */
  public static final int DEFAULT_IMAGE_WIDTH = 800 ;

  /**
   * <zh>ȱʡͼƬĸ߶.</zh>
   * <en>The default image height.</en>
   */
  public static final int DEFAULT_IMAGE_HEIGHT = 600 ;

  /**
   * <zh>ͼƬ.</zh>
   * <en>The image type.</en>
   */
  public static final ImageType DEFAULT_IMAGE_TYPE = ImageType.PNG ;

  /**
   * <zh>IJChart.</zh>
   * <en>The IJChart object.</en>
   */
  private IJChart chart ;

  /**
   * <zh>HttpServletRequest.</zh>
   * <en>The HttpServletRequest object.</en>
   */
  private HttpServletRequest request ;

  /**
   * <zh>ͼƬĿ.</zh>
   * <en>The image width.</en>
   */
  private int imageWidth ;

  /**
   * <zh>ͼƬĸ߶.</zh>
   * <en>The image height.</en>
   */
  private int imageHeight ;

  /**
   * <zh>ͼƬ.</zh>
   * <en>The image type.</en>
   */
  private ImageType imageType ;

  /**
   * <zh>DisplayChartServlet·.</zh>
   * <en>The path of DisplayChartServlet.</en>
   */
  private String displayServletPath ;

  /**
   * <zh>ǰchart id.</zh>
   * <en>The current value of chart id.</en>
   */
  private long currentChartId ;

  /**
   * <zh>ʾȵͼ.</zh>
   * <en>The types of hotspot shape.</en>
   */
  private List hotspotShapeType ;

  /**
   * <zh>IAreaFragment.</zh>
   * <en>The IAreaFragment object.</en>
   */
  private IAreaFragment areaFragment ;

  /**
   * <zh>.</zh>
   * <en>The constructor.</en>
   *
   * @param request HttpServletRequest <zh>HttpServletRequest (Ϊnull).</zh>
   * <en>The HttpServletRequest object (null not permitted).</en>
   * @param chart IJChart <zh>IJChart (Ϊnull).</zh>
   * <en>The IJChart object (null not permitted).</en>
   */
  public WebChart(HttpServletRequest request, IJChart chart) {
    this(request, chart, DEFAULT_IMAGE_WIDTH, DEFAULT_IMAGE_HEIGHT) ;
  }

  /**
   * <zh>.</zh>
   * <en>The constructor.</en>
   *
   * @param request HttpServletRequest <zh>HttpServletRequest (Ϊnull).</zh>
   * <en>The HttpServletRequest object (null not permitted).</en>
   * @param chart IJChart <zh>IJChart (Ϊnull).</zh>
   * <en>The IJChart object (null not permitted).</en>
   * @param imageWidth int <zh>ͼƬĿ.</zh>
   * <en>The image width.</en>
   * @param imageHeight int <zh>ͼƬĸ߶.</zh>
   * <en>The image height.</en>
   */
  public WebChart(HttpServletRequest request, IJChart chart, int imageWidth, int imageHeight) {
    if (request == null) {
      throw new IllegalArgumentException("Null 'request' argument.") ;
    }
    if (chart == null) {
      throw new IllegalArgumentException("Null 'chart' argument.") ;
    }
    if (imageWidth <= 0) {
      throw new IllegalArgumentException("Argument 'imageWidth' must be >0.") ;
    }
    if (imageHeight <= 0) {
      throw new IllegalArgumentException("Argument 'imageHeight' must be >0.") ;
    }

    this.chart = chart ;
    this.request = request ;
    this.imageWidth = imageWidth ;
    this.imageHeight = imageHeight ;
    this.imageType = DEFAULT_IMAGE_TYPE ;
    this.areaFragment = new DefaultAreaFragment(chart) ;

    this.hotspotShapeType = new ArrayList() ;
    this.hotspotShapeType.add(ChartShapeDrawType.RENDER_DATA_ITEM_SHAPE) ;

    this.currentChartId = ChartID.getInstance().getNextChartId() ;
    request.getSession().setAttribute(String.valueOf(this.currentChartId), this.chart) ;
  }

  /**
   * <zh>IJChart.</zh>
   * <en>Returns the IJChart object.</en>
   *
   * @return IJChart <zh>IJChart (Ϊnull).</zh>
   * <en>The IJChart object (never null).</en>
   */
  public IJChart getChart() {
    return this.chart ;
  }

  /**
   * <zh>IJChart.</zh>
   * <en>Sets the IJChart object.</en>
   *
   * @param chart IJChart <zh>IJChart (Ϊnull).</zh>
   * <en>The IJChart object (null not permitted).</en>
   */
  public void setChart(IJChart chart) {
    if (chart == null) {
      throw new IllegalArgumentException("Null 'chart' argument.") ;
    }

    this.chart = chart ;
    this.currentChartId = ChartID.getInstance().getNextChartId() ;
    request.getSession().setAttribute(String.valueOf(this.currentChartId), this.chart) ;
  }

  /**
   * <zh>HttpServletRequest.</zh>
   * <en>Returns the HttpServletRequest object.</en>
   *
   * @return HttpServletRequest <zh>HttpServletRequest (Ϊnull).</zh>
   * <en>The HttpServletRequest object (never null).</en>
   */
  public HttpServletRequest getRequest() {
    return this.request ;
  }

  /**
   * <zh>ͼƬĿ.</zh>
   * <en>Returns the image width.</en>
   *
   * @return int <zh>ͼƬĿ.</zh>
   * <en>The image width.</en>
   */
  public int getImageWidth() {
    return this.imageWidth ;
  }

  /**
   * <zh>ͼƬĿ.</zh>
   * <en>Sets the image width.</en>
   *
   * @param imageWidth int <zh>ͼƬĿ.</zh>
   * <en>The image width.</en>
   */
  public void setImageWidth(int imageWidth) {
    if (imageWidth <= 0) {
      throw new IllegalArgumentException("Argument 'imageWidth' must be >0.") ;
    }

    this.imageWidth = imageWidth ;
  }

  /**
   * <zh>ͼƬĸ߶.</zh>
   * <en>Returns the image height.</en>
   *
   * @return int <zh>ͼƬĸ߶.</zh>
   * <en>The image height.</en>
   */
  public int getImageHeight() {
    return this.imageHeight ;
  }

  /**
   * <zh>ͼƬĸ߶.</zh>
   * <en>Sets the image height.</en>
   *
   * @param imageHeight int <zh>ͼƬĸ߶.</zh>
   * <en>The image height.</en>
   */
  public void setImageHeight(int imageHeight) {
    if (imageHeight <= 0) {
      throw new IllegalArgumentException("Argument 'imageHeight' must be >0.") ;
    }

    this.imageHeight = imageHeight ;
  }

  /**
   * <zh>ͼƬ.</zh>
   * <en>Returns the image type.</en>
   *
   * @return ImageType <zh>ͼƬ.</zh>
   * <en>The image type.</en>
   */
  public ImageType getImageType() {
    return this.imageType ;
  }

  /**
   * <zh>ͼƬ.</zh>
   * <en>Sets the image type.</en>
   *
   * @param imageType ImageType <zh>ͼƬ (Ϊnull).</zh>
   * <en>The image type (null not permitted).</en>
   */
  public void setImageType(ImageType imageType) {
    if (imageType == null) {
      throw new IllegalArgumentException("Null 'imageType' argument.") ;
    }

    this.imageType = imageType ;
  }

  /**
   * <zh>DisplayChartServlet·.</zh>
   * <en>Returns the path of DisplayChartServlet.</en>
   *
   * @return String <zh>DisplayChartServlet·.</zh>
   * <en>The path of DisplayChartServlet.</en>
   */
  public String getDisplayServletPath() {
    return this.displayServletPath ;
  }

  /**
   * <zh>صǰchart id.</zh>
   * <en>Returns the current chart id.</en>
   *
   * @return long <zh>ǰchart id.</zh>
   * <en>The chart id.</en>
   */
  public long getCurrentChartId() {
    return this.currentChartId ;
  }

  /**
   * <zh>DisplayChartServlet·.</zh>
   * <en>Sets the path of DisplayChartServlet.</en>
   *
   * @param servletPath String <zh>DisplayChartServlet· (Ϊnull).</zh>
   * <en>The path of DisplayChartServlet (null permitted).</en>
   */
  public void setDisplayServletPath(String servletPath) {
    this.displayServletPath = servletPath ;
  }

  /**
   * <zh>IAreaFragment.</zh>
   * <en>Returns the IAreaFragment object.</en>
   *
   * @return IAreaFragment <zh>IAreaFragment (Ϊnull).</zh>
   * <en>The IAreaFragment object (never null).</en>
   */
  public IAreaFragment getAreaFragment() {
    return this.areaFragment ;
  }

  /**
   * <zh>IAreaFragment.</zh>
   * <en>Sets the IAreaFragment object.</en>
   *
   * @param areaFragment IAreaFragment <zh>IAreaFragment (Ϊnull).</zh>
   * <en>The IAreaFragment object (null not permitted).</en>
   */
  public void setAreaFragment(IAreaFragment areaFragment) {
    this.areaFragment = areaFragment ;
  }

  /**
   * <zh><img>ǩʾsrcֵʾ·.</zh>
   * <en>Returns the src value of the <img> tag.</en>
   *
   * @return String
   */
  public String getImgTagSrcValue() {
    String context = this.request.getContextPath() ;
    String chartPath = null ;
    if (StringUtil.isEmpty(this.displayServletPath)) {
      chartPath = context + (context.endsWith("/") ? "" : "/") + "DisplayChartServlet" ;
    } else {
      chartPath = (this.displayServletPath.startsWith(context) ? "" : context)
                  + this.displayServletPath ;
    }

    chartPath += "?" + PARAM_CHART_ID + "=" + this.currentChartId + "&"
      + PARAM_CHART_IMAGE_TYPE + "=" + this.imageType + "&" + PARAM_CHART_IMAGE_WIDTH + "="
      + this.imageWidth + "&" + PARAM_CHART_IMAGE_HEIGHT + "=" + this.imageHeight ;

    return chartPath ;
  }

  /**
   * <zh>һʾȵͼ.</zh>
   * <en>Add a type of hotspot shape.</en>
   *
   * @param drawType ChartShapeDrawType <zh>ChartShapeDrawType (Ϊnull).</zh>
   * <en>The ChartShapeDrawType object (null not permitted).</en>
   */
  public void addHotspotShapeType(ChartShapeDrawType drawType) {
    if (drawType == null) {
      throw new IllegalArgumentException("Null 'drawType' argument.") ;
    }

    this.hotspotShapeType.add(drawType) ;
  }

  /**
   * <zh>еʾȵͼ.</zh>
   * <en>Clear the types of hotspot shape.</en>
   */
  public void clearHotspotShapeType() {
    this.hotspotShapeType.clear() ;
  }

  /**
   * <zh>ʾȵͼ.</zh>
   * <en>Returns the types of hotspot shape.</en>
   *
   * @return Collection <zh>list (Ϊnull).</zh>
   * <en>The collection (never null).</en>
   */
  public Collection getHotspotShapeType() {
    return Collections.unmodifiableCollection(this.hotspotShapeType) ;
  }

  /**
   * <zh>ͼεȵArea.</zh>
   * <en>Returns the hotspot areas.</en>
   *
   * @return String <zh>ͼεȵArea (Ϊnull).</zh>
   * <en>The hotspot areas (maybe null).</en>
   */
  public String getImageAreas() {
    ChartShapeCollection chartShapes = this.chart.getChartShapes() ;
    if (chartShapes == null) { //ûдͼƬ,ȴ
      this.chart.createBufferedImage(this.imageWidth, this.imageHeight) ;
      chartShapes = this.chart.getChartShapes() ;
    }

    int count = 0 ;
    List areas = chartShapes.getHotspotAreas(this.hotspotShapeType, this.areaFragment) ;
    StringBuffer buffer = new StringBuffer() ;
    for (int i = 0 ; i < areas.size() ; i++) {
      if (areas.get(i) != null) {
        Object areaObj = areas.get(i) ;
        String tmpArea = null ;
        if (areaObj != null) {
          tmpArea = areaObj.toString() ;
        }
        if (!StringUtil.isEmpty(tmpArea)) {
          String inStr = "<area" ;
          int index = tmpArea.toLowerCase().indexOf(inStr) ;
          if (index >= 0) {
            buffer.append(tmpArea.substring(0, inStr.length()))
              .append(" shapeIndex=\"" + (count++) + "\"")
              .append(tmpArea.substring(index + inStr.length())).append("\n") ;
          }
        }
      }
    }

    return buffer.toString() ;
  }
}
